package brain.NetClient;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import mns.MNSEvent;
import mns.MNSListener;
import multitool.MultiFile;
import biocell.BioCell;
import biocell.BioThread;

/**
 * Author: Randall Borck Modified: 11-8-01 Assignment: CS442 Networking chat
 * program. (Prog. assign. #2) Description: Program implements a UDP chat client
 * on port 7000. Hard-coded server ip address is 137.229.114.140.
 */
public class RobotClient extends BioCell implements ActionListener, MNSListener {
	public RobotClient(JTextArea response, JTextField message,
			RobotClientListener listener) {
		
		super("netclient");
		this.response = response;
		this.listener = listener;
		this.message = message;
	}

	public void startClient() {
		new ChatClientThread().start();
	}

	public void process(MNSEvent mnEvent) {
		sendClientEvent((String) mnEvent.getData());
	}

	public void actionPerformed(ActionEvent event) {
		Object source = event.getSource();

		/**
		 * Execute broadcast code.
		 */
		if (source == sendButton) {
		}

		/**
		 * Execute the "WHO" request to the server.
		 */
		else if (source == whoButton) {
		}

		/**
		 * Execute server's LOGOFF request.
		 */
		else if (source == quitButton) {
			String tmpString;
			tmpString = "BYE";

			try {
				out.println(tmpString);
			} catch (Exception exception) {
				response.insert("quitButton Error: " + exception, 0);
			}
		}

		/**
		 * Execute the "WHISPER" fuction.
		 */
		else if (source == whisperButton) {
			whisperMethod();
		}

		else if (source == fileButton) {
			//fileSend();
		}

		/**
		 * Error catching
		 */
		else {
			startClient();
			showComm("Client Started");
		}
	}

	/**
	 * Gets input from user. Sends input with the "BROADCAST" server command.
	 */
	public void sendMessage() {
		String tmpString = "BROADCAST ";

		try {
			tmpString += message.getText();
			out.println(tmpString);
			message.setText("");
			message.requestFocus();
		}

		catch (Exception exception) {
			showComm("SendMessage Error: " + exception);

		}

	}

	/**
	 * Gets input from user. Sends input with the "INFORM" server command.
	 * This tag is the communication protocol for common events
	 * of any robotclient
	 */
	public void sendClientInform(String message) {
		String tmpString = "INFORM ";

		try {
			//tmpString += message;
			out.println(message);
			showComm("Sending event is over: " + tmpString+" "+message);
		}

		catch (Exception exception) {
			showComm("SendMessage Error: " + exception);
		}

	}

	/**
	 * Gets input from user. Sends input with the "BROADCAST" server command.
	 */
	public void sendClientEvent(String message) {
		String tmpString = "BROADCAST ";

		try {
			tmpString += message;
			out.println(tmpString);
			showComm("Sending event is over: " + tmpString+" "+message);
		}

		catch (Exception exception) {
			showComm("SendMessage Error: " + exception);
		}

	}

	protected void showComm(Object message) {
		if (response != null)
			response.insert(message + "\n", 0);
	}

	/**
	 * Whispers the input to the designated user.
	 */
	public void whisperMethod() {
		String tmpString;

		try {
			tmpString = "WHISPER "
					+ JOptionPane
							.showInputDialog("Enter the recipient username: ");
			tmpString += " " + message.getText();

			out.println(tmpString);
			message.setText("");
			message.requestFocus();
		}

		catch (Exception exception) {
			showComm("WhisperMethod Error: " + exception);

		}
	}

	public void fileSend(String filename, String recipient) {
		String tmpString;
		try {
			tmpString = new String("FILE " + recipient + " " + filename);

			//open the file stream
			FileReader reader = new FileReader(filename);
			BufferedReader fin = new BufferedReader(reader);

			//Send the FILE send information to the server
			out.println(tmpString);
			showComm("File command sent: " + tmpString);
			//Wait for "GO"
			do {
				tmpString = /* new String( */in.readLine();
			} while (tmpString.equals(""));

			if (tmpString.startsWith("GO")) {
				while (true) {
					try {
						tmpString = new String(fin.readLine());
						tmpString = new String("SEND " + tmpString);
						out.println(tmpString);
					} catch (Exception e) {
						tmpString = new String("STOP");
						out.println(tmpString);
						out.flush();
						showComm("End of Script Reached <:> Transaction completed");
						break;
					}
				}
			} else {
				showComm("Never received  GO. Aborting.");
			}
		} catch (Exception e) {
			showComm(e);
		}
	}

	/**
	 * Implements the client side of the "WHO" command.
	 */
	public void whoInquire() {
		String tmpString;

		try {
			tmpString = "WHO";
			out.println(tmpString);
			message.requestFocus();
		}

		catch (Exception exception) {
			showComm("whoInquire Error: " + exception);

		}

	}

	/**
	 * Creates a new thread to send/receive data from the server.
	 */
	private class ChatClientThread extends BioThread {
		/**
		 * Constructs datagram socket.
		 */
		public ChatClientThread() {
			super("robotclient");

			try {

				String address = JOptionPane
						.showInputDialog("Enter the Serve name or IP: ");
				showComm(address);

				int localPort = 0;
				InetAddress host = InetAddress.getByName(address);
				int toPort = 7000;
				conn = new Socket(host, toPort);

				in = new BufferedReader(new InputStreamReader(conn
						.getInputStream()));
				out = new PrintWriter(conn.getOutputStream(), true);

				showComm("server: " + host);

			}

			catch (Exception e) {
				showComm("Constructor Error: " + e);

			}

		}

		public void run() {
			try {
				tmpString = "REGISTER "
						+ JOptionPane
								.showInputDialog("Enter your username for the chat: ");

				showComm("Sending:  " + tmpString);

				out.println(tmpString);

				/**
				 * Loop that continually receive data.
				 */
				/***************************************************************
				 * Incoming \
				 **************************************************************/
				while (true) {
					tmpString = new String(in.readLine());
					tmpString = tmpString.trim();

					showComm("Recieved packet: " + tmpString);

					/**
					 * Server logoff signal.
					 */
					if (tmpString.toUpperCase().startsWith("GOOD-BYE!")) {
						conn.close();
						System.exit(0);
					}

					/***********************************************************
					 * *************************************************************************\
					 * Catch an incoming FILE transfer request. Incoming Syntax:
					 * FILE <fromUser><filenam>Example communication from
					 * receiver's perspective: server -> client: FILE
					 * puppetmaster robotFile.txt client -> server: GO server ->
					 * client: SEND ... server -> client: SEND ... server ->
					 * client: STOP \
					 **********************************************************/
					else if (tmpString.toUpperCase().startsWith("FILE")) {
						showComm("Incoming file request received");

						showComm(tmpString);

						String msg = tmpString.substring(5); //Gets rid of the
						// FILE part
						msg = new String(msg.trim());

						msg = new String(msg.concat(" "));
						String fromUser = new String(msg.substring(0, msg
								.indexOf(" ")));
						fromUser = new String(fromUser.trim());
						msg = new String(msg.substring(msg.indexOf(" ")));
						msg = new String(msg.trim());

						msg = new String(msg.concat(" "));
						String filename = new String(msg.substring(0, msg
								.indexOf(" ")));
						msg = new String(msg.substring(msg.indexOf(" ")));

						showComm("Request from = " + fromUser
								+ " for file transfer: " + filename);

						MultiFile f = new MultiFile(filename + ".rbt");
						if (!f.exists())
							f.createNewFile();
						FileOutputStream fout = new FileOutputStream(f);
						tmpString = new String("GO");
						if (files == null)
							files = new Vector(1, 1);
						out.println(tmpString + " " + fromUser);
						out.flush();
						while (true) {
							String line = new String(in.readLine());
							if (line.startsWith("STOP")) {
								showComm("Done writing file " + f.getName()
										+ ". Closing file stream.");
								fout.flush();
								fout.close();
								files.add(f);
								break;
							} else {
								fout.write(line.getBytes(), 5,
										line.length() - 5); //Cut off the <SEND > part of the stream, write it to the file
								fout.write('\n');
								fout.flush();
							}
						}
					} else if (tmpString.toUpperCase().endsWith("START")) {
						showComm("Starting");
						if (files != null && files.size() > 0)
							listener.clientCommand(new RobotClientEvent(
									(MultiFile) files.remove(0), "File"));
						listener.clientCommand(new RobotClientEvent("start"));
					} else if (tmpString.toUpperCase().endsWith("STOP")) {
						showComm("Stoping");
						listener.clientCommand(new RobotClientEvent("Stop"));
					} else if (tmpString.toUpperCase().startsWith("START")) {
						showComm("Starting");
						listener.clientCommand(new RobotClientEvent(
								(MultiFile) files.remove(0), "File"));
						listener.clientCommand(new RobotClientEvent("start"));
					} else if (tmpString.toUpperCase().startsWith("STOP")) {
						showComm("Stoping");
						listener.clientCommand(new RobotClientEvent("Stop"));
					} else if (tmpString.toUpperCase().startsWith("INFORM")) {
						showComm(tmpString);
						listener.clientCommand(new RobotClientEvent(tmpString));
					} else if (tmpString.toUpperCase().startsWith("BRODCAST")) {
						listener.clientCommand(new RobotClientEvent(tmpString));
						showComm("RECIEVED: " + tmpString);
						System.out.println("RECIEVED: " + tmpString);
					}
					/**
					 Output the data to the client in the "response" window.
					 */
					else {
						showComm(tmpString.trim());
					}
				}
			}
			catch (Exception e) {
				showComm("Error: " + e + " ReceiveThread");
				e.printStackTrace();
			}
		}

		String tmpString;
	}

	private Socket conn;

	private PrintWriter out;

	private BufferedReader in;

	private JTextField message;

	private JTextArea response;

	private JButton sendButton;

	private JButton whoButton;

	private JButton quitButton;

	private JButton fileButton;

	private JButton whisperButton;

	private RobotClientListener listener;

	private Vector files;
}