package brain.hedonic;

/*
 
 
 Class Energy
 the bottom class of the agent. 
 the low level information processing unit. USed for processing 
 energy and information parsing concerning the internal state. 
 Perceptron is connected to the environment and sends information
 to the agent deeper level of prediction. At the end of one cycle
 the agent executes commands via another neural network
 
 -multi-qualitative objects perception
 -creating of complex representation
 -predicting
 
 2002 by Martin Lukac
 
 */



//class taking care of all quasi biological processes
//all signal are also passing through it

public class Energy extends Thread{
	//Current energy
	public double nrg;

	//optimal energy 
	public double optnrg = 1;
	
	//the step energy consumption coefficient
	private double nrgConsCoeff = 0.0001;

	//the length of the radius memory
	private int rlength = 13;

	//radius of my evaluation function
	private double[] radius;

	public Energy() {

		//setting initial energy
		nrg = optnrg;

		//generate the sigam of random size in [-1,1]
		radius = new double[rlength];
		for (int a = 0; a < rlength; a++)
			radius[a] = (Math.random() * 2) - 1;
	}

	
	public Energy(double energy) {
		//setting optimal energy
		optnrg = energy;

		//setting initial energy
		nrg = optnrg;

		//generate the sigam of random size in [-1,1]
		radius = new double[rlength];
		for (int a = 0; a < rlength; a++)
			radius[a] = (Math.random() * 2) - 1;
	}

	/**
	 * 
	 */
	public void Act() {
		//consuming energy for perception
		Econs();
		if (nrg <= 0) {
			System.exit(1);
		}
	}

	/**
	 * consumption of the energy for each action or time period
	 */
	private void Econs() {
		// subtracts a random amount of energy
		nrg -= optnrg * nrgConsCoeff * Math.random();
	}

	/**
	 * energetic income of the system
	 * @return
	 */
	public double food(double nrg) {
		//determine how much energy i am far from the optimal amount
		double result = optnrg / (10 * nrg);
		return result;
	}
	
	//evaluating current energetic state
	//and making some basical predictions based on AR
	public double unitEval() {
		double result = 0;
		double[] predict = new double[rlength];

		//making prediction about energy

		predict[0] = (nrg - Math.random());
			for (int a = 1; a < rlength; a++) {
				predict[a] = optnrg * Math.random()
						+ (predict[a - 1] - (a + 1) * Math.random());
			}

		for (int a = 0; a < predict.length; a++) {
			//setting the real value for the amount of food [0..1]
			//by Gaussian curve
			predict[a] = NRGfunct(predict[a]);
			result *= predict[a];
		}

		//	System.out.println(result +" energy ");
		return result;
	}


	/**
	 * evaluating current energetic state and making some basical predictions based on AR
	 * @return
	 */
	public double Eval(double [] signal) {
		double result = 0;
		double[] predict = new double[signal.length];

		//making energy prediction
		predict[0] = 5 * signal[0] + (nrg - Math.random());
		if (signal.length > 1) {
			for (int a = 1; a < signal.length; a++) {
				predict[a] = 5 * signal[a]
						+ (predict[a - 1] - (a + 1) * Math.random());
			}
		}

		//setting the real value for the amount of food [0..1] by Gaussian curve
		for (int a = 0; a < predict.length; a++) {
			predict[a] = nrgfunct(predict[a]);
			result *= predict[a];
		}
		return result;
	}

	/**
	 * evaluating current energetic state and making some basical predictions based on AR
	 * @return
	 */
	public long EvalTime(long period) {
		long result = 0;
		double predict = (double)period;
		//making energy prediction
		predict = 5 * period + (nrg - Math.random());
		//setting the real value for the amount of food [0..1] by Gaussian curve
		result = period * (long)nrgfunct(predict);
		return result;
	}

	/**
	 * function evaluating the current energy neccessity compared to the optimal value
	 * @param val
	 * @return
	 */
	public double nrgfunct(double val) {
		double radius = Math.pow((optnrg - nrg), 2);
		//testing this method
		radius = shiftVal(radius, this.radius);
		double inter = Math.exp(-Math.pow((val - optnrg), 2)
				/ (2 * Math.pow(radius, 2)));
		return inter;
	}

	/**
	 * @param a
	 * @param arr
	 * @return
	 */
	private double shiftVal(double a, double[] arr) {
		double dump = 0;
		//left shifting
		for (int b = 1; b < arr.length; b++) {
			arr[b - 1] = arr[b];
			dump += arr[b - 1];
		}
		arr[arr.length - 1] = a;
		dump += arr[arr.length - 1];
		dump = dump / arr.length;
		return dump;
	}
	
	// function evaluating the current energy neccessity compared to the optimal value
	public double NRGfunct(double val) {
		double radius = Math.pow((this.optnrg - nrg), 2);
		//testing this method
		radius = shiftVal(radius, this.radius);
		double inter = Math.exp(-Math.pow((val - optnrg), 2)
				/ (2 * Math.pow(radius, 2)));

		//double result = (1/(radius*Math.sqrt(2*Math.PI)));
		//result *= inter;

		return inter;
	}

}

