package brain.hedonic;


import java.util.Vector;

/**
 * @author nabhoth
 * 
 * This class represents the control module of the energy based model. This
 * class should not be called by user classes but the
 * @BioCell and the
 * @BioEnergySpender interface should beused. This is the decision making
 *                   module. It is not a brain but rather a unitary cell of self
 *                   controlability and adaptation. It uses
 * @Energy wich should be connected in loop .
 */
public class Strategy extends Thread{

	private Energy E;

	private double[] lastAction;

	private double lastAct = 1;

	private double SCenter = 1;

	private double lambda = 0.3;

	private double Radius = 1;

	private Vector actions = new Vector();

	private int memSize = 10000;

	private double[] radius = new double[memSize];

	private double[] strategy = new double[memSize];

	//strategy between Inf and 1
	private double Strategy  = 0.5;

	//the last strategy
	private double lastStra = 4;

	//sample constructor using default values for the function radius and
	Strategy() {
		E = new Energy();
		InitAc(strategy, Strategy);
		InitAc(radius, Radius);
	}

	public void step() {
		//input from lower layer ... energy
		E.Act();
		double inStrgy = stVal();
		//predicting actions based on current input and current internal
		//Strategy predictions
		makeActions(inStrgy);
		//	System.out.println("Updated by "+sign);
		updateStrategy(stFunction(inStrgy * lastAct));
	}

	/**
	 * @param valuex
	 *            specific method for updating the strategy
	 */
	public void updateStrategy(double value) {
		lastStra = Strategy;
		int range = Window();
		//update strategy for this state even if direction will not be taken
		Strategy = updateAction(value, strategy, range);
		Radius = Math.sqrt(Math.pow((value - Strategy), 2));
		Radius = updateAction(Radius, radius, range);
	}


	/**
	 * @return
	 */
	public double stVal() {
		//stval corresponds to next possible energetics value from percieved
		//and analyzed data
		double stval = E.unitEval();
		//calculate the strategy for this direction;
		return stFunction(stval);
	}

	//method that is used as interface for updating different actions
	private void updateAction(double[] action) {
		double sign;
		if ((Strategy) > lastStra){
			//TODO Feedback
			sign = Math.random();
		}
		else
			sign = 0;
		//		if (action[0] == Move) {
		//			Move = updateAction(sign, move, Window());
		//			lastAct = Move;
		//		} else if (action[0] == Turn) {
		//			Turn = updateAction(sign, turn, Window());
		//			lastAct = Turn;
		//		} else if (action[0] == Eat) {
		//			//double val = E.food();
		//			// Eat = updateAction(val, eat, Window());
		//			lastAct = Eat;
		//		}
	}

	// execute Actions
	// input the curent incoming strategy coupled with all previous under-levels
	private void makeActions(double ext) {
		//dummy variables
		double[] inter = new double[3];
		double[] order = new double[3];
		//memory of last action
		lastAction = new double[2];
		//		inter[0] = Math.sqrt(Math.pow((Eat * ext - Strategy), 2));
		//		order[0] = Eat;
		//		//System.out.println(" Eat ="+inter[0]+" "+Eat);
		//
		//		inter[1] = Math.sqrt(Math.pow((Move * ext - Strategy), 2));
		//		order[1] = Move;
		//		//System.out.println(" Move ="+inter[1]+" "+Move);
		//
		//		inter[2] = Math.sqrt(Math.pow((Turn * ext - Strategy), 2));
		//		order[2] = Turn;
		//System.out.println(" Turn ="+inter[2]+" "+Turn);
		//setting order in action by their pertinence
		//evaluated by the distance from the Strategy of the current state
		for (int count = 0; count < inter.length; count++)
			for (int a = count; a < inter.length; a++)
				if (stFunction(inter[count]) < stFunction(inter[a])) {
					double interr = inter[a];
					double inerra = order[a];
					inter[a] = inter[count];
					order[a] = order[count];
					inter[count] = interr;
					order[count] = inerra;
				}
		//making the selected actions
		//if first best action is not done the following will be
		boolean done = false;
		//updating actions
		if (lastAction[1] != 0)
			updateAction(lastAction);
	}

	/**
	 * @param val
	 * @return
	 */
	private double stFunction(double val) {
		double result = Math.sqrt((Radius * Radius)
				+ Math.pow((val - SCenter), 2))
				/ Radius;
		return result;
	}

	private double ZeroToInf(double al, double fixe) {
		return Math.sqrt(Math.pow((al - fixe), 2));
	}

	/**
	 * @return
	 */
	private int Window() {
		//lambda
		//	lambda = 1;
		lambda = 1 / Strategy * lambda;
		//history
		int range = (int) (strategy.length - (strategy.length / Strategy));
		if (range <= 0)
			range = 1;
		return range;
	}

	//two methods for updating actions values
	//one within the whole evaluation interval the second within the
	//window defined by the Strategy-- cf. Lambda
	private double updateAction(double action, double[] Act, int depth) {
		double result = 0;
		for (int a = 1; a < Act.length; a++) {
			Act[a - 1] = Act[a];
			result += Act[a - 1];
		}
		Act[Act.length - 1] = action;
		for (int a = depth; a < Act.length; a++)
			result += Act[a] * Math.pow(lambda, (Act.length - a));
		result /= (Act.length - depth + 1);
		return result;
	}

	/**
	 * @param action
	 * @param Act
	 * @return
	 */
	private double updateAction(double action, double[] Act) {
		double result = 0;
		for (int a = 1; a < Act.length; a++) {
			Act[a - 1] = Act[a];
			result += Act[a - 1];
		}
		Act[Act.length - 1] = action;
		for (int a = 0; a < Act.length; a++)
			result += Act[a] * Math.pow(lambda, (Act.length - a));
		result /= (Act.length - Act.length + 1);
		return result;
	}

	/**
	 * @param Act
	 * @param base
	 * @return
	 */
	private double InitAc(double[] Act, double base) {
		double result = 0;
		for (int a = 0; a < Act.length; a++) {
			Act[a] = base * Math.random();
			result += Act[a];
		}
		result /= Act.length;
		return result;
	}

}