package qcnn;

/*
 * Created on Dec 21, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

/**
 * @author nabhoth
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

/*************************************************************************
 *  Compilation:  javac FFT.java
 *  Execution:    java FFT N
 *  Dependencies: complex.java
 *
 *  Compute the FFT and inverse FFT of a length N complex sequence.
 *  Bare bones implementation that runs in O(N log N) time.
 *
 *  Limitations
 *  -----------
 *   * assumes N is a power of 2
 *   * not the most memory efficient algorithm
 *  
 *************************************************************************/

/**
 * @author nabhoth
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
/**
 * @author nabhoth
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class FastFT {

	

	// compute the FFT of x[], assuming its length is a power of 2
    public static complex[] fft(complex[] x) {
        int N = x.length;
        complex [] y = new complex[N];
        //complex[] y = new complex[N];

        // base case
        if (N == 1) {
            y[0] = x[0];
            return y;
        }

        // radix 2 Cooley-Tukey FFT
        if (N % 2 != 0) throw new RuntimeException("N is not a power of 2");
        complex[] even = new complex[N/2];
        complex[] odd  = new complex[N/2];
        for (int k = 0; k < N/2; k++) even[k] = x[k << 1];//2*k];
        for (int k = 0; k < N/2; k++) odd[k]  = x[(k << 1) +1]; //2*k + 1];

        complex[] q = fft(even);
        complex[] r = fft(odd);

        for (int k = 0; k < N/2; k++) {
            double kth = -2 * k * Math.PI / N;
            complex wk = new complex(Math.cos(kth), Math.sin(kth));
            y[k]       = q[k].plus(wk.times(r[k]));
            y[k + N/2] = q[k].minus(wk.times(r[k]));
        }
        return y;
    }


    // compute the inverse FFT of x[], assuming its length is a power of 2
    public static complex[] ifft(complex[] x) {
        int N = x.length;

        // take conjugate
        for (int i = 0; i < N; i++)
            x[i] = x[i].conjugate();

        // compute forward FFT
        complex[] y = fft(x);

        // take conjugate again
        for (int i = 0; i < N; i++)
            y[i] = y[i].conjugate();

        // divide by N
        for (int i = 0; i < N; i++)
            y[i] = y[i].times(1.0 / N);

        return y;

    }

    
    
    public static int getMeasured(double max, complex [] state){
		double result = 0.0;
		double threshold = max*(Math.random());
		for (int a = 0; a < state.length; a++){
			result += state[a].modulusSquared();
			if (result >= threshold) 
				return a;
		}
		return 1;
	}

    // compute the convolution of x and y
    public static complex[] convolve(complex[] x, complex[] y) {
        if (x.length != y.length) throw new RuntimeException("Dimensions don't agree");
        int N = x.length;

        // compute FFT of each sequence
        complex[] a = fft(x);
        complex[] b = fft(y);

        // point-wise multiply
        complex[] c = new complex[N];
        for (int i = 0; i < N; i++)
            c[i] = a[i].times(b[i]);

        // compute inverse FFT
        return ifft(c);
    }


    public static complex[] normalize(complex [] x){
    	complex [] y = new complex[x.length];
		double result = 0.0;
		for (int a = 0; a < x.length; a++)
			result += x[a].modulusSquared();

		for (int a = 0; a < x.length; a++)
			y[a] = x[a].times(new complex(Math.sqrt(result),0));
		return y;
    }

    // test client
    public static void main(String[] args) { 
        int N = Integer.parseInt(args[0]);
        complex[] x = new complex[N];

        // original data
        for (int i = 0; i < N; i++) {
            x[i] = new complex(i-N/2, 0);
        }
        for (int i = 0; i < N; i++)
            System.out.println(x[i]);
        System.out.println();

        // FFT of original data
        complex[] y = fft(x);
        double max = 0.0;
        for (int i = 0; i < N; i++){
            System.out.println(i+":   "+y[i]);
            max += y[i].modulusSquared();
        }
        System.out.println("-- "+max);

		int rankh [] = new int[N];
		for (int a = 0; a < 50; a++){
			int m = getMeasured(max, y);
			if (m < rankh.length)
				rankh[m]++;
		}
		
        System.out.println("//RANKING ");
		 for (int i = 0; i < N; i++)
          System.out.print(rankh[i]+" ");
         System.out.println('\n'+" //RANKING ");
//        // take inverse FFT
//        complex[] z = ifft(y);
//        for (int i = 0; i < N; i++)
//            System.out.println(z[i]);
//        System.out.println();
//
//        // convolution of x with itself
//        complex[] c = convolve(x, x);
//        for (int i = 0; i < N; i++)
//            System.out.println(c[i]);

    }

}

