package qcnn;

/*
 * Created on Oct 24, 2004
 *
 */

/**
 * @author nabhoth
 * 
 * FFT - fast fourier transform requires two arrays of 2^m poins - real and
 * imaginary
 */
public class Fourier {

	Complex[] cx;

	int length;

	//direction 1 -> forward transform
	//direction -1 -> reverse transform
	int direction = 1;

	public Fourier() {

	}

	public void set(Complex[] cx) {
		this.cx = cx;
	}

	public void set(double[] data) {
		cx = new Complex[data.length];
		for (int a = 0; a < cx.length; a++)
			cx[a] = new Complex(data[a], 0);
	}

//	public void setlength(int length) {
//		this.length = length;
//	}

	public void show() {
		double result = 0.0;
		for (int a = 0; a < cx.length; a++)
			result += cx[a].modulusSquared();
		
		System.out.println();
		System.out.println("!!!! " + result + " !!!");
		System.out.println();
	}

	public int getMeasured(double threshold){
		double result = 0.0;
		for (int a = 0; a < cx.length; a++){
			result += cx[a].modulusSquared();
			if (result >= threshold) 
				return a;
		}
		return cx.length;
	}
	
	public void forwardFFT() {

		int m = cx.length;
		long n = 1;
		for (int i = 0; i < m; i++)
			n *= 2;

		//Bit reversal
		long i2 = n >> 1;
		int j = 0;
		
//		for (int i = j = 0; i < n; ++i) 
//		{	if (j >= i) 
//			{	double tempr = cx[j].re;
//				double tempi = cx[j].im;
//				cx[j].re = cx[i].re;
//				cx[j].im = cx[i].im;
//				cx[i].re = tempr;
//				cx[i].im = tempi;
//			}
//			int mm = m/2;
//			while ((mm >= 1) && (j >= mm)) 
//			{	j -= mm;
//				mm /= 2;
//			}
//			j += mm;
//		}

		
		for (int i = 0; i < n - 1; i++) {
			if (i < j) {
				Complex t = cx[i];
				cx[i] = cx[j];
				cx[j] = t;
			}
			long k = i2;
			while (k <= j) {
				j -= k;
				k >>= 1;
			}
			j += k;
		}

		//compute the FFT
		double c1 = -1;
		double c2 = 0;
		long l2 = 1;
		for (int l = 0; l < m; l++) {
			long l1 = l2;
			l2 <<= 1;
			double u1 = 1;
			double u2 = 0;
			for (j = 0; j < l1; j++) {
				for (int i = j; i < n; i += l2) {
					long i1 = i + l1;
					double t1 = u1 * cx[(int) i1].re - u2 * cx[(int) i1].im;
					double t2 = u1 * cx[(int) i1].im + u2 * cx[(int) i1].re;
					cx[(int) i1].re = cx[i].re - t1;
					cx[(int) i1].im = cx[i].im - t2;
					cx[i].re += t1;
					cx[i].im += t2;
				}
				double z = u1 * c1 - u2 * c2;
				u2 = u1 * c2 + u2 * c1;
				u1 = z;
			}
			c2 = Math.sqrt((1.0 - c1) / 2);
			if (direction == 1)
				c2 = -c2;
			c1 = Math.sqrt((1.0 + c1) / 2);
		}

		//Scaling for forward transform
		if (direction == 1) {
			for (int i = 0; i < n; i++) {
				cx[i].re /= Math.sqrt(n);
				cx[i].im /= Math.sqrt(n);
			}
		}
	}
}