package qcnn;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Vector;

/*
 * Created on Oct 4, 2004
 * 
 * 
 * First template of a quantum-deual level neural network. Type is perceptron
 * like, feedback made through back propagation, the connectivity is full, fixed
 * The energy is calculated directly from the input, output, current activation
 * level of the cell Further precision will be aded durring implementation
 */

/**
 * @author nabhoth
 * 
 */
public class QCNN {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new QCNN(10, 16, 16);
	}
	/**
	 *  
	 */
	public QCNN() {

	}

	/**
	 * @param input
	 * @param output
	 * @param hidden
	 */
	public QCNN(int input, int output, int hidden) {
		pC = new QPecep(input, output, hidden);
		dG = new DataGenerator(input, output);
		eH = new ErrorHandler(output);
		try {
			dG.genData(input, output);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		test();
	}

	/**
	 * @param input
	 * @param output
	 * @param hidden
	 * @param hidden_layers
	 */
	public QCNN(int input, int output, int hidden, int hidden_layers) {
		pC = new QPecep(input, output, hidden);
		dG = new DataGenerator();
		eH = new ErrorHandler(output);
		try {
			dG.genData(input, output);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		activate();
	}

	public void test(){
		try {
			RandomAccessFile raf = dG.getDataStream();
			int lengt = (int)raf.readDouble();
			int in = (int)raf.readDouble();
			int out = (int)raf.readDouble();
			double [] input = new double[in];
			double [] error = new double[out];
			eH.init();
			while (!eH.done()){
				for (int a = 0; a < in; a++){
					try{
						input[a] = raf.readDouble();						
					}catch (Exception e){
						System.out.println("  Start of the file  ");
						raf.seek(8*3);
						input[a] = raf.readDouble();
					}
				}
				
				error = pC.In(input);
				for (int a = 0; a < out; a++){
					error[a] = (raf.readDouble() - error[a]);
				}
				pC.learn(error);
				eH.setError(error);
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * activate the network - creates cells, connects them and initialize energy
	 * matrices
	 */
	private void activate() {
		for (int a = 0;a < input.size();a++)
			input.add(new QCell(hidden.size()));
		for (int a = 0;a < output.size();a++)
			output.add(new QCell(1));
		for (int a = 0;a < hidden.size();a++)
			hidden.add(new QCell(output.size()));
	}

	private double lbf(){
		
		return 0;
	}
	
	private double rbf(){
		
		return 0;
	}
	
	private Vector input, output, hidden;
	private QPecep pC;
	private DataGenerator dG;
	private ErrorHandler eH;
}


class ErrorHandler {
	
	public ErrorHandler(int size){
		errorInput = new double[size];
		init();
	}

	public void init(){
		localError = 0;
		statError = 1;
		totalSteps = 0;
		historyPointer = 0;
		history = new double[historyLength];
	}
	
	public void setError(double [] errorInput){
		
		localError = 0;
		
		for (int a = 0;a < errorInput.length; a++){
			this.errorInput[a] = Math.pow(errorInput[a],2);
			localError += this.errorInput[a];
		}
		localError /= errorInput.length;
		
		if (historyPointer <  historyLength)
			history[historyPointer] = localError;
		else
		{
			shiftHistory();
			history[historyPointer] = localError;
		}
		
		statError = 0;
		for (int a = 0;a < historyPointer; a++){
			statError += history[a];
		}
		
		statError /= historyPointer;
		totalSteps++;
		historyPointer++;
		
		System.out.println("Steps: "+totalSteps +" Current error: "+localError+" Overall Error "+statError);
	}
	
	
	private void shiftHistory(){
		for (int a = historyLength/2; a < historyLength; a++){
			history[a - historyLength/2] = history[a];
			history[a] = 0;
		}
		historyPointer = historyLength/2 +1;
	}
	
	public void setError(double errorInput){
		this.errorInput[0] = errorInput;
	}
	
	public boolean done(){
		if (Math.abs(statError) < sigma)
			return true;
		return false;
	}
	
	private double [] errorInput = null;
	
	private double localError;
	private double statError;
	private double [] history = null;
	
	private int historyPointer = 0;
	private int historyLength = 500;
	private int totalSteps = 0;
	private double sigma = 0.0001;
}


class QCell {

	/**
	 *  
	 */
	public QCell(int fconnections) {
		this.fconnections = new double[fconnections];
		for (int a = 0;a < fconnections;a++)
			//init the conenction on [-1,1]
			this.fconnections[a] = Math.random()*2 - 1;
	}
	
	public void energy(){
		this.energy = output*2*Math.PI;
	}
	
	
	private double output;
	private double input;
	private double energy;
	private double [] fconnections;

}