/************************************************************************
 * 
 * Created on Nov 30, 2005
 * Package qinterface
 * Filename qBrain.java
 * 
 ************************************************************************/

//***********************************************************************
//																		*
// Martin Lukac, 2003-2005 												*
// Intelligent Robotics Laboratory										*
// Portland State University 											*
// Office: 70-08 FAB, phone: 503-490-8478								*
// Mail:	P.O. Box 751												*
//			Portland, OR 97207-0751										*
// Email:	lukacm@cecs.pdx.edu											*
// Updates and documentation: 	www.pdx.edu\~lukacm						*
// **********************************************************************
//
//			*********************************************
//			*********************************************
// This program is provided WITHOUT ANY WARRANTY either expressed
// or implied.  You may study, use, modify, and distribute it for
// any non-commercial purposes.  For any commercial use, prior 
// permission from the authors must be granted.
// 			*********************************************
//			*********************************************
package qinterface;

import java.util.Vector;

import Quark2.Gates.GateComponent;
import Quark2.Gates.GateContainer;
import Quark2.Gates.QuantumRegister;
import Quark2.Gates.Implementation.GateStyle;
import Quark2.Gates.Implementation.Hadamard;
import Quark2.Gates.Implementation.Id;
import Quark2.Gates.Implementation.Measure;
import Quark2.Gates.Implementation.Permutation;
import Quark2.Gates.Implementation.SpecialGate;
import Quark2.ParseXML.ParseXML;
import biocell.BioThread;

import com.imsl.math.Complex;

/**
 * @author nabhoth
 *  
 */
/**
 * @author nabhoth
 *
 */
public final class qBrain extends BioThread {
    /**
     * @param name
     */
    public qBrain(String name) {
        super(name);
        g = new GateContainer("Brain", 0);
        qr = new QuantumRegister(10);
        init();
    }

    /**
     *  
     */
    public qBrain() {
        super();
        g = new GateContainer("Brain", 0);
        qr = new QuantumRegister(10);
        init();
    }

    /**
     * @param amps
     */
    public void updateRegister(int [] amps){
        /*TODO - add emotional states to qr*/
        
    }

    private void init() {
        int number = qr.numberOfQubits;

        double[] theta = new double[15];
        for (int i = 1; i < 15; i++) {
            theta[i] = Math.PI / Math.pow(2, i);
        }
        int ybits = 1;
        while (Math.pow(2, ybits) < number) {
            ybits++;
        }
        int numberOfQubits = qr.numberOfQubits;

        g.addGateComponent(new Id());
        //		SZ.QZ = new QuantumRegister(numberOfQubits);
        GateContainer H = new GateContainer("H", 1);
        g.addGateComponent(H);

        for (int i = 1; i <= numberOfQubits - 1; i++) {
            System.out.println(i);
            H.addGateComponent(new Hadamard(numberOfQubits, i));
        }

        GateContainer M = new GateContainer("M", 1);
        g.addGateComponent(M);

        for (int i = 0; i <= numberOfQubits; i++) {
            M.addGateComponent(new Measure(numberOfQubits, i));
        }

        GateContainer DFT = new GateContainer("DFT", 1);
        g.addGateComponent(DFT);

        int[] permu = new int[numberOfQubits];
        for (int i = 0; i < numberOfQubits; i++) {
            permu[i] = numberOfQubits - i;
        }
        for (int i = numberOfQubits; i < numberOfQubits; i++) {
            permu[i] = i + 1;
        }
        DFT.addGateComponent(new Permutation(numberOfQubits, permu));
        for (int i = numberOfQubits; i > 1; i--) {
            DFT.addGateComponent(new Hadamard(numberOfQubits, i));
            for (int j = numberOfQubits; j >= i; j--) {
                int[] ce = new int[1];
                if ((j - i + 1) == 1) {

                    ce[0] = i - 1;
                    DFT.addGateComponent(new SpecialGate(numberOfQubits, j,
                            ParseXML.parseQubitString(""), ce, new Complex(
                                    1.0f, 0.0f), new Complex(0.0f, 0.0f),
                            new Complex(0.0f, 0.0f), new Complex(0.0f, 1.0f),
                            1.0, new GateStyle(1, "T")));
                } else {
                    ce[0] = i - 1;
                    DFT.addGateComponent(new SpecialGate(numberOfQubits, j,
                            ParseXML.parseQubitString(""), ce, new Complex(
                                    1.0f, 0.0f), new Complex(0.0f, 0.0f),
                            new Complex(0.0f, 0.0f), new Complex((float) Math
                                    .cos(theta[j - i + 1]), (float) Math
                                    .sin(theta[j - i + 1])), 1.0,
                            new GateStyle(1, "T")));
                }

            }
        }
        DFT.addGateComponent(new Hadamard(numberOfQubits, 1));
        //		SZ.initnext();
        qr.doGroundState();
        //SZ.notifyObservers();

    }

    //GateComponent to be simulated next
    public static GateComponent next;

    //observers to be notified by notifyObservers
    Vector Observers;

    public void run() {
//        while (true) {
//
//            synchronized (this) {
//                next.applyTo(qr);
//                Vector gatelist = new Vector();
//                g.getShownGateList(gatelist);
//                int i = gatelist.indexOf(next);
//                i++;
//                try {
//                    next = (GateComponent) gatelist.elementAt(i);
//                } catch (Exception e) {
//                    next = null;
//                }
//                //System.out.println("Next" + next);
//                
//            }
//        }
//
    }

    //    /**
    //     * Notify registered Observers
    //     */
    //    public void notifyObservers() {
    //        try {
    //            MyObserver my;
    //            //System.out.println("notifying...");
    //            Enumeration ops = Observers.elements();
    //            while (ops.hasMoreElements()) {
    //                my = (MyObserver) ops.nextElement();
    //                //System.out.println(my);
    //                my.refresh();
    //            }
    //            Quark.scrollPane.doLayout();
    //        } catch (Exception e) {
    //        }
    //    }

    // Initial empty circuit
    GateContainer g = new GateContainer("Brain", 0);

    // Initial state vector
    QuantumRegister qr = new QuantumRegister(10);

}
