/*
 * Created on Jan 11, 2004
 *  
 */
package crlparser;

import java.io.File;
import java.io.PrintWriter;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.helpers.DefaultHandler;

import robot.CRLRobothandler;

/**
 * @author Martin Lukac
 * 
 * parser exclusively used to read and write a robot from/to a CRL file later
 * on this might be specified by a .dtd file
 */
public class CRLSAXParser extends Thread {

	/**
	 * default constructor
	 */
	public CRLSAXParser() {

		super();
		//fetch a SAX parser
		SAXParserFactory factory = SAXParserFactory.newInstance();
		try {
			parserSAX = factory.newSAXParser();
		} catch (Exception t) {
			t.printStackTrace();
		}
		start();
	}

	/**
	 * to get the output from this parser it has to have a listener otherise
	 * the file address or stream is lost
	 */
	public void addParserListener(ParserListener listener) {
		this.listener = listener;
	}

	/**
	 * First of two methods running this self starting threade class It parses
	 * the fille and constructs a robot
	 * 
	 * @param file -
	 *                   file to parse
	 */
	synchronized public void process(File file, Object o) {
		inputFile = file;
		handler = new CRLRobothandler(o);
		enabled = true;
		notifyAll();
	}

	/**
	 * First of two methods running this self starting threade class It parses
	 * the fille and constructs a robot
	 * 
	 * @param file -
	 *                   file to parse
	 */
	synchronized public void process(File file) {
		inputFile = file;
		enabled = true;
		notifyAll();
	}

	/**
	 * change the current handler
	 * 
	 * @param handler -
	 *                   the new handler
	 */
	public void chngHandler(DefaultHandler handler) {
		this.handler = handler;
	}

	/**
	 * Line is just send back until something else will be added
	 * 
	 * @param line -
	 *                   string to be parsed
	 */
	private String parse(String line) {
		if (listener != null)
			listener.load(new ParserEvent(line));
		return line;
	}

	/**
	 * overriden as form Thread
	 */
	public void run() {
		while (true) {
			try {
				synchronized (this) {
					while (!enabled) {
						try {
							wait(1000);
						} catch (InterruptedException intr) {
						}
					}
					if (handler != null && inputFile != null)
						parserSAX.parse(inputFile, handler);
					
					enabled = false;
				}
				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	//for multiple parsings of a file let's keep it in the memory
	private File inputFile;

	//The CNS is hanging on a listener to check synchronized objects
	private ParserListener listener;

	////The handler of teh sax parser - the DTD handler
	private DefaultHandler handler;

	//The factory yahooooooo - using the SAX method
	private SAXParserFactory factorySAX;

	//The factory for DOM models
	private DocumentBuilderFactory factoryDOM;

	//The parser it self
	private SAXParser parserSAX;

	//Parameters of the parser
	//parsing default is for one robot
	private boolean Multi = false;

	//writer
	private PrintWriter fW;
	
	//aloowable
	private boolean enabled = false;

}