/*
 * Created in 2003
 */
package crlparser;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
/**
 * @author martin Lukac 2003
 * 
 * A general purpose CRL threaded XML parser
 * It works only with files yet. The principle is the following.
 * The parser can be started once and each object using it 
 * have to implement a parserListener. For each use the parser
 *  will synthetize the given file and the parser gives a Vector of file pointers
 * as output encapsulated in the Parser Event to the listener.
 * It is threaded and synchronized however an eventQueue is required for an
 * overload in use. Without a listener the files can be retrieved by a provided public
 * return method
 */

public abstract class  Parser extends Thread {

	public Parser() {
		super();

	}

	/**
	 * to get the output from this parser it has to have a listener otherise the file address or stream is lost
	 */					 
	public void addParserListener(ParserListener listener) {
		this.listener = listener;
	}

	public void test() {
		File f = new File("robot.xml");
		process(f);
	}

	/**
	 * First of two methods running this self starting threade class
	 * It parses the fille according to default of previously loaded alphabet
	 * @param file - file to parse
	 */
	public abstract void process(File file);
	/**
	 * 
	 * @return array of files, all files for all robots or devices
	 */
	public File[] getFiles() {
		return (File[]) files.toArray();
	}

	/**
	 * 
	 */
	public abstract void process(Object mfE);
	
	/**
	 * unprotected method.. one must be sure to have the PrintWriter connected
	 * @param node
	 * @param indent
	 * @param fW
	 */
	public static void writeCRL(Node node, String indent, PrintWriter fW) {
		switch (node.getNodeType()) {
			case Node.DOCUMENT_NODE :
				{
					fW.print(indent + "<?xml version='1.0'?>");
					NodeList list = node.getChildNodes();
					int l = 0;
					while (l < list.getLength()) {
						writeCRL(list.item(l), indent, fW);
						l++;
					}
					break;
				}
			case Node.DOCUMENT_TYPE_NODE :
				{
					DocumentType docType = (DocumentType) node;
					fW.println("<!DOCTYPE " + docType.getName() + ">");
					break;
				}
			case Node.ELEMENT_NODE :
				{
					Element e = (Element) node;
					fW.println(indent + "<" + e.getTagName() + ">");

					String ind = indent + "   ";
					NodeList list = e.getChildNodes();
					int l = 0;
					while (l < list.getLength()) {
						writeCRL(list.item(l), ind, fW);
						l++;
					}

					fW.println(indent + "</" + e.getTagName() + ">");
					break;
				}
			case Node.TEXT_NODE :
				{
					String nod = ((Text)node).getData().trim();
					if (nod.length() > 0 && nod != null){
						fW.println(indent + nod);
					}
					break;
				}
				case Node.COMMENT_NODE: {
					System.out.println("Comment  node: " + node.getClass().getName());
					String nod = ((Text)node).getData().trim();
					if (nod.length() > 0 && nod != null){
						fW.println(indent + nod);
					}
					break;
				
				}
			default:
				System.out.println("Ignoring unknown node: " + node.getClass().getName());
				break;

		}

	}

	protected String[] tokenize(String input) {
		Vector v = new Vector();
		StringTokenizer t = new StringTokenizer(input);
		String cmd[];

		while (t.hasMoreTokens())
			v.addElement(t.nextToken());
		cmd = new String[v.size()];
		for (int i = 0; i < cmd.length; i++)
			cmd[i] = (String) v.elementAt(i);

		return cmd;
	}

	/**
	 * Creates a parametrized tokenizer returning an array of string
	 */
	protected String[] tokenize(String input, String delim) {
		Vector v = new Vector();
		StringTokenizer t = new StringTokenizer(input, delim);
		String cmd[];

		while (t.hasMoreTokens())
			v.addElement(t.nextToken());
		cmd = new String[v.size()];
		for (int i = 0; i < cmd.length; i++)
			cmd[i] = (String) v.elementAt(i);

		return cmd;
	}


	/**
	 * overriden as form Thread
	 */
	public void run() {
	}


	//the other level
	protected Vector files;

	//The CNS is hanging on a listener to check synchronized objects
	public ParserListener listener;

	//writer
	public PrintWriter fW;
}

class Gramm {
	Gramm() {

		rules = new String[8];
		rules[0] = "crl";
		rules[1] = "robot";
		rules[2] = "sync";
		rules[3] = "speech";
		rules[4] = "moves";
		rules[5] = "sensor";
		rules[6] = "visual";
		rules[7] = "wait";
	}

	private static String[] rules;

}


