package crlparser.xiay;

////////////////////////////////////////////////////////////////////////////
//  Name: CRL.Java
//  The entry file of this project
//  Compile all files: java CRL*.java
//  Run:   java CRL robot.xml [-n nthreads] [-p pattern] [-o output option]
//
//  Author:  Yu Xia
//  Created: 2/15/2004
/////////////////////////////////////////////////////////////////////////////
  
import java.io.*;

public class CRL 
{
	private	static String file_name;
	private	static int n_threads = 3;
	private	static String pattern = "robot";
	private	static int output_option = 0;
	private static String output_file_name;
	
	// showing error message for wrong input syntax
	public static void show_error() {
			System.out.println("USAGE: java CRL xml_file [-n nthreads] [-p pattern] [-o filename]");
			System.out.println("[-n nthreads] defines thread number used to process parsing. (default: 3)");
			System.out.println("[-p pattern]  decide which part of the xml will be send to the robots. (default robot)");
			System.out.println("[-o -1(screen)/1(file)] decide output options: screen / file. (default: screen)");
			System.out.println("");
			System.exit(0);
	}
	
	// checking user input command options
	public static void check_setting(String [] setting, int pos) {
		if(setting[pos].equals("-n"))  {
			n_threads = Integer.parseInt(setting[pos+1]);
		}  else if(setting[pos].equals("-p"))  {
			pattern = setting[pos+1];
		} else if(setting[pos].equals("-o")) {
			output_file_name = setting[pos+1];
			output_option = 1;
		} else {
			show_error();
		}
	}

	// main file
	public static void main(String [] args) throws IOException 
	{
		// checking the command line
		if((args.length > 0) || (args.length < 8)) {
				 if(args.length == 1)  {
				 		file_name = args[0];
				 } else if (args.length == 3) {
				 		file_name = args[0];
						check_setting(args, 1);
				 } else if (args.length == 5) {
				 		file_name = args[0];
						check_setting(args, 1);
						check_setting(args, 3);
				 } else if (args.length == 7) {
				 		file_name = args[0];
						check_setting(args, 1);
						check_setting(args, 3);
						check_setting(args, 5);
				 } else {
						show_error();
				}
		} else {
				show_error();
		} 
		
		// create file seperator object
		// seperate the one XML file to many different small files
		// according different robot
		// And get how many robot had been created
		CRL_File_Seperator file_sept = new CRL_File_Seperator(file_name);
		int file_num = file_sept.seperation();
		
		// create output file object
		CRL_Files crl_file = new CRL_Files(file_num);
		CRL_Output_File output_file;
		
		if(output_option == 1) { // save to file
			output_file = new CRL_Output_File(output_file_name);
		} else  {
			output_file = null;
		}

		// create threads for parsing robots
		for(int i=0; i<n_threads; i++)  {
			CRL_Parser crl_parsers = new CRL_Parser(crl_file, pattern, output_option, output_file);
			crl_parsers.start();
		}
		
		// close the file for output
		if(output_option == 1)
			output_file.close_file();
		
	} // end of main class
}
