package devs.motorpackage.asc16;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/*
 *	Martin Lukac, 2003-2005 		*
 *	Intelligent Robotics Laboratory										*
 *	Portland State University 											*
 *	Office: 70-08 FAB, phone: 503-490-8478								*
 *	Mail:	P.O. Box 751												*
 *			Portland, OR 97207-0751										*
 *	Email:	lukacm@cecs.pdx.edu										*
 *	Updates and documentation: 	www.pdx.edu\~lukacm				*
 ************************************************************************/
//************************************************************************
// * 	This program is provided WITHOUT ANY WARRANTY either expressed
// *	or implied.  You may study, use, modify, and distribute it for
// *	any non-commercial purposes.  For any commercial use, prior 
// *	permission from the authors must be granted.
// ************************************************************************/
/**
 * 
 * @author biamonte, lukac
 * 
 * The superclass of the Servo objects, definitions and abstract methods to be used by every single servo motor
 *
 */
public abstract class Motor extends Thread implements ASC16Listener {

	protected List motorListeners = new ArrayList();

	/**
	 * @return
	 */
	public abstract int getCurrentPosition();

	/**
	 * @return
	 */
	public abstract int getCurrentSpeed();

	/**
	 * @param pos
	 */
	public abstract void setSpeed(int speed);

	/**
	 * @param pos
	 */
	public abstract void setAcceleration(int acceleration);

	/**
	 * @return
	 */
	public abstract int getMotorNumber();

	/**
	 * @param n
	 */
	public void removeMotorListener(MotorListener n) {

		this.motorListeners.remove(n);

	}

	/**
	 * @param e
	 */
	public void fireMotorEvent(MotorEvent e) {

		Iterator nListeners = this.motorListeners.iterator();

		while (nListeners.hasNext()) {
			((MotorListener) nListeners.next()).motorReceived(e);
		}
	}

	
	protected static ASC16 motorController = null;  // every servo shares the same motor controller

	protected int currentPosition; 
	protected int highPosition;
	protected int lowPosition; 
	protected int midlePosition; 
	
	protected int newPosition; 
	protected int currentSpeed;
	protected int highSpeed;
	protected int lowSpeed;
	
	protected int newSpeed;
	protected int acceleration;
	protected int motorNumber; // Pin number servo is conected on STAMP

	

}
