package devs.motorpackage.asc16;
/*
 *	Martin Lukac, 2003-2005 		*
 *	Intelligent Robotics Laboratory										*
 *	Portland State University 											*
 *	Office: 70-08 FAB, phone: 503-490-8478								*
 *	Mail:	P.O. Box 751												*
 *			Portland, OR 97207-0751										*
 *	Email:	lukacm@cecs.pdx.edu										*
 *	Updates and documentation: 	www.pdx.edu\~lukacm				*
 ************************************************************************/
//************************************************************************
// * 	This program is provided WITHOUT ANY WARRANTY either expressed
// *	or implied.  You may study, use, modify, and distribute it for
// *	any non-commercial purposes.  For any commercial use, prior 
// *	permission from the authors must be granted.
// ************************************************************************/


import monitors.CommCenter;
import mns.BioDevice;
import mns.BioDeviceEvent;
import mns.BioDeviceListener;
import mns.MNSEvent;

/**
 * @author martin lukac
 * 
 * 			Controls servo motors, must point to a comunication class (ASC16) 
 *			Servo contains all information regarding servos. This
 *         includes the left right and center frequencies. 
 */
public class Servo extends Motor implements BioDevice, ASC16Listener {

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#getName()
	 */
	public String getname() {
		return name;
	}

	/**
	 * 
	 * @param servoNumber
	 */
	public Servo(int servoNumber) {

		super();
		try {
			if (servoNumber < 1 || servoNumber > 16) { // You have conection
													   // 0-15 for Motorspublic
													   // abstract void
													   // speed(int sp);
				throw new ServoException("Pin Number out of Range");
			}
		} catch (ServoException e) {
			CommCenter.showError("Pin number out of STAMP Range");
		}

		try {
			//this.setMotorController(motorController);
			// Set object to handle communication to board
		} catch (Exception e) {
			CommCenter
					.showError("In constructor Servo.bodyPartPointer or stampTalkerPointer forced to point to null");
		}
		setMotorNumber(servoNumber);
		initialize();
		motorController.register(this);

	}

	/**
	 * @param motorController -
	 *            ASC16
	 * @param low -
	 *            min position
	 * @param high -
	 *            max position
	 * @param low_speed -
	 *            min speed
	 * @param high_speed -
	 *            max speed
	 * @param servoNumber -
	 *            iD
	 */
	public Servo(ASC16 motorController, int low, int high, int low_speed,
			int high_speed, int servoNumber) {

		super();
		try {
			if (servoNumber < 1 || servoNumber > 16) { // You have conection
													   // 0-15 for Motorspublic
													   // abstract void
													   // speed(int sp);
				throw new ServoException("Pin Number out of Range");
			}
		} catch (ServoException e) {
			CommCenter.showError("Pin number out of STAMP Range");
		}

		try {
			//this.setMotorController(motorController);
			// Set object to handle communication to board
		} catch (Exception e) {
			CommCenter
					.showError("In constructor Servo.bodyPartPointer or stampTalkerPointer forced to point to null");
		}


		setHighPosition(high);
		setLowPosition(low);
		setMotorNumber(servoNumber);

		setHighSpeed(high_speed);
		setLowSpeed(low_speed);

		// All servos initialized to center position

		int center_pos1;
		int center_pos2;

		center_pos1 = this.getCenterPosition() / 256;
		center_pos2 = this.getCenterPosition() % 256;

		motorController.sendInt((int) this.motorNumber);
		motorController.sendInt(center_pos1);
		motorController.sendInt(center_pos2);
		setCurrentPosition(this.getCenterPosition());

		//initializing the serial port
		motorController.sendInt(111);
		motorController.sendInt(50);
		//initial settings
		setAcceleration(80);
		setSpeed(80);

		motorController.register(this);
		currentState = BioDevice.DONE;

		CommCenter.showComm("starting servo thread " + servoNumber);
	}

	/**
	 * @return Returns the pointer to the servo controller communication class.
	 * @throws NullPointerException
	 */
	public ASC16 getMotorController() throws NullPointerException {
		if (motorController == null)
			throw new NullPointerException("ASC16 points to NULL");
		return motorController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#getControls()
	 */
	public String getControls() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see mns.MNSListener#process(mns.MNSEvent)
	 */
	public void process(MNSEvent cnv) {
		// TODO Auto-generated method stub

	}
	
	/* (non-Javadoc)
	 * @see mns.BioDevice#addListener(mns.BioDeviceListener)
	 */
	synchronized public void addListener(BioDeviceListener listener) {
		this.listener = null;
		this.listener = listener;
		//System.out.println("listener to this servo is added:"+motorNumber+" "+this.listener);
		notifyAll();
	}

	/* (non-Javadoc)
	 * @see devs.motorpackage.asc16.ASC16Listener#stateChanged()
	 */
	public void stateChanged() {
		currentState = BioDevice.DONE;
		if (listener != null) {
			this.listener.bioEvent(new BioDeviceEvent(this, currentState));
		}
		CommCenter.showDebug("Servo " + getMotorNumber() + " is done "+this);
	}

	/**
	 * Sets Servo to center Independent of thread
	 */
	public void initialize() {

		// this sets all servos to center.
		synchronized (this) {

			//inint default init of the servo as defined by the manual
			CommCenter.showDebug(" servo: ");
			setHighPosition(HIGH_POSITION);
			CommCenter.showDebug(" servo: ");
			setLowPosition(LOW_POSITION);

			setHighSpeed(HIGH_SPEED);
			setLowSpeed(LOW_SPEED);
			
			// All servos initialized to center position

			int center_pos1;
			int center_pos2;

			center_pos1 = this.getCenterPosition() / 256;
			center_pos2 = this.getCenterPosition() % 256;

			motorController.sendInt((int) this.motorNumber);
			motorController.sendInt(center_pos1);
			motorController.sendInt(center_pos2);
			setCurrentPosition(this.getCenterPosition());

			//initializing the serial port
			motorController.sendInt(111);
			motorController.sendInt(50);
			//initial settings
			setAcceleration(80);
			setSpeed(80);

			rotate(50);
			motorController.register(this);
			currentState = BioDevice.DONE;
		}
	}

	/**
	 * 
	 * @author nabhoth
	 *  
	 */
	private class ServoException extends MotorException {

		ServoException(String message) {
			super(message);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#getCurrentPosition()
	 */
	public int getCurrentPosition() {
		return this.currentPosition * 100 / HIGH_POSITION;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#rotate(int)
	 */
	private void rotate(int newPos) {

		//check wether the servo is in iinversed direction - negative sign must be changed
		if (highPosition < lowPosition){
			newPos = 100 - newPos;
			newPos = Math.abs(newPos * (highPosition - lowPosition) / 100) + highPosition;
		} else 	newPos = (newPos * (highPosition - lowPosition) / 100) + lowPosition;

		CommCenter.showDebug("processing rotate on servo: "
				+ this.getMotorNumber() + " orig: " + newPos + " " + highPosition+"  "+lowPosition);
		

		int pos1 = newPos / 256;
		int pos2 = newPos % 256;
		int[] command = new int[4];
		command[0] = (int) this.motorNumber;
		command[1] = pos1;
		command[2] = pos2;
		command[3] = (180 + motorNumber);
		CommCenter.showDebug("processing rotate on servo: "
				+ this.getMotorNumber() + " orig: " + newPos + " " + pos1 + " "
				+ pos2 + " " + highPosition+"  "+lowPosition);
		motorController.sendInt(command);
		this.setCurrentPosition(newPos);
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#load(mns.MNSEvent)
	 */
	synchronized public void load(MNSEvent cnE) {

		//setting the device BUSY as soon sa possible
		currentState = BioDevice.BUSY;

		Object o = cnE.getData();
		if (o instanceof Number) {
			rotate(((Number) o).intValue());
		} else {
			try {
				String as = (String) o;
				int a = (Integer.parseInt(as));
				rotate(a);
			} catch (NumberFormatException e) {
				CommCenter.showError(e);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#loadCNTR(mns.MNSEvent)
	 */
	public void loadCNTR(MNSEvent mnE) {
		Object o = mnE.getData();
		if (o instanceof String[]) {
			String[] com = (String[]) o;
			com[0] = com[0].trim();
			if (com[0].startsWith("s")) {
				int a = (Integer.parseInt(com[1]));
				setSpeed(a);
			} else if (com[0].startsWith("a")) {
				int a = (Integer.parseInt(com[1]));
				setAcceleration(a);
			} else if (com[0].startsWith("m")) {
				int a = (Integer.parseInt(com[1]));
				setNormLowPosition(a);
			} else if (com[0].startsWith("n")) {
				int a = (Integer.parseInt(com[1]));
				setNormHighPosition(a);
			} else
				CommCenter.showComm("Servo do not know this control command: "+com[0]+"  "+com[1]);
			stateChanged();
			//CommCenter.showDebug("Servo executed command: "+com[0]+"  "+com[1]);

		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#flushQueue()
	 */
	public synchronized void flushQueue() {

	}

	/*
	 *  
	 */
	public int getCurrentState() {
		return this.currentState;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#getMotorNumber()
	 */
	public int getMotorNumber() {
		return motorNumber;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getCenterPosition()
	 */
	public int getCenterPosition() {
		return Math.abs(((this.getHighPosition() + this.getLowPosition()) / 2));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getHighPosition()
	 */
	public int getHighPosition() {
		return highPosition * 100 / HIGH_POSITION;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getLowPosition()
	 */
	public int getLowPosition() {
		return lowPosition * 100 / HIGH_POSITION;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getLowSpeed()
	 */
	public int getLowSpeed() {
		return lowSpeed * 100 / HIGH_SPEED;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getHighSpeed()
	 */
	public int getHighSpeed() {
		return highSpeed * 100 / HIGH_SPEED;
	}

	/* (non-Javadoc)
	 * @see devs.motorpackage.asc16.Motor#getCurrentSpeed()
	 */
	public int getCurrentSpeed() {
		return currentSpeed * 100 / HIGH_SPEED;
	}

	/* (non-Javadoc)
	 * @see devs.motorpackage.asc16.Motor#setAcceleration(int)
	 */
	public void setAcceleration(int acceleration) {
		this.acceleration = acceleration * highSpeed / 100;
		int [] com = new int [2];
		com[0] = motorNumber + 80;
		com[1] = acceleration;
		motorController.sendInt(com);
	}

	public int getAcceleration() {
	    return acceleration *100 / highSpeed;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see devs.motorpackage.asc16.Motor#setSpeed(int)
	 */
	public void setSpeed(int newSpeed) {
		newSpeed = newSpeed * highSpeed / 100;
		currentSpeed = newSpeed;
		int [] com = new int [2];
		com[0] = motorNumber + 60;
		com[1] = newSpeed;
		motorController.sendInt(com);
	}


	/**
	 * @param highPosition
	 *            The highPosition to set. - using the current servo SCALED values
	 */
	public void setNormHighPosition(int highPosition) {
		this.highPosition = highPosition * HIGH_POSITION / 100;
		currentState = BioDevice.DONE;
	}

	/**
	 * @param lowPosition
	 *            The lowPosition to set. - using the current servo SCALED values
	 */
	public void setNormLowPosition(int lowPosition) {
		this.lowPosition = lowPosition * HIGH_POSITION / 100;
		currentState = BioDevice.DONE;
	}

	/**
	 * @retrun highPosition returned.  - using the current servo SCALED values
	 */
	public int getNormHighPosition() {
		return this.highPosition * 100 / HIGH_POSITION;
	}

	/**
	 * @param lowPosition returned.  - using the current servo SCALED values
	 */
	public int getNormLowPosition() {
		return this.lowPosition * 100 / HIGH_POSITION;
	}

	/**
	 * @param highSpeed
	 *            The highSpeed to set.  - using the current servo SCALED values
	 */

	public void setNormHighSpeed(int highSpeed) {
		this.highSpeed = highSpeed * HIGH_SPEED / 100;
		currentState = BioDevice.DONE;
	}

	/**
	 * @param lowSpeed
	 *            The lowSpeed to set.  - using the current servo SCALED values
	 */
	public void setNormLowSpeed(int lowSpeed) {
		this.lowSpeed = lowSpeed * HIGH_SPEED /100;
		currentState = BioDevice.DONE;
	}

	
	/**
	 * @param highSpeed
	 *            The highSpeed to set.  - using the current servo REAL values
	 */

	private void setHighSpeed(int highSpeed) {
		this.highSpeed = highSpeed;
	}

	/**
	 * @param lowSpeed
	 *            The lowSpeed to set.  - using the current servo REAL values
	 */
	private void setLowSpeed(int lowSpeed) {
		this.lowSpeed = lowSpeed;
		currentState = BioDevice.DONE;
	}

	/**
	 * @param highPosition
	 *            The highPosition to set. - using the current servo REAL values
	 */
	private void setHighPosition(int highPosition) {
		this.highPosition = highPosition;
		currentState = BioDevice.DONE;
	}

	/**
	 * @param lowPosition
	 *            The lowPosition to set. - using the current servo REAL values
	 */
	private void setLowPosition(int lowPosition) {
		this.lowPosition = lowPosition;
		currentState = BioDevice.DONE;
	}

	/**
	 * @param motorNumber
	 *            The motorNumber to set.
	 */
	private void setMotorNumber(int motorNumber) {
		this.motorNumber = motorNumber;
		name = "servo "+motorNumber;
	}


	/**
	 * @param currentPosition
	 *            The currentPosition to set.
	 */
	private void setCurrentPosition(int currentPosition) {
		this.currentPosition = currentPosition;
	}

	/**
	 * @param currentSpeed
	 *            The currentPosition to set.
	 */
	private void setCurrentSpeed(int currentSpeed) {
		this.currentSpeed = currentSpeed;
	}

	/**
	 * @param currentSpeed
	 *            The currentPosition to set.
	 */
	private void setCurrentAcceleration(int currentAcceleration) {
		this.acceleration = currentAcceleration;
	}

	/**
	 * Comment for <code>currentState</code>
	 */
	private int currentState = 0;

	/**
	 * Comment for <code>listener</code>
	 */
	private BioDeviceListener listener;

	private String name;
	/**
	 *  
	 */
	static {
		motorController = new ASC16("COM1");
	}


}