package devs.motorpackage.miniSSCII;
/**
/************************************************************************
 * 	This program is provided WITHOUT ANY WARRANTY either expressed 		*
 *	or implied.  You may study, use, modify, and distribute it for 		*
 *	any non-commercial purposes.  For any commercial use, prior 		*
 *	permission from the author must be granted.							*
 *	? Jacob D. Biamonte, 2003-2004 Created on Jan 22, 2004				*
 *	Intelligent Robotics Laboratory										*
 *	Portland State University 											*
 *	Office: 70-08 FAB, phone: 503-490-8478								*
 *	Mail:	P.O. Box 751												*
 *			Portland, OR 97207-0751										*
 *	Email:	biamonte@cecs.pdx.edu										*
 *	Updates and documentation: 	www.ece.pdx.edu\~biamonte				*
 ************************************************************************/

import javax.comm.*;
import java.io.*;

/**
 * @author biamonte
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
public class MiniSSCII implements CommunicatorInterface {

	//TODO add array list to add servo's by number and make sure
	// only one is using each pin number.

	/**
	 * Comment for <code>mPortId</code>
	 */
	public static CommPortIdentifier mPortId = null;
	/**
	 * Comment for <code>mPort</code>
	 */
	public static SerialPort mPort = null;
	/**
	 * Comment for <code>os</code>
	 */
	public static OutputStream os = null;
	/**
	 * Comment for <code>SYNC_MARKER</code>
	 */
	public final static byte SYNC_MARKER = (byte) 255;

	/**
	 * @param portName
	 */
	public MiniSSCII(String portName) {
		try {
			this.openPort(portName);
		} catch (Exception e) {
			e.printStackTrace();
		}
		int baudRate = 9600;
		try {
			this.initPort(baudRate);
		} catch (UnsupportedCommOperationException e) {
			System.out.println(e.getMessage());
			System.exit(1);
		}

		// Open the output stream for the port.
		try {
			MiniSSCII.os = this.getOutputStream();
		} catch (IOException e) {
			this.closePort();
			System.out.println(e.getMessage());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see CommunicatorInterface#send(byte)
	 */
	public synchronized void sendByte(byte b) {
		try {
			MiniSSCII.os.write(b);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see CommunicatorInterface#receiveByte(byte)
	 */
	public void receiveByte(byte r) {
		// TODO Auto-generated method stub

	}

	/**
	 * @param s
	 */
	public synchronized void sendServoPacket(MotorPacket s) {
		try {
			while(busy){
				wait(1000);
				System.out.println("sleeping");
			}
			System.out.println("Servo "+s.getPin());
			System.out.println(s.getPos());
			// 255 to let PIC know you are talking to it.
				busy = true;
				MiniSSCII.os.write(MiniSSCII.SYNC_MARKER);
				MiniSSCII.os.write(s.getPin()); // PIN servo is conected
				MiniSSCII.os.write(s.getPos()); // new position to move

			busy = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
		notifyAll();

	}
	/**
	 * @param portName
	 * @throws NoSuchPortException
	 * @throws PortInUseException
	 */
	public void openPort(String portName)
		throws NoSuchPortException, PortInUseException {
		System.out.println("Opening port..." + portName);
		MiniSSCII.mPortId = CommPortIdentifier.getPortIdentifier(portName);
		MiniSSCII.mPort = (SerialPort) MiniSSCII.mPortId.open(portName, 30000);
	}

	/**
	 *  
	 */
	public void closePort() {
		System.out.println("Closing port...");
		MiniSSCII.mPort.close();
	}

	/**
	 * @param baud
	 * @throws UnsupportedCommOperationException
	 */
	public void initPort(int baud) throws UnsupportedCommOperationException {
		System.out.println("Initializing Port...");

		// Set the baud rate and other parameters.
		MiniSSCII.mPort.setSerialPortParams(
			baud,
			SerialPort.DATABITS_8,
			SerialPort.STOPBITS_1,
			SerialPort.PARITY_NONE);

		// Setup the flow control
		MiniSSCII.mPort.setFlowControlMode(
			javax.comm.SerialPort.FLOWCONTROL_NONE);
	}

	/**
	 * Get an output stream that we can use to control the servo controller
	 * (MiniSSCII).
	 * 
	 * @return OutputStream to use for communicating with the servo controller.
	 * @exception IOExeception
	 *                if the OutputStream can not be openned.
	 * @throws IOException
	 */
	OutputStream getOutputStream() throws IOException {
		return MiniSSCII.mPort.getOutputStream();
	}

	public static void main(String[] args) {

		MiniSSCII s = new MiniSSCII("COM1");
		s.sendServoPacket(new MotorPacket((byte) 4, (byte) 200));
		System.out.println("End");
	}
	
	private boolean busy = false;
}
