package devs.motorpackage.miniSSCII;
// Servo.java
// (c) Biamonte 2003
/**
 * Controls servo motors, must point to a comunication class (SSCII) All values
 * relating to position of servo represent value from 0 - 254 where 127 is
 * center Left and Right values are one time
 */

/**
 * @author biamonte Servo contains all information regarding servos. This
 *         includes the left right and center frequencies. Motors fire
 *         MotorEvents when they reach a new destination. Servo(SSCII sscii,
 *         int low, int high, int servoNumber)
 *  
 */
public class Servo extends Motor implements MotorInterface {

	protected MiniSSCII SSCIIPointer = null;
	//SSCII servo range 0-254 where 127 is center
	protected int currentPosition; // 
	protected int highPosition; // 
	protected int lowPosition; // 
	protected int newPosition; // 
	protected int motorNumber; // Pin number servo is conected on STAMP

	/**
	 * Comment for <code>instructionQueue</code>
	 */
	public InstructionQueue instructionQueue = new InstructionQueue();

	/**
	 * @param sscii
	 * @param low
	 * @param high
	 * @param servoNumber
	 */
	public Servo(MiniSSCII sscii, int low, int high, int servoNumber) {

		super();
		try {
			if (servoNumber < 0
				|| servoNumber > 15) { // You have conection 0-15 for Motors
				throw new ServoException("Pin Number out of Range");
			}
		} catch (ServoException e) {
			System.out.println("Pin number out of STAMP Range");
		}

		try {
			this.setSSCIIPointer(sscii);
			// Set object to handle communication to board
		} catch (Exception e) {
			System.out.println(
				"In constructor Servo.bodyPartPointer or stampTalkerPointer forced to point to null");
		}

		this.setHighPosition(high);
		this.setLowPosition(low);
		this.setMotorNumber(servoNumber);

		// All servos initialized to center position outside of thread.
		this.initialize();
		// runs in thread untill instruction Queue gets data. i.e. size = 0
		System.out.println("starting servo thread" + servoNumber);
		start();
	}

	/**
	 * @return Returns the pointer to the servo controller communication class.
	 * @throws NullPointerException
	 */
	public MiniSSCII getSSCIIPointer() throws NullPointerException {
		if (this.SSCIIPointer == null)
			throw new NullPointerException("stampTalker points to null");
		return this.SSCIIPointer;
	}

	/**
	 * Sets Servo to center Independent of thread
	 */
	public void initialize() {

		// this synchronized block sets all servos to center.
		// if you do not want this to happen override this method when you
		// declare a servo
		//synchronized (this) {

//			this.SSCIIPointer.sendByte(MotorCommand.SYNC_MARKER);
//			this.SSCIIPointer.sendByte((byte) this.motorNumber);
//			this.SSCIIPointer.sendByte((byte) this.getCenterPosition());
			this.SSCIIPointer.sendServoPacket(
					new MotorPacket((byte) this.motorNumber, (byte) this.getCenterPosition()));

			// CurrentPosition is always set after each movement.
			this.setCurrentPosition(this.getCenterPosition());

		//}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#run()
	 */
	// Thread waits for instruction Queue to grow > 0
	public void run() {

		while (true) {
			synchronized (this) {
			while (this.instructionQueue.size() == 0) {

				try {
					//System.out.println("sleeping " + this.getName());
					wait(100); //wait 1/10 sec

				} catch (InterruptedException e) {
					System.err.println("Servo sleep interrupted");
				}
			}

			if (this.instructionQueue.size() == 0) {
				System.err.println("Error in Servo, instructionQueue mismatch");
			}

			System.out.println("Awake! " + this.getName());
			// take next instruction out of queue and cast it as a MotorCommand
			MotorCommand c = (MotorCommand) this.instructionQueue.remove();
			int nextInstruction = c.getNewPosition();

			int i = 0; // counter for next loops

			if (nextInstruction > this.getCurrentPosition()) {

//				i = this.getCurrentPosition();
//				while (i <= nextInstruction){
//					//this.setCurrentPosition(i);
					this.setCurrentPosition(nextInstruction);
//					this.SSCIIPointer.sendServoPacket(
//						new MotorPacket((byte) this.motorNumber, (byte) i));
					this.SSCIIPointer.sendServoPacket(
							new MotorPacket((byte) this.motorNumber, (byte) nextInstruction));
//					i += 5;
//				}

			} else if (nextInstruction < this.getCurrentPosition()) {

//				i = this.getCurrentPosition();
//				while (i > nextInstruction){
//					this.setCurrentPosition(i);
//					this.SSCIIPointer.sendServoPacket(
//						new MotorPacket((byte) this.motorNumber, (byte) i));
//					i -= 5;
//				}
				this.setCurrentPosition(nextInstruction);
//				this.SSCIIPointer.sendServoPacket(
//						new MotorPacket((byte) this.motorNumber, (byte) i));
				this.SSCIIPointer.sendServoPacket(
					new MotorPacket((byte) this.motorNumber, (byte) nextInstruction));

			}
		}
		}
	}

	/**
	 * @param stampTalker
	 */
	private void setSSCIIPointer(MiniSSCII sscii) throws Exception {
		if (sscii == null)
			throw new Exception("Servo.stampTalkerPointer can not point to null");
		this.SSCIIPointer = sscii;
	}

	private class ServoException extends MotorException {

		ServoException(String message) {
			super(message);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#getCurrentPosition()
	 */
	public int getCurrentPosition() {
		// TODO Auto-generated method stub
		return this.currentPosition;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#rotate(int)
	 */
	public synchronized void rotate(int newPos) {
		//System.out.println("roate passed: " + pos)
//		MotorCommand mc = new MotorCommand(newPos);
//		this.instructionQueue.add(mc);
		this.setCurrentPosition(newPos);
//		this.SSCIIPointer.sendServoPacket(
//				new MotorPacket((byte) this.motorNumber, (byte) i));
		this.SSCIIPointer.sendServoPacket(
			new MotorPacket((byte) this.motorNumber, (byte) newPos));

		//System.out.println("rotate to " + mc.getNewPosition());
		notifyAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Motor#getMotorNumber()
	 */
	public int getMotorNumber() {

		return this.motorNumber;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getCenterPosition()
	 */
	public int getCenterPosition() {

		return Math.abs(((this.getHighPosition() + this.getLowPosition()) / 2));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getHighPosition()
	 */
	public int getHighPosition() {
		//TODO error check
		return this.highPosition;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getLowPosition()
	 */
	public int getLowPosition() {
		//TODO error check
		return this.lowPosition;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see MotorInterface#getNewPosition()
	 */
	public int getNewPosition() {

		return this.newPosition;
	}

	/**
	 * @param currentPosition
	 *            The currentPosition to set.
	 */
	private void setCurrentPosition(int currentPosition) {

		this.currentPosition = currentPosition;
	}

	/**
	 * @param highPosition
	 *            The highPosition to set.
	 */
	private void setHighPosition(int highPosition) {

		this.highPosition = highPosition;
	}

	/**
	 * @param lowPosition
	 *            The lowPosition to set.
	 */
	private void setLowPosition(int lowPosition) {

		this.lowPosition = lowPosition;
	}

	/**
	 * @param motorNumber
	 *            The motorNumber to set.
	 */
	private void setMotorNumber(int motorNumber) {

		this.motorNumber = motorNumber;
	}

	/**
	 * @param newPosition
	 *            The newPosition to set.
	 */
	private void setNewPosition(int newPosition) {

		this.newPosition = newPosition;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		MiniSSCII s = new MiniSSCII("COM1");
		Servo s1 = new Servo(s, 0, 254, 1);
		Servo s2 = new Servo(s, 0, 254, 0);

//		s1.rotate(254);
//		s1.rotate(0);
//		s2.rotate(254);
//		s2.rotate(0);
//		s1.rotate(254);
//		s1.rotate(0);
//
//		s2.rotate(254);
//		s2.rotate(0);

	}

}
