/************************************************************************
 * 
 * Created on Nov 3, 2005
 * Package devs.musicpackage
 * Filename DanceGenerator.java
 * 
 ************************************************************************/

//***********************************************************************
//																		*
// Martin Lukac, 2003-2005 												*
// Intelligent Robotics Laboratory										*
// Portland State University 											*
// Office: 70-08 FAB, phone: 503-490-8478								*
// Mail:	P.O. Box 751												*
//			Portland, OR 97207-0751										*
// Email:	lukacm@cecs.pdx.edu											*
// Updates and documentation: 	www.pdx.edu\~lukacm						*
// **********************************************************************
//
//			*********************************************
//			*********************************************
// This program is provided WITHOUT ANY WARRANTY either expressed
// or implied.  You may study, use, modify, and distribute it for
// any non-commercial purposes.  For any commercial use, prior 
// permission from the authors must be granted.
// 			*********************************************
//			*********************************************
package devs.musicpackage;

import java.awt.List;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.logging.Level;

import kj.dsp.KJDigitalSignalProcessingAudioDataConsumer;
import mns.BioDevice;
import mns.BioDeviceListener;
import mns.MNSEvent;
import monitors.CommCenter;
import nns.BioSensor;
import nns.BioSensorEvent;
import nns.BioSensorListener;
import nns.NNS;

import com.lightdev.app.playlisteditor.ui.JPlayerDisplay;
import com.lightdev.lib.audio.Player;
import com.lightdev.lib.audio.PlayerListener;

/**
 * @author nabhoth
 * 
 */
public class DanceGenerator extends BioSensor implements PlayerListener,
        ActionListener, SpectrumAnalyzerListener {

    /* ----------------The generator part ------------------ */
    /*
     * (non-Javadoc)
     * 
     * @see devs.musicpackage.SpectrumAnalyzerListener#update(int[])
     */
    public void update(int[] coeffs) {
        String t = "";
        for (int a = 0; a < coeffs.length; a++)
            t += coeffs[a] + " ";
        //System.out.println(getListener().toString());
        if (getListener() != null)
            this.getListener().sensorPercieved(
                    new BioSensorEvent("dancegenerator", t));

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
    }

    
    /* (non-Javadoc)
     * @see nns.BioSensor#process(mns.MNSEvent)
     */
    public void process(MNSEvent cnv) {
        String data = (String)cnv.getData();
        if (data.indexOf('*') >= 0){
            data = data.substring(data.indexOf('*')+1);
        }
        if (data.startsWith("start") || data.startsWith("dance")){
                play.actionPerformed(new ActionEvent(this, 0, "play"));
                this.seState(BioDevice.BUSY);
        } else if (data.startsWith("stop")){
            stop.actionPerformed(new ActionEvent(this, 0, "stop"));
            this.seState(BioDevice.DONE);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.lightdev.lib.audio.PlayerListener#stateChanged(com.lightdev.lib.audio.Player,
     *      int, int)
     */
    public void stateChanged(Player p, int fromStatus, int toStatus) {
        System.out.println("Player stateChanged: fromStatus=" + fromStatus
                + ", toStatus=" + toStatus + " " + getClass() + " "
                + Level.FINEST);
        switch (toStatus) {
        case Player.INITIALIZING:
            System.out.println("Player stateChanged: initializing" + " "
                    + getClass() + " " + Level.INFO);
            break;

        case Player.SKIPPING:
            System.out.println("Player stateChanged: skipping" + " "
                    + getClass() + " " + Level.INFO);
            break;

        case Player.READY:
            System.out.println("Player stateChanged: ready" + " " + getClass()
                    + " " + Level.INFO);
            break;

        case Player.PLAYING:
            synchronizer.start(player.getSourceDataLine());
            if (fromStatus == Player.READY) { // started initially
                /*
                 * System.out.println(" bytePerFrame=" + p.getSizeBytes() /
                 * p.getFrameCount() + ", bytePerSec=" + p.getFrameRate() *
                 * p.getSizeBytes() / p.getFrameCount() + ", sampleSizeInBits=" +
                 * p.getSampleSizeInBits() + ", channels=" + p.getChannels() +
                 * ",\r\n" + " frame size=" + p.getFrameSize() + ", sample
                 * rate=" + p.getSampleRate() + ", frame rate=" +
                 * p.getAudioFrameRate() + ", isbigendian=" + p.isBigEndian());
                 */
                System.out.println("Player stateChanged: started "
                        + p.getArtist() + " - " + player.getTitle() + " "
                        + getClass() + " " + Level.INFO);

            } else if (fromStatus == Player.PAUSED) { // resumed from pause
                System.out.println("Player stateChanged: resume " + getClass()
                        + " " + Level.INFO);
                synchronizer.stop();
            }
            break;

        case Player.PAUSED:
            System.out.println("Player stateChanged: paused" + " " + getClass()
                    + " " + Level.INFO);
            synchronizer.stop();
            break;

        case Player.STOPPED_PLAY:
            System.out.println("Player stateChanged: stopped_play" + " "
                    + getClass() + " " + Level.INFO);
            synchronizer.stop();
            break;

        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.lightdev.lib.audio.PlayerListener#progress(com.lightdev.lib.audio.Player,
     *      long)
     */
    public void progress(Player p, long progressValue) {
        // if(!userMovesSlider) {
        // songSlider.setValue( (int) (progressValue / 1000l + sliderOffset));
        // }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.lightdev.lib.audio.PlayerListener#decoded(com.lightdev.lib.audio.Player,
     *      byte[], int)
     */
    public void decoded(Player p, byte[] buf, int byteCount) {
        // Application.log("decoded", getClass(), Level.INFO);
        p.write(buf, buf.length);
        synchronizer.writeAudioData(buf);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.lightdev.lib.audio.PlayerListener#exception(com.lightdev.lib.audio.Player,
     *      java.lang.Exception)
     */
    public void exception(Player p, Exception ex) {
        ex.printStackTrace();
        // Application.logStackTrace(ex, getClass());
    }

    /**
     * @param name
     */
    public DanceGenerator(BioSensorListener nns) {
        super(EXTERO_SENSOR, "dancegenerator");
        Translator t = new Translator((NNS) nns);
        addListener(t);
        System.out.println("LIstener "+t.toString());
        play = new PlayItemAction(this);
        stop = new StopPlaybackAction(this);
        init();
    }

    /**
     * get the currently running play thread
     * 
     * @return PlayThread the running thread or null if none
     */
    public Player getPlayer() {
        return this.player;
    }

    /**
     * get the currently set volume
     * 
     * @return int the volume
     */
    public int getVolume() {
        return 50;// volume.getValue();
    }

    /**
     * @return Returns the currentsong.
     */
    public synchronized String getCurrentsong() {
        return currentsong;
    }

    /**
     * @param currentsong
     *            The currentsong to set.
     */
    public synchronized void setCurrentsong(String currentsong) {
        this.currentsong = currentsong;
    }

    /**
     * @return Returns the analyzer.
     */
    public synchronized SpectrumAnalyzer getAnalyzer() {
        return analyzer;
    }

    /**
     * @param analyzer
     *            The analyzer to set.
     */
    public synchronized void setAnalyzer(SpectrumAnalyzer analyzer) {
        this.analyzer = analyzer;
        if (this.analyzer != null)
            this.analyzer.setListener(this);
    }

    /**
     * @return Returns the synchronizer.
     */
    public synchronized KJDigitalSignalProcessingAudioDataConsumer getSynchronizer() {
        return synchronizer;
    }

    /**
     * @param synchronizer
     *            The synchronizer to set.
     */
    public synchronized void setSynchronizer(
            KJDigitalSignalProcessingAudioDataConsumer synchronizer) {
        this.synchronizer = synchronizer;
    }

    /**
     * @param player
     *            The player to set.
     */
    public synchronized void setPlayer(Player player) {
        this.player = player;
        this.player.addPlayerListener(this);
        this.player.addPlayerListener(analyzer);
        this.player.addPlayerListener(new JPlayerDisplay());
        notifyAll();
    }

    private String getSongByLocation(int i){
    
        if (i < songList.getSize().height)
            return songList.getItem(i);
        else return songList.getItem(0);
    }
    
    private void init() {
        File dir = new File(musicdefault);
        String[] children = dir.list();
        songList = new List();
        for (int i = 0; i < children.length; i++) {
            File file = new File(dir, children[i]);
            if (!(file.isDirectory()) && file.getName().endsWith("mp3"))
                songList.add(dir+"/"+file.getName());
            else if (file.isDirectory()) visitAllFiles(file);
        }
        currentsong = getSongByLocation(((int)(Math.random()*songList.getItemCount())));
        CommCenter.showComm("\n danceGenerator Loaded: "+songList.getItemCount());
        CommCenter.showComm("CurrentSong Loaded: "+currentsong);
    }

    // Process only files under dir
    private  void visitAllFiles(File dir) {
        String[] children = dir.list();
        for (int i=0; i<children.length; i++) {
            File file = new File(dir, children[i]);
            if (!(file.isDirectory()) && file.getName().endsWith("mp3"))
                songList.add(dir+"/"+file.getName());
            else if (file.isDirectory()) visitAllFiles(file);
        }
    }

    /** the currently active play thread, if any */
    private Player player;

    /** button to switch between spectrum analyzer and display panel */
    /** the SpectrumAnalyzer */
    private SpectrumAnalyzer analyzer;

    /** slider to indicate/select position within current song */
    private KJDigitalSignalProcessingAudioDataConsumer synchronizer = new KJDigitalSignalProcessingAudioDataConsumer(
            4096, 10);

    private List songList;

    private String currentsong = null;

    public static final float VOLUME_CORRECTION = 35;

    private String musicdefault = "c:\\Documents and Settings\\martin\\Desktop\\music";    
    private PlayItemAction play;
    private StopPlaybackAction stop;
    
}
