/*
 * Light Development Playlist Editor
 * Copyright (C) 2004 Ulrich Hilger
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package devs.musicpackage;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

import com.lightdev.lib.audio.Player;

public class PlayItemAction extends AbstractAction {

  /**
   * constructor
   * @param app PlaylistEditor  the application this action belongs to
   */
  public PlayItemAction(DanceGenerator app) {
    this.app = app;
    putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
      KeyEvent.VK_Y, KeyEvent.CTRL_MASK));
    this.setEnabled(true);
  }

  /**
   * play the currently selected playlist item
   *
   * <p>Selects the first item in the playlist if no item is selcted so far</p>
   *
   * @param e ActionEvent  the event that led to this action
   */
  public void actionPerformed(ActionEvent e) {
    Player player = app.getPlayer();
    if(player != null && player.isPaused()) {
      player.resumePlay();
    }
    else {
      app.setPlayer(new Player());
      player = app.getPlayer();
      player.setVolume(player.getVolume(), player.getVolume());
      player.setBalance(player.getBalance());
      player.setVolume(100, DanceGenerator.VOLUME_CORRECTION);
      System.out.println(player.getBalance());
      System.out.println(player.getVolume());
      System.out.println("starting playback");
      System.out.println(new File(app.getCurrentsong()));
      File f = new File(app.getCurrentsong());
      if (f.exists())
          player.startPlay(f);
    }
  }

  /** the application this action belongs to */
  private DanceGenerator app;

}
