/************************************************************************
 * 
 * Created on Nov 3, 2005
 * Package devs.musicpackage
 * Filename Translator.java
 * 
 ************************************************************************/

//***********************************************************************
//																		*
// Martin Lukac, 2003-2005 												*
// Intelligent Robotics Laboratory										*
// Portland State University 											*
// Office: 70-08 FAB, phone: 503-490-8478								*
// Mail:	P.O. Box 751												*
//			Portland, OR 97207-0751										*
// Email:	lukacm@cecs.pdx.edu											*
// Updates and documentation: 	www.pdx.edu\~lukacm						*
// **********************************************************************
//
//			*********************************************
//			*********************************************
// This program is provided WITHOUT ANY WARRANTY either expressed
// or implied.  You may study, use, modify, and distribute it for
// any non-commercial purposes.  For any commercial use, prior 
// permission from the authors must be granted.
// 			*********************************************
//			*********************************************
package devs.musicpackage;

import java.util.StringTokenizer;

import mns.BioDeviceEvent;
import nns.BioSensorEvent;
import nns.BioSensorListener;
import nns.NNS;
import biocell.BioCell;

/**
 * @author nabhoth
 *
 */
public class Translator extends BioCell implements BioSensorListener {

    /* (non-Javadoc)
     * @see nns.BioSensorListener#sensorCRLPercieved(nns.BioSensorEvent)
     */
    public void sensorCRLPercieved(BioSensorEvent event) {
    }

    /* (non-Javadoc)
     * @see nns.BioSensorListener#sensorPercieved(nns.BioSensorEvent)
     */
    public void sensorPercieved(BioSensorEvent event) {
        process(event.getEventString());

    }

    /* (non-Javadoc)
     * @see mns.BioDeviceListener#bioEvent(mns.BioDeviceEvent)
     */
    public void bioEvent(BioDeviceEvent bdE) {
    }

    /**
     * @param nns
     */
    public Translator(NNS nns) {
        super("DanceGenerator Translator");
        this.nns = nns;
        for (int a = 0; a < states.length; a++)
            for (int b = 0; b < states[0].length; b++) {
                states[a][b] = 50;
            }

    }

    /**
     * @param motionDirection
     */
    public void process(String motionData) {
        //get the states of the robot motor controller
        //System.out.println(motionData+" : "+nns.getMotState().length);
        shiftVal(nns.getMotState(), states);
        //shift the current position into the history
        StringTokenizer st = new StringTokenizer(motionData, " ");
        int[] indata = new int[st.countTokens()];
        int a = 0;
        while (st.hasMoreTokens()) {
            indata[a] = Integer.parseInt(st.nextToken());
            a++;
        }
        shiftVal(indata, input);
        //get the current approximation
        int[] data = getVal(input);
        //get the real amplification
        for (int f = 0; f < data.length; f++)
            data[f] -= indata[f];
        System.out.println("Translator activated");
        for (a = 0; a < nns.getMotState().length; a++) {
            states[0][a] = states[0][a]
                    + (int) ((Math.random() * 2 - 1) * data[a]);
            if (states[0][a] <= 5)
                states[0][a] = 10;
            else if (states[0][a] >= 95)
                states[0][a] = 90;
        }

        states[0][8] = 100 - states[0][6];
        states[0][7] = 100 - states[0][5];
        nns.setMotState(states[0]);
        //TODO add the moodlookup       
    }

    /**
     * @param val
     */
    public void shiftVal(int[] val, int[][] destination) {
        for (int a = destination.length - 1; a > 0; a--)
            for (int b = 0; b < destination[0].length; b++)
                destination[a][b] = destination[a - 1][b];
        for (int a = 0; a < val.length; a++)
            destination[0][a] = val[a];
    }

    private int[] getVal(int[][] destination) {
        int result[] = new int[destination[0].length];
        for (int a = 0; a < destination.length; a++)
            for (int b = 0; b < destination[0].length; b++)
                result[b] += destination[a][b];

        for (int b = 0; b < result.length; b++)
            result[b] /= destination.length;
        return result;
    }

    private int[][] states = new int[13][13];;

    private int[][] input = new int[13][13];

    private String[] histor = new String[13];

    //the uplink
    private NNS nns;
}
