package devs.sensorpackage.milford5120;

/*******************************************************************************
 * This program is provided WITHOUT ANY WARRANTY either expressed or implied.
 * You may study, use, modify, and distribute it for any non-commercial
 * purposes. For any commercial use, prior permission from the authors must be
 * granted.
 * 
 * @author martin lukac
 ******************************************************************************/
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;

import javax.comm.CommPortIdentifier;
import javax.comm.NoSuchPortException;
import javax.comm.PortInUseException;
import javax.comm.SerialPort;
import javax.comm.UnsupportedCommOperationException;

import nns.BioSensor;
import nns.BioSensorEvent;
import nns.BioSensorListener;
import biocell.BioThread;

/*
 * Example of a implementation of a simple sensor: Sonar sensor Milford 5120 It
 * runs in pseudo autonomous mode, because it is controlled by software but it
 * range of sweeping is controlled by significant change detection
 * 
 * Connects directly to the serial or serial emulated through USB Just change
 * the name of the dedicated port to either to windows names or unix like names.
 */
public class Milford5120 extends BioSensor {

	/* (non-Javadoc)
	 * @see nns.BioSensorListener#sensorCRLPercieved(nns.BioSensorEvent)
	 */
	public void sensorCRLPercieved(BioSensorEvent event) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see nns.BioSensorListener#sensorPercieved(nns.BioSensorEvent)
	 */
	public void sensorPercieved(BioSensorEvent event) {
		// TODO Auto-generated method stub

	}

	/**
	 * @param portName -
	 *            the name of the port
	 */
	public Milford5120(String portName) {
		super(EXTERO_SENSOR, "Milford5120");
		int baudRate = 9600;
		try {
			openPort(portName);
			initPort(baudRate);
			bos = new BufferedOutputStream(getOutputStream());
			is = getInputStream();
			setEnabled(true);
			new InputReader();
			start();
		} catch (Exception e) {
			e.printStackTrace();
		}
		//start directly as soon as the class is started
	}

	/**
	 * @param portName -
	 *            name of the port
	 * @param listener -
	 *            the Device listener
	 */
	public Milford5120(String portName, BioSensorListener listener) {

		super(EXTERO_SENSOR, "Milford5120");
		int baudRate = 9600;
		try {
			openPort(portName);
			initPort(baudRate);
			bos = new BufferedOutputStream(getOutputStream());
			is = getInputStream();
			addSensorListener(listener);
			new InputReader();
			start();
		} catch (Exception e) {
			e.printStackTrace();
		}
		//start directly as soon as the class is started
		setEnabled(true);
	}

	/**
	 * @param rangeLow
	 * @param rangeHigh
	 */
	public void setRange(int center) {
		try {
			System.out.println("Old range is:" + rangeLow + " :: " + center
					+ " :: " + rangeHigh);
			int counter0 = 0;
			int counter5 = 0;
			for (int a = rangeLow; a < rangeLow + sweepSize / 2; a++) {
				if (visualChanged[a])
					counter0++;
			}
			for (int a = rangeLow + sweepSize / 2; a < rangeHigh; a++) {
				if (visualChanged[a])
					counter5++;
			}

			for (int a = rangeLow; a < rangeHigh; a++)
				visualChanged[a] = false;

			if (counter5 > counter0)
				center += sweepSize / 2;
			else if (counter5 < counter0)
				center -= sweepSize / 2;
			else {
				center = (int)((Math.random()*2)-1)*sweepSize + rangeLow; 
			}
			if (center + sweepSize / 2 > MAX_SWEEP) {
				this.rangeLow = MAX_SWEEP - sweepSize;
				this.rangeHigh = MAX_SWEEP;
			} else if (center - sweepSize / 2 < MIN_SWEEP) {
				this.rangeLow = MIN_SWEEP;
				this.rangeHigh = MIN_SWEEP + sweepSize;
			} else {
				this.rangeLow = center - sweepSize / 2;
				this.rangeHigh = center + sweepSize / 2;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println("New range set:" + rangeLow + " :: " + center
				+ " :: " + rangeHigh);
		//notifyAll();
	}

	/**
	 * Sends a command to the ASC16 micro-controller
	 * 
	 * @param i -
	 *            integer
	 */
	public void sendInt(int i) {
		try {
			bos.write(84);
			bos.flush();
			bos.write(i);
			bos.flush();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return Returns the enabled.
	 */
	synchronized public boolean isEnabled() {
		return enabled;
	}

	/**
	 * @param enabled
	 *            The enabled to set.
	 */
	synchronized public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 * a sleep/awake thread that as soon as is awaken
	 * send a pulse to the sensor. Then waits until being
	 * enabled.
	 */
	public void run() {
		int counter = 0;

		boolean ud = true;
		try {
			while (true) {
				if (isEnabled()) {
					setEnabled(false);
					if (!ud) {
						counter -= sweepStep;
						spaceCounter--;
						if (counter <= rangeLow)
							ud = true;
					} else {
						counter += sweepStep;
						spaceCounter++;
						if (counter >= rangeHigh)
							ud = false;
					}
					//System.out.println("Counter: "+counter+" space counter: "+spaceCounter);
					currentPosition = counter;
					sendInt(counter);
				} else {
					try {
						wait(10);
					} catch (Exception e) {

					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param portName
	 * @throws NoSuchPortException
	 * @throws PortInUseException
	 */
	private void openPort(String portName) throws NoSuchPortException,
			PortInUseException {
		System.out.println("Opening port..." + portName);
		mPortId = CommPortIdentifier.getPortIdentifier(portName);
		mPort = (SerialPort) mPortId.open(portName, 30000);
	}

	/**
	 * Close the currently open serial port
	 */
	private void closePort() {
		System.out.println("Closing port...");
		mPort.close();
	}

	/**
	 * @param baud
	 * @throws UnsupportedCommOperationException
	 */
	private void initPort(int baud) throws UnsupportedCommOperationException {
		System.out.println("Initializing Port...");
		// Set the baud rate and other parameters.
		mPort.setSerialPortParams(baud, SerialPort.DATABITS_8,
				SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);

		// Setup the flow control
		mPort.setFlowControlMode(javax.comm.SerialPort.FLOWCONTROL_NONE);

	}

	/**
	 * @return OutputStream to use for communicating with the servo controller.
	 * @throws IOException
	 */
	private OutputStream getOutputStream() throws IOException {
		return mPort.getOutputStream();
	}

	/**
	 * @return InuputStream to use for communicating with the servo controller.
	 * @throws IOException
	 */
	private InputStream getInputStream() throws IOException {
		return mPort.getInputStream();
	}

	/**
	 * 
	 * @author Lukac
	 * 
	 * A threaded listener of the serial port. It seems the ASC16 is almost
	 * random how it sends the ybtes back, thus this class is catching them all
	 */
	class InputReader extends BioThread {

		int answer = -100;

		public InputReader() {
			super();
			this.start();
		}

		public void run() {
			boolean ud = true;
			try {
				bir = new BufferedReader(new InputStreamReader(is));
				InputStreamReader isr = new InputStreamReader(is);
				synchronized (this) {
					while (true) {

						if (!isEnabled()) {
							answer = isr.read();
							if (spaceCounter > -1) {
								//                        System.out.println("old: " + visualField[spaceCounter]
								//                                + " new: " + (answer * 42)+" non transformed: "+answer);
								if (Math.abs(answer * 42
										- visualField[spaceCounter]) > ((double) visualField[spaceCounter]) * 0.2) {
									visualField[spaceCounter] = Math
											.abs(answer * 42);
									System.out.println(" CHANGE : "
											+ visualField[spaceCounter]+" : "+spaceCounter);
//									//sleep(30);
//									if (spaceCounter <= rangeLow
//											|| spaceCounter >= rangeHigh)
//										setRange(spaceCounter);
									visualChanged[spaceCounter] = true;
									setEnabled(true);
									notifyAll();

								} else
									System.out.println(" NO CHANGE : "
											+ visualField[spaceCounter]+" : "+spaceCounter);
								//                        if (visualField[spaceCounter] >= 100) {
								//                            fireSensorEvent(new BioSensorEvent(
								//                                    this,
								//                                    "POS "
								//                                            + Integer
								//                                                    .toString(getCurrentPosition())
								//                                            + " VAL "
								//                                            + Integer
								//                                                    .toString(visualField[spaceCounter])));
								//                        }
								visualChanged[spaceCounter] = false;
								setEnabled(true);
								notifyAll();
							}

						} else {
							try {
								sleep(5);
							} catch (Exception e) {

							}
						}
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Comment for <code>enabled</code>
	 */
	private boolean enabled = true;

	/**
	 * Comment for <code>mPortId</code>
	 */
	public static CommPortIdentifier mPortId = null;

	/**
	 * Comment for <code>mPort</code>
	 */
	public static SerialPort mPort = null;

	/**
	 * Comment for <code>is</code>
	 */
	public static InputStream is = null;

	/**
	 * Comment for <code>currentPosition</code>
	 */
	private int currentPosition;

	/**
	 * Comment for <code>visualField</code>
	 */
	public static int[] visualField = new int[180];

	/**
	 * Comment for <code>visualChanged</code>
	 */
	public static boolean[] visualChanged = new boolean[180];

	/**
	 * Comment for <code>rangeLow</code>
	 */
	private int rangeLow = 70;

	/**
	 * Comment for <code>rangeHigh</code>
	 */
	private int rangeHigh = 130;

	/**
	 * Comment for <code>sweepArea</code>
	 */
	private int sweepStep = 3;

	/**
	 * Comment for <code>sweepSize</code>
	 */
	private int sweepSize = 60;

	public static int MAX_SWEEP = 160;

	public static int MIN_SWEEP = 0;

	/**
	 * Comment for <code>bos</code>
	 */
	public static BufferedOutputStream bos = null;

	/**
	 * Comment for <code>bir</code>
	 */
	public static BufferedReader bir = null;

	/**
	 * the blocking variable <code>busy</code>
	 */
	protected boolean changed = false;

	/**
	 * the step array <code>stepArray</code>
	 */
	private int[] stepArray;

	/**
	 * the current position in the array <code>spaceCounter</code>
	 */
	private int spaceCounter = 0;

	public static void main(String[] args0) {
		new Milford5120("/dev/ttyS0");
	}
}

