package devs.sensorpackage.sonarexplorer;
/************************************************************************
 * 	This program is provided WITHOUT ANY WARRANTY either expressed
 *	or implied.  You may study, use, modify, and distribute it for
 *	any non-commercial purposes.  For any commercial use, prior 
 *	permission from the authors must be granted.
 *@author lukac, biamonte
 ************************************************************************/
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Vector;

import javax.comm.CommPortIdentifier;
import javax.comm.NoSuchPortException;
import javax.comm.ParallelPort;
import javax.comm.PortInUseException;
import javax.comm.UnsupportedCommOperationException;

import biocell.BioThread;
/*
 *Class implementing the interface to the ASC16 controller. 
 *Only basic functions are implemented becuse the controll is on higher level 
 */
public class SensorExplorer {

	/**
	 * Comment for <code>mPortId</code>
	 */
	public static CommPortIdentifier mPortId = null;
	/**
	 * Comment for <code>mPort</code>
	 */
	public static ParallelPort mPort = null;
	/**
	 * Comment for <code>os</code>
	 */
	public static OutputStream os = null;
	/**
	 * Comment for <code>is</code>
	 */
	public static InputStream is = null;
	
	/**
	 * Comment for <code>servos</code>
	 */
	public static Vector servos;

	public static BufferedOutputStream bos = null;
	/**
	 * @param portName
	 */
	public SensorExplorer(String portName) {

   		Enumeration  portList = CommPortIdentifier.getPortIdentifiers();
        boolean portFound = false;
        try {

        while (portList.hasMoreElements()) {
        	mPortId = (CommPortIdentifier) portList.nextElement();
            if (mPortId.getPortType() == CommPortIdentifier.PORT_PARALLEL) {
                System.out.println("the port name is " + mPortId.getName());
                mPort = (ParallelPort) mPortId.open(mPortId.getName(),2000);
                
                //mPort.setMode(ParallelPort.LPT_MODE_PS2);
                int parallelPortMode = mPort.getMode();
                System.out.println("the mode " +  parallelPortMode);

                switch(parallelPortMode){
                    case ParallelPort.LPT_MODE_ECP:
                        System.out.println("mode ECP");
                        break;
                    case ParallelPort.LPT_MODE_EPP:
                        System.out.println("mode EPP");
                        break;
                    case ParallelPort.LPT_MODE_NIBBLE:
                        System.out.println("mode NIBBLE");
                        break;
                    case ParallelPort.LPT_MODE_PS2:
                        System.out.println("mode PS2 - byte mode ");
                        break;
                    case ParallelPort.LPT_MODE_SPP:
                        System.out.println("mode SPP- compatibility mode");
                        break;
                    default:
                        System.out.println("invalid mode");
                }
            }
        }

 			//openPort(portName);
			//initPort(baudRate);
			SensorExplorer.os = getOutputStream();
			bos = new BufferedOutputStream(getOutputStream());
			SensorExplorer.is = getInputStream();

			new InputReader();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Register a Servo with ASC16 controller to obtain direct feedback
	 * @param s - Servo
	 */
//	public void register(Sensor s){
//		servos.add(s);
//	}
	
	/**
	 * Sends a command to the ASC16 micro-controller 
	 * @param i - integer
	 */
	public void sendInt(int i) {
		try {
			//bos.write(84);			
			//bos.flush();
			bos.write(i);
			bos.flush();
			System.out.println("Sent: "+i);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 * @param portName
	 * @throws NoSuchPortException
	 * @throws PortInUseException
	 */
	public void openPort(String portName)
		throws NoSuchPortException, PortInUseException {
		if  (SensorExplorer.mPort != null) closePort();
		System.out.println("Opening port..." + portName);
		SensorExplorer.mPortId = CommPortIdentifier.getPortIdentifier(portName);
		SensorExplorer.mPort = (ParallelPort) SensorExplorer.mPortId.open(portName, 30000);
	}

	/**
	 *  Close the currently open serial port
	 */
	public void closePort() {
		System.out.println("Closing port...");
		SensorExplorer.mPort.close();
	}

	/**
	 * @param baud
	 * @throws UnsupportedCommOperationException
	 */
	public void initPort(int baud) throws UnsupportedCommOperationException {
		System.out.println("Initializing Port...");

		// Set the baud rate and other parameters.
//		SensorExplorer.mPort.setSerialPortParams(
//			baud,
//			SerialPort.DATABITS_8,
//			SerialPort.STOPBITS_1,
//			SerialPort.PARITY_NONE);
//
//		// Setup the flow control
//		SensorExplorer.mPort.setFlowControlMode(
//			javax.comm.SerialPort.FLOWCONTROL_NONE);
	}

	/**
	 * @return OutputStream to use for communicating with the servo controller.
	 * @throws IOException
	 */
	OutputStream getOutputStream() throws IOException {
		return SensorExplorer.mPort.getOutputStream();
	}

	/**
	 * @return InuputStream to use for communicating with the servo controller.
	 * @throws IOException
	 */
	InputStream getInputStream() throws IOException {
		return SensorExplorer.mPort.getInputStream();
	}

	
	/**
	 * 
	 * @author Lukac
	 *
	 * A threaded listener of the serial port. It seems the ASC16
	 * is almost random how it sends the ybtes back, thus this
	 * class is catching them all
	 */
	class InputReader extends BioThread{
		
		int counter = 0;
		public InputReader(){
			super();
			start();
		}
		
		public void run(){
			int answer;
			boolean ud = true;
			sendInt(1);
			
			while(true){
				try{
					//System.out.println("Milford Stream Listener waiting");
					sleep(0);
					//answer = is.read();
					//System.out.println("The sensor send a feedback: "+ (answer * 42)+ " mm");
//					if (!ud) counter -= 3;
//					else counter+= 3;
//					if (counter >= 180) { counter = 180; ud = false;}
//					else if (counter <= 0) {counter = 0; ud = true;}
					sendInt(counter);
					 counter+= 1;
				}catch (Exception e){
					e.printStackTrace();
				}
			}
		}
	}
	
	
	public static void main (String [] args){
		SensorExplorer mil = new SensorExplorer("/dev/lp0");
	}
	
	//the blocking variable
	protected boolean busy = false;
}
