package devs.speechpackage;

import java.beans.PropertyVetoException;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.speech.synthesis.SpeakableAdapter;
import javax.speech.synthesis.SpeakableEvent;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;
import javax.speech.synthesis.Voice;
import javax.xml.parsers.DocumentBuilderFactory;

import mns.BioDevice;
import mns.BioDeviceEvent;
import mns.BioDeviceListener;
import mns.MNSEvent;
import monitors.CommCenter;
import devs.speechpackage.fortunegen.FortuneTeller;

/**
 * The main speak device for the interface. To be done : - voice changing -
 * voice manipulation - create a graphical interface to the voice manipulation
 */
public class SpeechGenerator extends SpeakableAdapter implements BioDevice {

	/**
	 *  
	 */
	public SpeechGenerator() {
		mainSynth = new SpeechSynthesizer();
		this.name = "speech generator";
		 fortune = new FortuneTeller();
		setVoices();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#load(mns.MNSEvent)
	 */
	public void load(MNSEvent cnE) {
		Object o = cnE.getData();
		if (o == null)
			return;

		CommCenter.showComm("Speech initiated with: " + o);
		if (o instanceof File) {
			speakFile((File) o);
		} else if (o instanceof String) {
			if (((String)o).startsWith("fortune")){
				String f = fortune.getRandom();
				CommCenter.showComm("Speech generated with: "+f);
				speakLine(fortune.getRandom());
			}
			else speakLine((String) o);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#loadCNTR(mns.MNSEvent)
	 */
	public void loadCNTR(MNSEvent mnE) {
		Object o = mnE.getData();
		if (o instanceof String[]) {
			String[] com = (String[]) o;
			switch(com[0].charAt(0)){
				case 'm':
					setVolume(Float.parseFloat(com[1]));
				break;
				case 'p':
					setPitch(Float.parseFloat(com[1]));
				break;
				case 'v':
					setVoice(com[1]);
				break;
				case 'r':
					setRate(Float.parseFloat(com[1]));
				break;
				default:
					CommCenter.showComm("Speech synthesizer do not know this control command: "+com[0]+"  "+com[1]);
				break;
			}

			if (listener != null)
				listener.bioEvent(new BioDeviceEvent(this, BioDevice.DONE));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.MNSListener#process(mns.MNSEvent)
	 */
	public void process(MNSEvent cnv) {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#getControls()
	 */
	public String getControls() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#flushQueue()
	 */
	public void flushQueue() {
		mainSynth.getEngine().cancelAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#addListener(mns.BioDeviceListener)
	 */
	public void addListener(BioDeviceListener rP) {
		listener = rP;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#getCurrentPosition()
	 */
	public int getCurrentPosition() {
		return -100;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see mns.BioDevice#getCurrentState()
	 */
	public int getCurrentState() {
		if (mainSynth.getEngine().getEngineState() == Synthesizer.QUEUE_NOT_EMPTY)
			return BioDevice.BUSY;
		else
			return BioDevice.DONE;
	}

	/**
	 * @param voiceName -
	 *            set a voice based on its name
	 */
	private synchronized void setVoice(String name) {
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
			if (name != mainVoice.getName()) {
				LocalVoice lVoice = getLocalVoice(name);
				if (lVoice != null) {
					if (lVoice.getengineName() != mainSynth.getSynthDesc()
							.getEngineName()) {
						mainSynth.setEngine(lVoice.getengineName());
						mainSynth.getEngine().waitEngineState(
								Synthesizer.ALLOCATED);
					}
					mainVoice = lVoice.getlocalVoice();
					mainSynth.getEngine().getSynthesizerProperties().setVoice(
							mainVoice);
				} else
					CommCenter.showError("SpeechGenerator:" + "Voice " + "\'"
							+ name + "\' not found ");
			}
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + "Voice not found " + e
					+ "," + name);
		}

		CommCenter.showComm("voice " + mainVoice.getName()
				+ " is up and running");
	}

	/**
	 * @param name
	 * @return
	 */
	private LocalVoice getLocalVoice(String name) {
		Iterator i = voiceList.iterator();
		while (i.hasNext()) {
			LocalVoice lV = (LocalVoice) i.next();
			if (lV.getlocalName().equalsIgnoreCase(name))
				return lV;
		}
		return null;
	}

	/**
	 * @return an array of strings representing the voices
	 */
	public String[] getVoices() {
		String[] result = new String[voiceList.size()];
		for (int a = 0; a < voiceList.size(); a++) {
			result[a] = ((LocalVoice) voiceList.get(a)).getlocalName();
		}
		return result;
	}

	/**
	 * @return initiate the list of all voices
	 */
	private void setVoices() {
		voiceList = new Vector();
		for (int i = 0; i < mainSynth.getEngineList().size(); i++) {
			SynthesizerModeDesc desc = (SynthesizerModeDesc) mainSynth
					.getEngineList().get(i);
			Voice[] voices = desc.getVoices();
			for (int a = 0; a < voices.length; a++) {
				voiceList.add(new LocalVoice(voices[a], desc.getEngineName()));
				CommCenter.showComm(voices[a].getName() + " "
						+ desc.getEngineName());
			}
		}
		mainVoice = ((LocalVoice) voiceList.get(0)).getlocalVoice();
		CommCenter.showComm("found total of: " + voiceList.size() + " Voices");
		CommCenter.showComm("Default voice installed is: "
				+ mainVoice.getName());
	}

	/**
	 * sets the pitch
	 * 
	 * @param pitch -
	 *            float
	 */
	private void setPitch(float pitch) {
		try {
			//			change from 0 - 100 to 0 - 300 Hz
			float ptch = pitch * 3;
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
			mainSynth.getEngine().getSynthesizerProperties().setPitch(ptch);
//			if (listener != null)
//				listener.bioEvent(new BioDeviceEvent(this, BioDevice.DONE));

			CommCenter.showDebug("Pitch set");
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
	}

	/**
	 * sets the speaking speed
	 * 
	 * @param rate -
	 *            float
	 */
	private void setRate(float rate) {
		try {
			//change from 0 - 100 to 0 - 250
			float rte = (float) (rate * 2.5);
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
			mainSynth.getEngine().getSynthesizerProperties().setSpeakingRate(
					rte);
			CommCenter.showDebug("Rate set");
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
	}

	/**
	 * sets the volume of the current synthesizer
	 * 
	 * @param volume -
	 *            float
	 */
	private void setVolume(float volume) {
		try {
			//transform from 0 - 100 to 0 - 1
			float vol = volume / 100;
			//mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
			mainSynth.getEngine().getSynthesizerProperties().setVolume(vol);
//			if (listener != null)
//				listener.bioEvent(new BioDeviceEvent(this, BioDevice.DONE));

			CommCenter.showDebug("Volume set");
			
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
//		catch (InterruptedException e) {
//			CommCenter.showError("SpeechGenerator:" + e);
//		}
	}

	/**
	 * @param pitchrange
	 */
	private void setPitchRange(float pitchrange) {
		try {
			//change from 0 - 100 to 20% to 80% of the pitch base
			float ptchrng = mainSynth.getEngine().getSynthesizerProperties()
					.getPitch();
			ptchrng *= pitchrange / 100;
			mainSynth.getEngine().getSynthesizerProperties().setPitch(ptchrng);
//			if (listener != null)
//				listener.bioEvent(new BioDeviceEvent(this, BioDevice.DONE));

		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
	}

	/**
	 * @return - the volume
	 */
	public float getVolume() {
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
		} catch (InterruptedException e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
		return mainSynth.getEngine().getSynthesizerProperties().getVolume();
	}

	/**
	 * @return - the speed of speech
	 */
	public float getRate() {
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
		} catch (InterruptedException e) {
			CommCenter.showError("SpeechGenerator:" + (e));
		}
		return mainSynth.getEngine().getSynthesizerProperties()
				.getSpeakingRate();
	}

	/**
	 * @return - the pitch value
	 */
	public float getPitch() {
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
		} catch (InterruptedException e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
		return mainSynth.getEngine().getSynthesizerProperties().getPitch();
	}

	/**
	 * @return - the effective pitch range
	 */
	public float getPitchRange() {
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.ALLOCATED);
		} catch (InterruptedException e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
		return mainSynth.getEngine().getSynthesizerProperties().getPitchRange();
	}

	/**
	 * speaks a simple non-JSML line of text
	 * 
	 * @param line -
	 *            String
	 */
	public void speakLine(String line) {
		CommCenter
				.showComm("Speechsynthsizer: Voice initiated with following settings: ");
		CommCenter
				.showComm("Speed: " + getRate() + ", Volume: " + getVolume()
						+ "' Pitch: " + getPitch() + "' PitchRange: "
						+ getPitchRange());
		if (line.startsWith("fortune"))
			line = fortune.getRandom();
			CommCenter.showComm("Speech generated with: "+line);
		try {
			mainSynth.getEngine().waitEngineState(Synthesizer.RESUMED);
			mainSynth.getEngine().speak(line, this);
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
	}

	/**
	 * speaks a JSML formated text
	 * 
	 * @param f -
	 *            the file to speak
	 */
	public void speakFile(File f) {
		String line;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		try {
			BufferedReader bR = new BufferedReader(new InputStreamReader(
					new FileInputStream(f)));
			String fileText = "";
			while ((line = bR.readLine()) != null)
				fileText += line;
			mainSynth.getEngine().speak(fileText, this);
		} catch (Exception e) {
			CommCenter.showError("SpeechGenerator:" + e);
		}
	}

	public void markerReached(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.BUSY));
	}

	public void speakableCancelled(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.END));
	}

	public void speakableEnded(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.DONE));
	}

	public void speakablePaused(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.PAUSED));
	}

	public void speakableResumed(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.RESUMED));
	}

	public void speakableStarted(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.START));
	}

	public void topOfQueue(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.BUSY));
	}

	public void wordStarted(SpeakableEvent e) {
		dumpEvent(e);
		if (listener != null)
		listener.bioEvent(new BioDeviceEvent(this, BioDevice.START));
	}

	private void dumpEvent(SpeakableEvent e) {
		CommCenter.showComm(" EVT: " + e.paramString() + " source: "
				+ e.getSource());
	}

	/**
	 * @return the name of this element
	 */
	public String getname() {
		return name;
	}

	/**
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @author Martin Lukac
	 */
	class LocalVoice {
		public LocalVoice(Voice localVoice, String engineName) {
			this.localVoice = localVoice;
			this.engineName = engineName;
		}

		public Voice getlocalVoice() {
			return this.localVoice;
		}

		public String getengineName() {
			return this.engineName;
		}

		public String getlocalName() {
			return localVoice.getName();
		}

		private String engineName;

		private Voice localVoice;
	}

	//Voice - one voice per engine allowed
	private Voice mainVoice;

	//main synthesizer
	private SpeechSynthesizer mainSynth;

	//list of all voices - for inconvenient time moments
	private List voiceList;

	//the upper courrant of the information
	private BioDeviceListener listener;

	//name of this thing
	private String name;
	
	private FortuneTeller fortune;
}
