/*
 * Created on Feb 13, 2004
 * 
 */
package devs.speechpackage;
import java.util.List;

import javax.speech.Central;
import javax.speech.EngineList;
import javax.speech.synthesis.SpeakableAdapter;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;

import monitors.CommCenter;
import biocell.BioThread;
/**
 * @author Martin Lukac
 */
public class SpeechSynthesizer extends BioThread {
	/**
	 * default constructor simply check the ressources available
	 * from the speech.properties file
	 */
	public SpeechSynthesizer() {
		super("speech synthesizer");
		try {
			//generate the initial list of synthesizers
			engineList = Central.availableSynthesizers(null);
			//default setting the first engine
			setEngine((SynthesizerModeDesc) engineList.get(0));
			//start the thread
			threadSuspended = false;
			this.start();
		} catch (Exception e) {
			CommCenter.showError("Prolems starting the speech synthesizer: "
					+ e);
			CommCenter.showError("SpeechSynthesizzer: "+e);
		}
	}
	/**
	 * @return - main sytnehsizer
	 */
	public Synthesizer getEngine() {
		return mainSynth;
	}
	/**
	 * @return main Sytnhesizer Description
	 */
	public SynthesizerModeDesc getSynthDesc() {
		return mainSynthDesc;
	}
	/**
	 * @return EngineList - list of registered engines
	 */
	public EngineList getEngineList() {
		return engineList;
	}
	/**
	 * set a speech synthesizer according to its name
	 * 
	 * @param engineName -
	 *                   the name of an engine
	 */
	synchronized public void setEngine(String engineName) {
		try {
			//checking if it is done with work
			if (mainSynth != null)
				mainSynth.waitEngineState(Synthesizer.QUEUE_EMPTY);
			else
				mainSynth = Central.createSynthesizer(mainSynthDesc);
			if (engineName != mainSynthDesc.getEngineName()) {
				//find the correct
				for (int a = 0; a < engineList.size(); a++) {
					if (((SynthesizerModeDesc) engineList.get(a))
							.getEngineName() == engineName) {
						setEngine((SynthesizerModeDesc) engineList.get(a));
						break;
					}
				}
				//the setting of engine is done ia a thread
				threadSuspended = false;
				notify();
			}
		} catch (Exception ee) {
			CommCenter.showError("SpeechSynthesizzer: engine setting problem "+ee);
		}
	}
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		while (true) {
			try {
				//yet only to avoid sending text before previous is done
				synchronized (this) {
					while (threadSuspended) {
						try {
							wait(5);
						} catch (Exception e) {
							CommCenter.showError("SpeechSynthesizer: Thread Problem1 : "+e);
						}
					}
					CommCenter.showComm("Allocating speech synthesizer: "
							+ mainSynthDesc.getEngineName());
					mainSynth = Central.createSynthesizer(mainSynthDesc);
					mainSynth.allocate();
					mainSynth.resume();
					CommCenter
							.showComm("Speech synthesizer allocated and ready");
				}
			} catch (Exception e) {
				CommCenter.showError("SpeechSynthesizer: Thread Problem2 : "+e+" "+mainSynth.getEngineState());
			}
			threadSuspended = true;
		}
	}
	/**
	 * sets an synthesizer according to a SynthesizerModeDesc
	 * 
	 * @param desc -
	 *                   the SynthesizerModeDesc
	 */
	synchronized private void setEngine(SynthesizerModeDesc desc) {
		try {
			//checking if it is done with work
			if (mainSynth != null) {
				mainSynth.waitEngineState(Synthesizer.QUEUE_EMPTY);
				//little bit wastefull but enough till now
				if (mainSynthDesc.getEngineName() != desc.getEngineName()) {
					mainSynthDesc = new SynthesizerModeDesc(
							desc.getEngineName(),
							desc.getModeName(),
							desc.getLocale(),
							desc.getRunning(),
							desc.getVoices());
					//the setting of engine is done ia a thread
					threadSuspended = false;
					notify();
				}

			} else{
				mainSynthDesc = new SynthesizerModeDesc(
						desc.getEngineName(),
						desc.getModeName(),
						desc.getLocale(),
						desc.getRunning(),
						desc.getVoices());
			}
			
		} catch (Exception ee) {
			CommCenter.showError(("SpeechSynthesizzer: Engine setting problem "+ee));
		}
	}
	/**
	 * @return - the list of all synthesizers
	 */
	public List getSynth() {
		return engineList;
	}
	/**
	 * add a synthesizer listener, which is useful to check the status of the
	 * speech, and some time constants
	 */
	public void addsynthListener(SpeakableAdapter sA) {
		mainSynth.addSpeakableListener(sA);
	}
	private boolean threadSuspended;
	//ModDescription - describes the current used synthesizer
	private SynthesizerModeDesc mainSynthDesc;
	//main synthesizer
	private Synthesizer mainSynth;
	//list of all synthesizers - engines
	private EngineList engineList;
	//thread
	private Thread speechThread;
}