package devs.visionpackage.org.cmotch.imageviewer;

/**
 * <p>Title: DisplayImage</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Chris Motch
 * @version 1.0
 */

import java.awt.image.BufferedImage;

import brain.reflexgenerator.PMotionCalculator;
import biocell.BioThread;
import devs.visionpackage.org.cmotch.framegrabber.FrameGrabber;
import devs.visionpackage.org.cmotch.visionlibrary.ColorDetection;
import devs.visionpackage.org.cmotch.visionlibrary.ColorManipulation;
import devs.visionpackage.org.cmotch.visionlibrary.EdgeDetection;
import devs.visionpackage.org.cmotch.visionlibrary.GaussianBlurFilter;
import devs.visionpackage.org.cmotch.visionlibrary.Manipulations;

/**
 * Threaded class that will capture images from a Framegrabber and process them
 * depending on what processing is enabled
 */
public class DisplayImage extends BioThread {
    private ImagePanel mainIP;

    private FrameGrabber mainFG;

    private PMotionCalculator pmc;

    //Flags to determine what processing needs to occur
    private boolean stopThread = false;

    private boolean processFindFullEdges = false;

    private boolean processFindEdges = false;

    private boolean processFindMotion = false;

    private boolean processGreyScale = false;

    private boolean processBlackWhite = false;

    private boolean processColorDetection = false;

    private boolean processGaussianBlur = false;

    private int edgeDetectionType = 0;

    private int frameCount = 0;

    private int detectcount = 0;

    /**
     * Constructor
     * 
     * @param ip
     *            The image panel that the image will be displayed on
     * @param frameGrab
     *            The framegrabber that will capture the images from the camera
     */

    public DisplayImage(ImagePanel ip, FrameGrabber frameGrab,
            PMotionCalculator pmc) {
        //	super("vision panel");
        mainIP = ip;
        mainFG = frameGrab;
        this.pmc = pmc;
    }

    /**
     * Stop the capture / display thread from running
     *  
     */
    public void stopCapture() {
        stopThread = true;
    }

    /**
     * Set the edge type used by the edge detection processor
     * 
     * @param edgeType
     *            The edgetype that the edge detection processor will use
     */
    public void setEdgeType(int edgeType) {
        edgeDetectionType = edgeType;
    }

    /**
     * Toggle the edge detection processor on and off
     *  
     */
    public void toggleFindEdges() {
        if (processFindEdges) {
            processFindEdges = false;
        } else {
            processFindEdges = true;
        }
    }

    /**
     * Toggle the full edge detection processor, this processor combinins
     * binirization, gaussian blur and horizontal and vertical edge detection
     *  
     */
    public void toggleFindFullEdges() {
        if (processFindFullEdges) {
            processFindFullEdges = false;
        } else {
            processFindFullEdges = true;
        }
    }

    /**
     * Toggle gray scale of the image
     *  
     */
    public void toggleGrayScale() {
        if (processGreyScale) {
            processGreyScale = false;
        } else {
            processGreyScale = true;
        }
    }

    /**
     * Toggle the color detection processor
     *  
     */
    public void toggleColorDetection() {
        if (processColorDetection) {
            processColorDetection = false;
        } else {
            processColorDetection = true;
        }
    }

    /**
     * Toggle the binarization image processor
     *  
     */
    public void toggleBlackWhite() {
        if (processBlackWhite) {
            processBlackWhite = false;
        } else {
            processBlackWhite = true;
        }
    }

    /**
     * Toggle motion detection
     *  
     */
    public void toggleFindMotion() {
        if (processFindMotion) {
            processFindMotion = false;
        } else {
            processFindMotion = true;
        }
    }

    /**
     * Toggle gaussian blur processor
     *  
     */
    public void toggleGaussianBlur() {
        if (processGaussianBlur) {
            processGaussianBlur = false;
        } else {
            processGaussianBlur = true;
        }
    }

    /**
     * Get the current framecount
     *  
     */
    public int getFrameCount() {
        return frameCount;
    }

    /**
     * Reset the framecount
     *  
     */
    public void resetFrameCount() {
        frameCount = 0;
    }

    /**
     * Start the thread, the thread runs through and checks to see what
     * processing is enabled and executes the processing, then sleeps for 25ms
     *  
     */
    public void run() {
        try {
            frameCount = 0;
            while (stopThread == false) {
                frameCount++;
                sleep(25);
                BufferedImage tempImage = mainFG.getBufferedImage();
                if (processGaussianBlur) {
                    GaussianBlurFilter gb = new GaussianBlurFilter();
                    tempImage = gb.gaussianBlur(tempImage);
                }
                if (processColorDetection) {
                    tempImage = ColorDetection.colorProcessImage(tempImage);
                }
                if (processGreyScale) {
                    tempImage = ColorManipulation.grayscaleSimple(tempImage);
                }

                if (processFindEdges) {
                    tempImage = EdgeDetection.edgeDetection(tempImage,
                            edgeDetectionType);
                }

                if (processBlackWhite) {
                    tempImage = ColorManipulation.Binarization(tempImage);
                    tempImage = ColorManipulation.removeBinaryNoise(tempImage);
                }

                if (processFindFullEdges) {
                    GaussianBlurFilter gb = new GaussianBlurFilter();
                    BufferedImage processImage = gb.gaussianBlur(tempImage);
                    processImage = ColorManipulation.Binarization(processImage);
                    tempImage = EdgeDetection.drawPoints(processImage,
                            tempImage);
                }
                if (processFindMotion) {
                    sleep(99);
                    BufferedImage tempImage1 = mainFG.getBufferedImage();
                    tempImage = Manipulations.subtractImageJAI(tempImage,
                            tempImage1);

                    if (detectcount < 2) {
                        pmc.setData(tempImage);
                        detectcount++;
                    } else
                        detectcount = 0;
                }
                mainIP.setImage(tempImage);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
