package devs.visionpackage.org.cmotch.visionlibrary;

/**
 * <p>Title: ColorManipulation</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Chris Motch
 * @version 1.0
 */
import java.awt.image.*;
import java.awt.color.*;

/**
 * Color manipulation class
 */
public class ColorManipulation {

  private static final int WHITE = 0x00FFFFFF;
  private static final int BLACK = 0x00000000;

  private static final int BWHITE = 0xFF;
  private static final int BBLACK = 0x00;

  /**
   * Take the input image and output it in grayscale by adjusting the colorspace
   * @param image BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage grayscale(BufferedImage image) {

    if (image.getType() == BufferedImage.TYPE_BYTE_GRAY) {
      return image;
    }
    int imageType = BufferedImage.TYPE_BYTE_GRAY;

    int w = image.getWidth();
    int h = image.getHeight();
    BufferedImage newImg = new BufferedImage(w, h, imageType);
    ColorSpace srcSpace = image.getColorModel().getColorSpace();
    ColorSpace newSpace = newImg.getColorModel().getColorSpace();
    ColorConvertOp convert = new ColorConvertOp(srcSpace, newSpace, null);
    convert.filter(image, newImg);
    return newImg;
  }

  /**
   * Take the input image and output it in grayscale by adjusting the colorspace (More Efficent)
   * @param image BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage grayscaleSimple(BufferedImage image) {
    ColorSpace cs = ColorSpace.getInstance(ColorSpace.CS_GRAY);
    ColorConvertOp op = new ColorConvertOp(cs, null);
    image = op.filter(image, null);
    return image;
  }

  static private int threshold(int[] px) {
    int[] p = new int[256]; //	probability grayscale 8bit
    for (int i = 0; i < 256; i++) {
      p[i] = 0;
    }

    int N = px.length;
    for (int i = 0; i < N; i++) { //	build histogram
      p[px[i]] += 1; //	grayscale 8bit
    }
    int muet = 0;
    for (int k = 1; k <= 256; k++) {
      muet += k * p[k - 1];
    }
    int omega = 0;
    int mue = 0;
    int max = 0;
    double smax = 0;
    double sigma = 0;
    for (int k = 1; k <= 256; k++) {
      omega += p[k - 1];
      mue += k * p[k - 1];

      sigma = ( (double) muet * omega) / N;
      sigma -= ( (double) mue);
      sigma *= sigma;
      sigma /= (omega * (1 - ( ( (double) omega) / N)));

      if (smax < sigma) {
        smax = sigma;
        max = k - 1;
      }
    }

    if (max == 0) {
      return 160;
    }
    return max; //	threshold
  }
  /**
   * Take the input image and output it in black and white (Binarization)
   * @param image BufferedImage
   * @return BufferedImage
   */
  static public BufferedImage Binarization(BufferedImage image) {
    int w = image.getWidth();
    int h = image.getHeight();
    int size = w * h;

    BufferedImage src = grayscale(image);
    WritableRaster swr = src.getRaster();

    BufferedImage dest = new BufferedImage(w, h, BufferedImage.TYPE_BYTE_BINARY);
    WritableRaster dwr = dest.getRaster();

    int[] px = null;
    px = swr.getPixels(0, 0, w, h, px);
    int th = threshold(px); //	threshold
    int pixel;
    for (int y = 0; y < h; y++) {
      for (int x = 0; x < w; x++) {
        pixel = swr.getSample(x, y, 0);
        pixel = (pixel < th) ? BBLACK : BWHITE;
        dwr.setSample(x, y, 0, pixel);
      }
    }
    return dest;
  }

  /**
   * Remove binary noise from a binary image by checking the current pixel and the surronding pixels, if all the surronding pixels are black and the center pixel is white, it is deemed noise and set to black
   * @param inputImage1 BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage removeBinaryNoise(BufferedImage inputImage) {
    ImageManipulation image, outputImg;

    image = new ImageManipulation(inputImage);
    outputImg = new ImageManipulation(new BufferedImage(inputImage.getWidth(), inputImage.getHeight(), BufferedImage.TYPE_INT_RGB));

    for (int y = 1; y < inputImage.getHeight()-1; y++) {
      for (int x = 1; x < inputImage.getWidth()-1; x++) {

        int a = image.getbluePixel(x,y); //Center Pixel
        int b = image.getbluePixel(x-1,y-1); //NW Pixel
        int c = image.getbluePixel(x-1,y); //W Pixel
        int d = image.getbluePixel(x-1,y+1);//SW Pixel
        int e = image.getbluePixel(x,y+1);//South Pixel
        int f = image.getbluePixel(x+1,y+1);//SE Pixel
        int g = image.getbluePixel(x+1,y);//E Pixel
        int h = image.getbluePixel(x+1,y-1);//NE Pixel
        int i = image.getbluePixel(x,y-1);// N Pixel

        int p = a&(b&c&d&e&f&g&h&i);
        outputImg.setRGBPixels(x,y, p,p,p);
      }
    }
    return outputImg.getImage();
  }
}
