/**
 * <p>Title: EdgeDetection</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */


package devs.visionpackage.org.cmotch.visionlibrary;

import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.PlanarImage;



/**
 * Edge detection class
 */
public class EdgeDetection {

  public static final int SOBEL_HORIZ=0;
  public static final int SOBEL_VERT=1;
  public static final int ROBERTS_HORIZ=2;
  public static final int ROBERTS_VERT=3;
  public static final int PREWITT_HORIZ=4;
  public static final int PREWITT_VERT=5;
  public static final int FREICHEN_HORIZ=6;
  public static final int FREICHEN_VERT=7;

  private static final float[][] edgeMatrix =
    {{1.0F, 0.0F, -1.0F,
      2.0F, 0.0F, -2.0F,
      1.0F, 0.0F, -1.0F},//Sobel Horinzontal
    {-1.0F, -2.0F, -1.0F,
    0.0F, 0.0F, 0.0F,
    1.0F, 2.0F, 1.0F},//Sobel Vertical
    {0.0F, 0.0F, -1.0F,
    0.0F, 1.0F, 0.0F,
    0.0F, 0.0F, 0.0F},//Roberts Horizontal
    {-1.0F, 0.0F, 0.0F,
    0.0F, 1.0F, 0.0F,
    0.0F, 0.0F, 0.0F},//Roberts Vertical
    {1.0F, 0.0F, -1.0F,
    1.0F, 0.0F, -1.0F,
    1.0F, 0.0F, -1.0F}, //Prewitt Horizontal
    {-1.0F, -1.0F, -1.0F,
    0.0F, 0.0F, 0.0F,
    1.0F, 1.0F, 1.0F},//Prewitt Vertical
    {1.0F, 0.0F, -1.0F,
    1.414F, 0.0F, -1.414F,
    1.0F, 0.0F, -1.0F}, //Freichen Horizontal
    {-1.0F, -1.414F, -1.0F,
    0.0F, 0.0F, 0.0F,
    1.0F, 1.414F, 1.0F}}; //Freichen Vertical

  private KernelJAI[] kernels;
  private KernelJAI kern_h, kern_v;

  private static final String[] kernelLabels = {
      "Original Image",
      "Sobel",
      "Roberts",
      "Prewitt",
      "Frei-chen"
  };

  /**
   * Initialize the JAI kernels for processing
   */
  private void initKernels() {
    kernels = new KernelJAI[kernelLabels.length * 2];

    float[] normal_h_data = {
        1.0F};
    float[] normal_v_data = {
        0.0F};

    kernels[0] = new KernelJAI(1, 1, normal_h_data);
    kernels[1] = new KernelJAI(1, 1, normal_v_data);
    kernels[2] = new KernelJAI(3, 3, edgeMatrix[0]);
    kernels[3] = new KernelJAI(3, 3, edgeMatrix[1]);
    kernels[4] = new KernelJAI(3, 3, edgeMatrix[2]);
    kernels[5] = new KernelJAI(3, 3, edgeMatrix[3]);
    kernels[6] = new KernelJAI(3, 3, edgeMatrix[4]);
    kernels[7] = new KernelJAI(3, 3, edgeMatrix[5]);
    kernels[8] = new KernelJAI(3, 3, edgeMatrix[6]);
    kernels[9] = new KernelJAI(3, 3, edgeMatrix[7]);
    kern_h = kernels[0];
    kern_v = kernels[1];
  }

  /**
   * Perform the sobal detection on the input image using the JAI (Very Slow)
   * @param image0 BufferedImage
   * @return BufferedImage
   */
  public BufferedImage sobalDetectionJAI(BufferedImage image0) {
    RenderingHints hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                              RenderingHints.VALUE_ANTIALIAS_ON);
    if ( (kern_h == null) || (kern_v == null)) {
      initKernels();
    }
    kern_h = kernels[4];
    kern_v = kernels[5];
    ParameterBlock paramBlock = new ParameterBlock();
    paramBlock.addSource(PlanarImage.wrapRenderedImage((image0)));
    paramBlock.add(kern_h);
    paramBlock.add(kern_v);

    return (JAI.create("gradientmagnitude", paramBlock, hints).getAsBufferedImage());
  }

  /**
   * Perform edge detection on the input image
   * @param img BufferedImage
   * @param EdgeType int Edge Detection Type
   * @return BufferedImage
   */
  public static BufferedImage edgeDetection(BufferedImage img, int EdgeType) {
    ConvolveOp edgeOp = getEdgeOp(EdgeType);
    BufferedImage destImage = edgeOp.createCompatibleDestImage(img, img.getColorModel());
    destImage = edgeOp.filter(img, destImage);
    return destImage;
  }

  private static ConvolveOp getEdgeOp(int matrixType) {
    Kernel kernel = new Kernel(3, 3, edgeMatrix[matrixType]);
    return getConvolveOp(kernel);
  }

  private static ConvolveOp getConvolveOp(Kernel kernel) {

    RenderingHints hints =
        new RenderingHints(RenderingHints.KEY_RENDERING,
                           RenderingHints.VALUE_RENDER_QUALITY);
    ConvolveOp op = new ConvolveOp(kernel,
                                   ConvolveOp.EDGE_NO_OP,
                                   hints);
    return op;
  }

  /**
   * Draw points on an image.  Taking an image that is the result of edge detection on the original image, and then drawing those points on it.
   * @param img BufferedImage
   * @param origImg BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage drawPoints(BufferedImage img, BufferedImage origImg) {
    ImageManipulation image1, outputImg;

    image1 = new ImageManipulation(img);
    outputImg = new ImageManipulation(origImg);

    for (int i = 0; i < img.getHeight(); i++) {
      for (int j = 0; j < img.getWidth(); j++) {
        int red = image1.getredPixel(j,i);
        int green = image1.getgreenPixel(j,i);
        int blue = image1.getbluePixel(j,i);

        int sum = red+green+blue;
        int p = (sum / 3);
        if (p>0)
            outputImg.setRGBPixels(j,i, p,p,p);
      }
    }
    return outputImg.getImage();
  }
}
