package devs.visionpackage.org.cmotch.visionlibrary;

/**
 * <p>Title: Manipulations</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Chris Motch
 * @version 1.0
 */
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;

public class Manipulations {
  /**
   * Subtract two images using the JAI
   * @param image0 BufferedImage
   * @param image1 BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage subtractImageJAI(BufferedImage image0,
                                               BufferedImage image1) {
    RenderingHints hints = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
                                              RenderingHints.VALUE_ANTIALIAS_ON);

    ParameterBlock pb = new ParameterBlock();
    pb.addSource(PlanarImage.wrapRenderedImage( (image0)));
    pb.addSource(PlanarImage.wrapRenderedImage( (image1)));

    return (JAI.create("subtract", pb, hints).getAsBufferedImage());
  }
  /**
   * Subtract two image (Fast)
   * @param inputImage1 BufferedImage
   * @param inputImage2 BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage subtractImageFast(BufferedImage inputImage1, BufferedImage inputImage2) {
    ImageManipulation image1, image2, outputImg;

    image1 = new ImageManipulation(inputImage1);
    image2 = new ImageManipulation(inputImage2);
    outputImg = new ImageManipulation(new BufferedImage(inputImage1.getWidth(), inputImage1.getHeight(), BufferedImage.TYPE_INT_RGB));

    for (int i = 0; i < inputImage1.getHeight(); i++) {
      for (int j = 0; j < inputImage1.getWidth(); j++) {

        int x = image1.getbluePixel(j,i);
        int y = image2.getbluePixel(j,i);

        int diff = x - y;
        if (diff == 0){
            int p = (diff / 2) + 128;
            outputImg.setRGBPixels(j,i, 0,p,p);
        } else {
        	int p = (diff / 2) + 128;
        	outputImg.setRGBPixels(j,i, p,p,p);
        }
      }
    }
    return outputImg.getImage();
  }

  /**
   * Add two binary images together
   * @param inputImage1 BufferedImage
   * @param inputImage2 BufferedImage
   * @return BufferedImage
   */
  public static BufferedImage addImageFast(BufferedImage inputImage1, BufferedImage inputImage2) {
    ImageManipulation image1, image2, outputImg;

    image1 = new ImageManipulation(inputImage1);
    image2 = new ImageManipulation(inputImage2);
    outputImg = new ImageManipulation(new BufferedImage(inputImage1.getWidth(), inputImage1.getHeight(), BufferedImage.TYPE_INT_RGB));

    for (int i = 0; i < inputImage1.getHeight(); i++) {
      for (int j = 0; j < inputImage1.getWidth(); j++) {

        int x = image1.getbluePixel(j,i);
        int y = image2.getbluePixel(j,i);

        int diff = x | y;
            outputImg.setRGBPixels(j,i, diff,diff,diff);
      }
    }
    return outputImg.getImage();
  }

}
