package mainframe;
import java.io.*;
import java.awt.datatransfer.*;
import java.util.*;

/** This class stores all family data that we use in the example. You should
note that no error checking is done, but you probably will want to set that up
in your scheme especially if you are using beans! */
public class DeviceInfo implements Transferable, Serializable {

  final public static DataFlavor INFO_FLAVOR =
    new DataFlavor(DeviceInfo.class, "Device Information");

  static DataFlavor flavors[] = {INFO_FLAVOR };

  public static final int DEVICE = 0;
  public static final int FEATURE = 1;
  public static final int ELEMENTS = 2;
  public static final int EXPRESSION = 3;
  
  private String name = null;
  private int type = 1;
  private DeviceInfo Parent = null;
  private Vector Children = null;
  

  public DeviceInfo(String name, int type) {
    Children = new Vector();
    this.name = name;
    this.type = type;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public int getType() {
    return type;
  }

  public void setType(int type) {
    this.type = type;
  }

  public boolean isDevice() {
    return type == DeviceInfo.DEVICE;
  }

  public boolean isFeature() {
  	return type == DeviceInfo.FEATURE;
  }
  
  public boolean isExpression() {
  	return type == DeviceInfo.EXPRESSION;
  }
  
  public void add(DeviceInfo info) {
    info.setParent(this);
    Children.add(info);
  }

  public void remove(DeviceInfo info) {
    info.setParent(null);
    Children.remove(info);
  }

  public DeviceInfo getParent() {
    return Parent;
  }

  public void setParent(DeviceInfo parent) {
    Parent = parent;
  }

  public Vector getChildren() {
    return Children;
  }

  public Object clone() {
    return new DeviceInfo(name, type);
  }

  public String toString() {
    return name;
  }

  // --------- Transferable --------------

  public boolean isDataFlavorSupported(DataFlavor df) {
    return df.equals(INFO_FLAVOR);
  }

  /** implements Transferable interface */
  public Object getTransferData(DataFlavor df)
      throws UnsupportedFlavorException, IOException {
    if (df.equals(INFO_FLAVOR)) {
      return this;
    }
    else throw new UnsupportedFlavorException(df);
  }

  /** implements Transferable interface */
  public DataFlavor[] getTransferDataFlavors() {
    return flavors;
  }

  // --------- Serializable --------------

   private void writeObject(java.io.ObjectOutputStream out) throws IOException {
     out.defaultWriteObject();
   }

   private void readObject(java.io.ObjectInputStream in)
     throws IOException, ClassNotFoundException {
     in.defaultReadObject();
   }
}
