package mainframe;

/**
 * @author Martin Lukac Jan 2, 2004
 * 
 * The main class of the interface, the interface it self. All components are
 * added to listeners in order to avoid the loading of all classes at the
 * beginning. None of here visible classes are declared as threads this is
 * handled on a lower level.
 */
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.RandomAccessFile;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLayeredPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EtchedBorder;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalIconFactory;
import javax.swing.plaf.metal.MetalLookAndFeel;

import monitors.BioMonitor;
import monitors.CoLsole;
import monitors.CommCenter;
import monitors.MNSRobotPanel;
import monitors.MultiPanel;
import monitors.RythmGenerator;
import monitors.SpeechPanel;
import robot.MControl;
import robot.Robot;
import crlparser.Parser;
import devs.musicpackage.DanceGenerator;
import devs.visionpackage.org.cmotch.imageviewer.VisionWindow;

public class MainFrame extends JFrame implements ActionListener {
    /**
     * Default Constructor
     */
    public MainFrame() {
        super("Brain Controller");
        super.setFont(MainFrame.CrideIFont);
        new CommCenter();

        try {
            UIManager.put("swing.boldMetal", Boolean.FALSE);
            JDialog.setDefaultLookAndFeelDecorated(true);
            JFrame.setDefaultLookAndFeelDecorated(true);
            Toolkit.getDefaultToolkit().setDynamicLayout(true);
            System.setProperty("sun.awt.noerasebackground","true");

            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (Exception e) {
            System.out.println(e);
        }
        System.out.println("Running default theme: "+MetalLookAndFeel.getCurrentTheme().getName());;

        //Make the big window be indented 30 pixels from each edge
        //of the screen.
        int inset = 30;
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(inset, inset, screenSize.width - inset * 2, screenSize.height
                - inset * 2);
        // Set the component to show the popup menu
        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                if (evt.isPopupTrigger()) {
                    popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }
            
            public void mouseReleased(MouseEvent evt) {
                if (evt.isPopupTrigger()) {
                    popupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
                }
            }
        });
        //Set up the GUI.
        desktop = new JDesktopPane(); //a specialized
        //dragging - only outline
        desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
        //generates the mainMenubar
        setJMenuBar(createMenuBar());
        // Setup Toolbar
        setupToolBar();
        setupSystemToolBar();
    

        getContentPane().add(toolBar, BorderLayout.NORTH);
        getContentPane().add(desktop, BorderLayout.CENTER);
        //getContentPane().add(systoolBar, BorderLayout.SOUTH);
        //a class that might perhaps become a server to communicate between
        // different robots.
        CommCenter.setDesktop(desktop);
        //CommCenter.createConsole(new JPanel());
        //
        //desktop.add(CommCenter.colsole, JLayeredPane.DEFAULT_LAYER);

    }
    
    /**
     * Creates the JMenuBar
     * 
     * @return JMenuBar
     */
    private JMenuBar createMenuBar() {
        menuBar = new JMenuBar();
        commands = new Vector();
        
        JMenu menu = null;
        int counter = 0;
        File f = new File(PROPERTY_FILE);
        try {
            String line;
            RandomAccessFile fr = new RandomAccessFile(f, "r");
            while ((line = fr.readLine()) != null) {
                if (line.substring(0, 4).equalsIgnoreCase("menu")) {
                    menu = new JMenu(line.substring(5, line.length()));
                    menu.setFont(MainFrame.CrideFont);
                    menuBar.add(menu);
                } else {
                    if (line.substring(0, 4).equalsIgnoreCase("item")) {
                        if (line.indexOf("separator") != -1)
                            menu.addSeparator();
                        else
                            addJMitem(line.substring(5, line.indexOf(',')),
                                    line.substring(line.indexOf(',') + 1, line
                                            .length()), menu);
                    }
                }
            }
            popupMenu.add(menu);
        } catch (Exception e) {
            sysMessage("well the menu is bad: " + e);
        }
        return menuBar;
    }
    
    /**
     * 
     * Creates the button toolbar. Default is not floatable because on OS with
     * sloppy focus the button menu can disapear beyond its parent component
     */
    private void setupToolBar() {
        Dimension d = new Dimension(56, 18);

        JPanel panel = new JPanel(new GridLayout(3,2));
        toolBar = new JToolBar();
        toolBar.setFloatable(false);
        toolBar.setFont(MainFrame.CrideBFont);
        JButton button = new JButton("Start");
        button.setFont(MainFrame.CrideBFont);
        button.setActionCommand("scripts");
        button.addActionListener(this);

        panel.add(button);
        button = new JButton("Edit");
        button.setFont(MainFrame.CrideBFont);
        button.setActionCommand("edit");
        button.addActionListener(this);
        button.setSize(d);
        panel.add(button);

        button = new JButton("Motors");
        button.setFont(MainFrame.CrideBFont);
        button.setSize(d);
        button.setActionCommand("mcontrol");
        button.addActionListener(this);
        panel.add(button);

        button = new JButton("Speech");
        button.setFont(MainFrame.CrideBFont);
        button.setSize(d);
        button.setActionCommand("speechmenu");
        button.addActionListener(this);
        panel.add(button);

        panel.add(button);
        button = new JButton("Vision");
        button.setFont(MainFrame.CrideBFont);
        button.setSize(d);
        button.setActionCommand("visionmenu");
        button.addActionListener(this);
        panel.add(button);
        
        button = new JButton("Quit");
        button.setFont(MainFrame.CrideBFont);
        button.setActionCommand("quit");
        button.addActionListener(this);
        panel.add(button);

        JPanel main = new JPanel(new GridBagLayout());
        GridBagConstraints gc = new GridBagConstraints();
        gc.fill = GridBagConstraints.HORIZONTAL;
        gc.gridx = 0;
        gc.gridy = 0;
        gc.weightx = 0;
        gc.weighty = 10;
        main.add(panel, gc);
//        gc.fill = GridBagConstraints.HORIZONTAL;
//        gc.weightx = 100;
//        gc.weighty = 0;
//        gc.gridx = 0;
//        gc.gridy = 1;
//        main.add(new Label("----------------------"), gc);
        gc.fill = GridBagConstraints.BOTH;
        gc.weightx = 100;
        gc.weighty = 100;
        gc.gridx = 3;
        gc.gridy = 0;
        robotPanel = new JPanel(new GridBagLayout());
        robotPanel.setBorder(BorderFactory
                .createTitledBorder("Auxilliary Robot Tools"));
        main.add(robotPanel, gc);
        toolBar.add(main,BorderLayout.CENTER);
    }
 
    /**
     * 
     * Creates the button toolbar. Default is not floatable because on OS with
     * sloppy focus the button menu can disapear beyond its parent component
     */
    private void setupSystemToolBar() {

        systemPanel = new JPanel(new GridBagLayout());
        systoolBar = new JToolBar();
        systoolBar.setFloatable(false);
        systoolBar.setFont(MainFrame.CrideBFont);
        systemPanel.setBorder(BorderFactory
                .createTitledBorder("System output"));
        systoolBar.add(systemPanel,BorderLayout.CENTER);
    }

    //TODO: this should be automated but until i am not sure what commands to
    //React to menu selections.
    public void actionPerformed(ActionEvent e) {
        
        String command = e.getActionCommand();
        System.out.println(command);
        if ("new".equalsIgnoreCase(e.getActionCommand())
                || command.equalsIgnoreCase("newrobot")) { //new
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    robotSystem = new Robot();
                }
            });
            
        } else if ("edit".equalsIgnoreCase(e.getActionCommand())) {
            createFrame(new Editor());
        } else if ("saverobot".equalsIgnoreCase(e.getActionCommand())) {
            robotSystem.saveRobot();
        } else if ("quit".equalsIgnoreCase(e.getActionCommand())) {
            quit();
        } else if ("settings".equalsIgnoreCase(e.getActionCommand())) {
            //createFrame(new Configurator());
        } else if ("mtool".equalsIgnoreCase(e.getActionCommand())) {
            createFrame(new MultiPanel());
        } else if ("test".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("iView".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("devices".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("mcontrol".equalsIgnoreCase(e.getActionCommand())) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    createFrame(new MControl(robotSystem));
                }
            });
        } else if ("motormenu".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("sensormenu".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("speechmenu".equalsIgnoreCase(e.getActionCommand())) {
            if (robotSystem == null)
                CommCenter
                .showComm("To activate the voice you need to create the robot first");
            else
                createFrame(new SpeechPanel(Robot.motorsystem));
        } else if ("visionmenu".equalsIgnoreCase(e.getActionCommand())) {
            createFrame(new VisionWindow(Robot.getSensor()));
        } else if ("iview".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("scripts".equalsIgnoreCase(e.getActionCommand())) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    if (robotSystem == null) {
                        robotSystem = new Robot();
                        createFrame(new MNSRobotPanel(robotSystem, robotPanel));
                    } else {
                        CommCenter.showComm("Robot already Created, using it!");
                        createFrame(new MNSRobotPanel(robotSystem, robotPanel));
                    }
                }
            });
            //CommCenter.showComm("This is not yet implemented");
        } else if ("loadbrain".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("braincontrol".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("lpanel".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("graphs".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("biomonitor".equalsIgnoreCase(e.getActionCommand())) {
            createFrame(new BioMonitor());
        } else if ("dancegenerator".equalsIgnoreCase(e.getActionCommand())) {
            createFrame(new RythmGenerator(MNSRobotPanel.analyzer, (DanceGenerator)Robot.getSensor().getSensor("dance")));
        } else if ("cwins".equalsIgnoreCase(e.getActionCommand())) {
            cascadeF();
        } else if ("twins".equalsIgnoreCase(e.getActionCommand())) {
            tileF();
        } else if ("about".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("help".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("topics".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("crl".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        } else if ("alice".equalsIgnoreCase(e.getActionCommand())) {
            CommCenter.showComm("This is not yet implemented");
        }
        
    }
    
    /**
     * Here follows only commands of the GUI according to the ActionEvents
     * thrown by different components
     */
    
    public void sysMessage(String Message) {
        JOptionPane.showInternalMessageDialog(getContentPane(), Message);
    }
    
    /**
     * Create a new internal frame.
     * 
     * @param o -
     *            object
     */
    public void createFrame(Object o) {
        
        final JInternalFrame frame;
        if (o instanceof MIFrame) {
            frame = (MIFrame) o;
        } else if (o instanceof JPanel) {
            frame = new MIFrame((JPanel) o);
        } else if (o instanceof CoLsole) {
            frame = (MIFrame) o;
        } else
            frame = new MIFrame();
        //veto listener to prevent any internal frame being closed accidentally
        //			frame.addVetoableChangeListener(new VetoableChangeListener() {
        //				public void vetoableChange(PropertyChangeEvent event)
        //				throws PropertyVetoException {
        //					String name = event.getPropertyName();
        //					Object value = event.getNewValue();
        //					if (name.equals("closed") && value.equals(Boolean.TRUE)) {
        //						int result = JOptionPane.showInternalConfirmDialog(frame,
        //						"OK to close?");
        //						if (result != JOptionPane.YES_OPTION)
        //							throw new PropertyVetoException("Cancelled", event);
        //					}
        //				}
        //			});
        frame.setVisible(true);
        desktop.add(frame, JLayeredPane.DEFAULT_LAYER);
        try {
            frame.setSelected(true);
        } catch (java.beans.PropertyVetoException e) {
            CommCenter.showError(e);
        }
    }
    
    /**
     * @param c -
     *            container to which the popup will be attached
     */
    private void addPopupItem(Component co) {
        JMenuItem item = new JMenuItem(co.getName());
        item.setFont(MainFrame.CrideBFont);
        item.setActionCommand(((JMenuItem) co).getActionCommand());
        item.addActionListener(this);
        popupMenu.add(item);
    }
    
    /**
     * @adding a item to the only level of menu
     * @param name
     *            and command
     */
    private void addJMitem(String S, String aComm, JMenu menu) {
        if (S.toLowerCase().startsWith("separator"))
            popupMenu.addSeparator();
        else {
            JMenuItem menuItem = new JMenuItem(S);
            menuItem.setFont(MainFrame.CrideBFont);
            menuItem.setName(S);
            menuItem.setActionCommand(aComm);
            commands.addElement(aComm);
            menuItem.addActionListener(this);
            menu.add(menuItem);
            //same element in the popup menu
            addPopupItem(menuItem);
        }
    }
    
    /**
     * cascading the Frames
     */
    protected void cascadeF() {
        JInternalFrame[] frames = (JInternalFrame[]) desktop.getAllFrames();
        int x = 0;
        int y = 0;
        int width = desktop.getWidth() / 2;
        int height = desktop.getHeight() / 2;
        for (int i = 0; i < frames.length; i++) {
            if (!frames[i].isIcon()) {
                try {
                    frames[i].setMaximum(false);
                    frames[i].reshape(x, y, width, height);
                    x += f_Distance;
                    y += f_Distance;
                    if (x + width > desktop.getWidth())
                        x = 0;
                    if (y + height > desktop.getHeight())
                        y = 0;
                } catch (PropertyVetoException e) {
                    sysMessage(e.getMessage());
                }
            }
        }
    }
    
    /**
     * Tiling the Frames
     */
    protected void tileF() {
        JInternalFrame[] frames = desktop.getAllFrames();
        //no iconified frames
        int frameCount = 0;
        for (int i = 0; i < frames.length; i++) {
            if (!frames[i].isIcon())
                frameCount++;
        }
        int rows = (int) Math.sqrt(frameCount);
        int cols = frameCount / rows;
        int extra = frameCount % rows;
        int width = desktop.getWidth() / cols;
        int height = desktop.getHeight() / rows;
        int r = 0;
        int c = 0;
        for (int i = 0; i < frames.length; i++) {
            if (!frames[i].isIcon()) {
                try {
                    frames[i].setMaximum(false);
                    frames[i].reshape(c * width, r * height, width, height);
                    r++;
                    if (r == rows) {
                        r = 0;
                        c++;
                        if (c == cols - extra) {
                            rows++;
                            height = desktop.getHeight() / rows;
                        }
                    }
                } catch (PropertyVetoException e) {
                    sysMessage(e.getMessage());
                }
            }
        }
    }
    
    /**
     * Quit the application.
     */
    private void quit() {
        int result = JOptionPane.showInternalConfirmDialog(desktop,
        "Everything not saved will be lost, still OK to close?");
        if (result == JOptionPane.YES_OPTION)
            System.exit(0);
    }
    
    public static void main(String[] args) {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);
        //Create and set up the window.
        MainFrame frame = new MainFrame();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //Display the window.
        frame.setVisible(true);
        CommCenter.createConsole();
    }
    
    /**
     * inherited from MainFrameListener
     */
    public void process(MainFrameEvent Me) {
        JOptionPane.showInternalMessageDialog(desktop, Me.getBody(), Me
                .getName(), JOptionPane.INFORMATION_MESSAGE);
    }
    
    public static void showModuleMessages(Object o) {
        
    }
    
    //the Desktop Manager
    private JDesktopPane desktop;
    
    //next x position for cascading windows
    private int next_FX;
    
    //next y position for cascading windows
    private int next_FY;
    
    //distance between cascaded windows
    private int f_Distance = 30;
    
    //commands fetched from the class loader
    private Vector commands;
    
    //menuBar of this MainFrame
    private JMenuBar menuBar;
    
    //Button or the toolbar of the main menu
    private JToolBar toolBar;
    
    private JToolBar systoolBar;
    
    //The Parser - tool to synthetize the CR Language
    private Parser parser;
    
    private JPanel robotPanel;
    
    private JPanel systemPanel;
    //Until now it is only for one robot
    //later on all listeners will be migrated to 
    //the CommCenter class that will distribute 
    //events to concerned listeners
    private Robot robotSystem;
    
    //disabling the checking of the root panel
    protected boolean rootPaneCheckingEnabled = false;
    
    //file storying the data of the mainframe
    private String PROPERTY_FILE = "./mainframe/MainFrame.properties";
    
    //the static popumenu
    private final JPopupMenu popupMenu = new JPopupMenu();
    
    public static Font CrideFont = new Font("", 1,10);
    public static Font CrideNFont = new Font("", 0,10);
    public static Font CrideBFont = new Font("", 1,10);
    public static Font CrideIFont = new Font("", 2,10);
    public static Font CrideUFont = new Font("", 3,10);
    
}