package monitors;
/*
 * Created on Jan 1, 2004
 * A simple GUI interface to calibrate the robot Servos and other Motor Controlers
 * Create New commands, Loading and Saving a Robot directly from the running version of the robot
 * 
 * This graphical interface communicates directly with the MNS and 
 * should not be used as a control tool in order to avoid collision between
 * commands form MNS and fomr this Interface.
 * 
 * This interface will tech the devices from the MNS and based on their number will
 * generate the corresponding number of sliders, buttons and textfields.
 * 
 * The bottom of the interface is static and represents commands to open/load a 
 * robot, create (add) or remove (delete) commands currently installed in the robot
 * so as to save the robot to a CRL descriptor file
 */
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.BevelBorder;

import mainframe.MIFrame;
import mainframe.RobotTree;
import biocell.DynamicMood;
import biocell.GlobalState;
import biocell.ZeroMood;


/**
 * @author Martin Lukac @Jan 1, 2004
 *  
 */
public class BioMonitor extends MIFrame implements ActionListener, VetoableChangeListener {

	/**
	 * 
	 */
	public BioMonitor() 
	{
		
		super(0, 0, "Bio Monitor (Control) Panel", true, false);
		addVetoableChangeListener(this);

		super.setSize(new Dimension(600, 430));
		GridBagLayout mainGrid = new GridBagLayout();
		getContentPane().setLayout(mainGrid);
		GridBagConstraints mainC = new GridBagConstraints();
		mainC.fill = GridBagConstraints.BOTH;

		JLabel label =
			new JLabel();
		mainC.fill = GridBagConstraints.BOTH;
		mainC.anchor = GridBagConstraints.NORTH;
		add(label, mainC, 0, 0, 10, 1);
		label.setBorder(BorderFactory.createTitledBorder("BioMonitor"));

		//panel set TextArea for the data
		JPanel devPanel = new JPanel();
		GridBagConstraints devC = new GridBagConstraints();
		GridBagLayout devG = new GridBagLayout();
		devPanel.setLayout(devG);
		devPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		mainC.weightx = 100;
		mainC.weighty = 10;
		mainC.fill = GridBagConstraints.HORIZONTAL;

		Vector moods = GlobalState.getMoods();
		Vector moodnames = new Vector();
		for (int a = 0; a < moods.size(); a++)
			moodnames.add(((DynamicMood)(moods.get(a))).getname());
			
		JComboBox moodSelector = new JComboBox(moodnames);
		moodSelector.addItemListener(new ItemListener(){
			
			public void itemStateChanged(ItemEvent evt) {
				JComboBox cb = (JComboBox)evt.getSource();
				
				// Get the affected item
				Object item = evt.getItem();
				
				if (evt.getStateChange() == ItemEvent.SELECTED) {
					GlobalState.setTemplateMood((String)item);
				} else if (evt.getStateChange() == ItemEvent.DESELECTED) {
					// Item is no longer selected
				}
			}
		});

		//generate separate section  for each robot part
		add(moodSelector, mainC, 0, 1, 10, 1);

		label = new JLabel(" Monitor ");
		mainC.weighty = 0;
		mainC.fill = GridBagConstraints.HORIZONTAL;
		mainC.anchor = GridBagConstraints.NORTH;
		add(label, mainC, 0, 2, 10, 1);

		mainC.weightx = 100;
		mainC.weighty = 100;
		mainC.fill = GridBagConstraints.BOTH;
		BioDataFrame dbpanel = new BioDataFrame();
		add(dbpanel, mainC, 0, 3, 10, 7);
		
		((ZeroMood)GlobalState.getGlobalMood()).setDisplay(dbpanel);

		GridBagLayout controlGrid = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();

		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.weightx = 100;
		constraints.weighty = 0;

		label = new JLabel("Secondary Bio controls:");
		add(label, constraints, 0, 10, 10, 1);
		
		feed = new JButton("Feed");
		feed.setActionCommand("feed");
		feed.addActionListener(this);
		add(feed, constraints, 0, 11, 1, 1);

		starve = new JButton("Remove Food");
		starve.setActionCommand("starve");
		starve.addActionListener(this);
		add(starve, constraints, 1, 11, 1, 1);


		repaint();
	}

	public void vetoableChange(PropertyChangeEvent event)
		throws PropertyVetoException {
			String name = event.getPropertyName();
			Object value = event.getNewValue();
			if (name.equals("closed") && value.equals(Boolean.TRUE)) {
				if (((ZeroMood)GlobalState.getGlobalMood()).getDisplay() != null)
				    ((ZeroMood)GlobalState.getGlobalMood()).setDisplay(null);
				int result = JOptionPane.showInternalConfirmDialog(this,
				"OK to close?");
				if (result != JOptionPane.YES_OPTION)
					throw new PropertyVetoException("Cancelled", event);
			}
		}

	public void add(
		Component c,
		GridBagConstraints gbc,
		int x,
		int y,
		int w,
		int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		getContentPane().add(c, gbc);
	}

	public void add(
		Container cont,
		Component c,
		GridBagConstraints gbc,
		int x,
		int y,
		int w,
		int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		cont.add(c, gbc);
	}

	/**
	 * Creating JSlider
	 * @param min - min value
	 * @param max - max value
	 * @param name - name of the JSlider
	 * @return the JSlider
	 */
	protected JSlider createJSlider(int min, int max, String name) {

		JSlider slider = new JSlider(min, max);
		slider.setName(name);
		// Determine if currently showing ticks
		boolean b = slider.getPaintTicks(); // false

		// Show tick marks
		slider.setPaintTicks(true);

		// Set major tick marks every 25 units
		int tickSpacing = 25;
		slider.setMajorTickSpacing(tickSpacing);

		// Set minor tick marks every 5 units
		tickSpacing = 5;
		slider.setMinorTickSpacing(tickSpacing);

		slider.setPaintLabels(true);

		return slider;
	}


	private JMenuBar createMenuBar() {
		menuBar = new JMenuBar();
		JMenu menu = new JMenu("Robot");
		menuBar.add(menu);
		addJMitem("Exit", "exit", menu);

		return menuBar;
	}

	/**
	 * @adding a item to the only level of menu
	 * @param name
	 *                  and command
	 */
	private void addJMitem(String S, String aComm, JMenu menu) {
		JMenuItem menuItem = new JMenuItem(S);
		menuItem.setName(S);
		menuItem.setActionCommand(aComm);
		menuItem.addActionListener(this);
		menu.add(menuItem);
	}


	/**
	 * Implementing the ActionListener
	 */
	public void actionPerformed(ActionEvent e) {
		String name = e.getActionCommand();
		if (name.equalsIgnoreCase("exit")) {
			try {
				this.setClosed(true);
			} catch (Exception ev) {
				System.out.println(" this is not yet implemented");
			}
		} else{ 
			
		}
			
	}

	//menuBar of this MainFrame
	private JMenuBar menuBar;
	
	//the tree of the robot
	RobotTree robotTree;
	
	//the panel to show the tree
	JPanel jpane;
	
	private JButton feed, starve;
}
