package monitors;
/*
 * Created on Jan 1, 2004
 * A simple GUI interface to calibrate the robot Servos and other Motor Controlers
 * Create New commands, Loading and Saving a Robot directly from the running version of the robot
 * 
 * This graphical interface communicates directly with the MNS and 
 * should not be used as a control tool in order to avoid collision between
 * commands form MNS and fomr this Interface.
 * 
 * This interface will tech the devices from the MNS and based on their number will
 * generate the corresponding number of sliders, buttons and textfields.
 * 
 * The bottom of the interface is static and represents commands to open/load a 
 * robot, create (add) or remove (delete) commands currently installed in the robot
 * so as to save the robot to a CRL descriptor file
 */
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.border.BevelBorder;
import javax.swing.tree.DefaultMutableTreeNode;

import mainframe.MIFrame;
import mainframe.RobotTree;
import mns.MNSRobotPart;
import robot.Constructor;
import robot.Robot;


/**
 * @author Martin Lukac @Jan 1, 2004
 *  
 */
public class MControl extends MIFrame implements ActionListener {

	/**
	 * Constructor with adding the CNSListener
	 * @param robot - CNSListener
	 */
	public MControl(Robot robot) {
		super(0, 0, "Motor Control Panel", true, false);
		super.setSize(new Dimension(600, 730));
		GridBagLayout mainGrid = new GridBagLayout();
		getContentPane().setLayout(mainGrid);
		GridBagConstraints mainC = new GridBagConstraints();
		mainC.fill = GridBagConstraints.BOTH;

		JLabel label =
			new JLabel(
				"Servo Motor Controller and Calibrator",
				javax.swing.plaf.metal.MetalIconFactory.getTreeComputerIcon(),
				JLabel.CENTER);
		mainC.weightx = 100;
		mainC.weighty = 0;
		mainC.fill = GridBagConstraints.BOTH;
		mainC.anchor = GridBagConstraints.NORTH;
		add(label, mainC, 0, 0, 20, 1);
		label.setBorder(BorderFactory.createTitledBorder("Motor Control Panel"));

		

		//panel with sliders for every servo,
		//beside those each servo has one field to set the speed
		//one for the acceleration and one filed to type-in commands
		JPanel devPanel = new JPanel();
		JScrollPane scroller = new JScrollPane();
		devPanel.add("Center", scroller);
		GridBagConstraints devC = new GridBagConstraints();
		GridBagLayout devG = new GridBagLayout();
		devPanel.setLayout(devG);
		devPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		mainC.weightx = 100;
		mainC.weighty = 100;
		mainC.fill = GridBagConstraints.BOTH;
		add(devPanel, mainC, 0, 1, 10, 10);

		//generate separate section  for each robot part
		Vector devices = Robot.getController().getCnsRobotParts();
		int sections = devices.size();
		for (int a = 0; a < sections; a++) { 
			devC.gridx = 0;
			devC.gridy = a*2;
			devC.weightx = 100;
			devC.weighty = 100;
			devC.fill = GridBagConstraints.BOTH;
		    
			add(devPanel, new MNSPartPanel(((MNSRobotPart)devices.get(a))), devC, 0, a*2, 10, 2);
		    }
		
		
		JPanel treePanel = new JPanel();
		JScrollPane treescroller = new JScrollPane();
		treePanel.add("NORTH", scroller);
		devC = new GridBagConstraints();
		devG = new GridBagLayout();
		treePanel.setLayout(devG);
		treePanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		mainC.weightx = 100;
		mainC.weighty = 100;
		mainC.fill = GridBagConstraints.BOTH;
		add(treePanel, mainC, 10, 1, 10, 10);

		devC.weightx = 100;
		devC.weighty = 50;
		devC.fill = GridBagConstraints.HORIZONTAL;
		add(treePanel, new RobotTree(Constructor.constructModel(robot)), devC, 0, 0, 10, 9);
		devC.fill = GridBagConstraints.BOTH;
		devC.weightx = 100;
		devC.weighty = 100;
		
		repaint();
		}

	
	public void add(
		Component c,
		GridBagConstraints gbc,
		int x,
		int y,
		int w,
		int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		getContentPane().add(c, gbc);
	}

	public void add(
		Container cont,
		Component c,
		GridBagConstraints gbc,
		int x,
		int y,
		int w,
		int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		cont.add(c, gbc);
	}

	/**
	 * Creating JSlider
	 * @param min - min value
	 * @param max - max value
	 * @param name - name of the JSlider
	 * @return the JSlider
	 */
	protected JSlider createJSlider(int min, int max, String name) {

		JSlider slider = new JSlider(min, max);
		slider.setName(name);
		// Determine if currently showing ticks
		boolean b = slider.getPaintTicks(); // false

		// Show tick marks
		slider.setPaintTicks(true);

		// Set major tick marks every 25 units
		int tickSpacing = 25;
		slider.setMajorTickSpacing(tickSpacing);

		// Set minor tick marks every 5 units
		tickSpacing = 5;
		slider.setMinorTickSpacing(tickSpacing);

		slider.setPaintLabels(true);

		return slider;
	}


	private JMenuBar createMenuBar() {
		menuBar = new JMenuBar();
		JMenu menu = new JMenu("Robot");
		menuBar.add(menu);
		addJMitem("New", "new", menu);
		addJMitem("Load", "load", menu);
		addJMitem("Save", "save", menu);
		addJMitem("Exit", "exit", menu);

		return menuBar;
	}

	/**
	 * @adding a item to the only level of menu
	 * @param name
	 *                  and command
	 */
	private void addJMitem(String S, String aComm, JMenu menu) {
		JMenuItem menuItem = new JMenuItem(S);
		menuItem.setName(S);
		menuItem.setActionCommand(aComm);
		menuItem.addActionListener(this);
		menu.add(menuItem);
	}


	/**
	 * Implementing the ActionListener
	 */
	public void actionPerformed(ActionEvent e) {
		String name = e.getActionCommand();
		if (name.equalsIgnoreCase("exit")) {
			try {
				this.setClosed(true);
			} catch (Exception ev) {
				System.out.println(" this is not yet implemented");
			}
		} else 
			if(name.equalsIgnoreCase("new")){
				DefaultMutableTreeNode root = new DefaultMutableTreeNode("robot");
				root.add(new DefaultMutableTreeNode("Node Label"));
				//robotTree = new RobotTree(Constructor.constructModel(Robot));
				getContentPane().add(robotTree);
				getContentPane().validate();
		} else 
			if (name.equalsIgnoreCase("load")){
				System.out.println(" this is not yet implemented");
			}else if(name.equalsIgnoreCase("save")){
				System.out.println(" this is not yet implemented");
			}
	}

	//menuBar of this MainFrame
	private JMenuBar menuBar;
	
	//the tree of the robot
	RobotTree robotTree;
	
	//the panel to show the tree
	JPanel jpane;
	
}
