package monitors;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import brain.NetClient.RobotClient;

import mainframe.MIFrame;
import mns.MNSEvent;
import robot.Robot;
/*
 * Created on Jan 7, 2004
 * 
 */
/**
 * @author Martin Lukac
 *
 * This is a part of mns package
*/
public class MNSMotionCalibratorPanel extends MIFrame implements ActionListener {
	public MNSMotionCalibratorPanel(Robot robotSystem) {
		super(350, 600, "MNS - motion calibrator", true, false);
		robot = robotSystem;
		Container contentPane = getContentPane();
		GridBagLayout mainGrid = new GridBagLayout();
		contentPane.setLayout(mainGrid);
		GridBagConstraints mainC = new GridBagConstraints();
		mainC.fill = GridBagConstraints.BOTH;
		JLabel label = new JLabel("Calibrator control panel");
		label.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED));
		mainC.weightx = 50;
		mainC.weighty = 0;
		mainC.fill = GridBagConstraints.BOTH;
		mainC.anchor = GridBagConstraints.NORTH;
		add(label, mainC, 0, 0, 4, 1);
		GridBagLayout controlGrid = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		mainC.weightx = 100;
		mainC.weighty = 100;
		JPanel outputPanel = new JPanel(controlGrid, false);
		outputPanel.setBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.RAISED));
		add(outputPanel, mainC, 0, 1, 4, 2);
		controlGrid = new GridBagLayout();
		constraints = new GridBagConstraints();
		mainC.weightx = 100;
		mainC.weighty = 100;
		JPanel inputPanel = new JPanel(controlGrid, false);
		inputPanel.setBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.RAISED));
		add(inputPanel, mainC, 0, 5, 4, 2);
		controlGrid = new GridBagLayout();
		constraints = new GridBagConstraints();
		mainC.weightx = 100;
		mainC.weighty = 100;
		JPanel buttonPanel = new JPanel(controlGrid, false);
		buttonPanel.setBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.RAISED));
		add(buttonPanel, mainC, 0, 8, 4, 3);
		resultArea = new JTextArea();
		resultArea.setColumns(20);
		resultArea.setMaximumSize(new Dimension(50, 50));
		resultArea.setLineWrap(true);
		resultArea.setEditable(true);
		resultArea.setBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.RAISED));
		constraints.fill = GridBagConstraints.BOTH;
		constraints.weightx = 100;
		constraints.weighty = 100;
		add(new JScrollPane(resultArea), constraints, 0, 1, 1, 1, outputPanel);
		constraints = new GridBagConstraints();
		constraints.weightx = 0;
		constraints.weighty = 0;
		add(new JLabel("Input:"), constraints, 0, 2, 2, 1, inputPanel);
		inputField = new JTextField(20);
		add(inputField, constraints, 2, 2, 2, 1, inputPanel);
		controlGrid = new GridBagLayout();
		constraints = new GridBagConstraints();
		constraints.weightx = 100;
		constraints.weighty = 100;
		constraints.fill = GridBagConstraints.BOTH;
		robotCreator = new JButton("Create Robot");
		robotCreator.setActionCommand("runCreat");
		robotCreator.addActionListener(this);
		add(robotCreator, constraints, 0, 1, 2, 1, buttonPanel);
		rFileLoader = new JButton("Load Descriptor");
		rFileLoader.setActionCommand("loadRobot");
		rFileLoader.addActionListener(this);
		add(rFileLoader, constraints, 2, 1, 1, 1, buttonPanel);
		robotLoader = new JButton("Construct Robot");
		robotLoader.setActionCommand("runConstr");
		robotLoader.addActionListener(this);
		add(robotLoader, constraints, 3, 1, 1, 1, buttonPanel);
		//obot.addMNSMonitor(this);
	}
	/**
	 * @param c
	 * @param gbc
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 */
	public void add(Component c, GridBagConstraints gbc, int x, int y, int w,
			int h) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		getContentPane().add(c, gbc);
	}
	/**
	 * @param c
	 * @param gbc
	 * @param x
	 * @param y
	 * @param w
	 * @param h
	 * @param destination
	 */
	public void add(Component c, GridBagConstraints gbc, int x, int y, int w,
			int h, JPanel destination) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		destination.add(c, gbc);
	}
	/**
	 * @param message
	 */
	public void showMessage(String message) {
		resultArea.insert(message + '\n', 0);
	}
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String comm = e.getActionCommand();
		if (comm.equalsIgnoreCase(robotCreator.getActionCommand())) {
			showMessage("Detecting and Initiating hardware");
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					robot.wake();
				}
			});
		} else if (comm.equalsIgnoreCase(robotLoader.getActionCommand())) {
		} else if (comm.equalsIgnoreCase(scriptLoader.getActionCommand())) {
		} else if (comm.equalsIgnoreCase(sFileLoader.getActionCommand())) {
		} else if (comm.equalsIgnoreCase(rFileLoader.getActionCommand())) {
		} else if (comm.equalsIgnoreCase(scriptClear.getActionCommand())) {
			robot.stopScript();
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					netclient.sendMessage();
				}
			});
		}
	}
	/**
	 * @param mnEvent
	 */
	public void process(MNSEvent mnEvent) {
		if (mnEvent.getData() == "DONE") {
			netclient.sendClientEvent("DONE");
		}
	}
	//the listener to the robot
	private Robot robot;
	private RobotClient netclient;
	private JButton rFileLoader, robotLoader, robotCreator, sFileLoader,
			scriptLoader, scriptClear, netconect;
	private JTextArea resultArea;
	public JTextField inputField;
	protected FileDialog fileDialog;
}