/*******************************************************************************
 * Martin Lukac, 2003-2005 * Intelligent Robotics Laboratory * Portland State
 * University * Office: 70-08 FAB, phone: 503-490-8478 * Mail: P.O. Box 751 *
 * Portland, OR 97207-0751 * Email: lukacm@cecs.pdx.edu * Updates and
 * documentation: www.pdx.edu\~lukacm *
 ******************************************************************************/
//***********************************************************************
// 	This program is provided WITHOUT ANY WARRANTY either expressed
//	or implied. You may study, use, modify, and distribute it for
//	any non-commercial purposes. For any commercial use, prior
//	permission from the authors must be granted.
// **********************************************************************
/*
 * Created on Jul 5, 2004
 *  
 */

package monitors;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mns.MNSEvent;
import mns.MNSRobotPart;

public class MNSPartPanel extends JPanel implements ActionListener,
        ChangeListener, ItemListener {

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    public void itemStateChanged(ItemEvent e) {
        JCheckBox source = (JCheckBox) e.getSource();
        if (e.getStateChange() == ItemEvent.DESELECTED) {
            String device = source.getName();
            int devnumber = Integer.parseInt(device.substring(device
                    .indexOf(' ') + 1));
            if (device.startsWith("j")) {
                join[devnumber] = false;
            } else if (device.startsWith("i")) {
                inv[devnumber] = false;
            }
        } else if (e.getStateChange() == ItemEvent.SELECTED) {
            String device = source.getName();
            int devnumber = Integer.parseInt(device.substring(device
                    .indexOf(' ') + 1));
            if (device.startsWith("j")) {
                join[devnumber] = true;
            } else if (device.startsWith("i")) {
                inv[devnumber] = true;
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider) e.getSource();
        if (!source.getValueIsAdjusting()) {
            int fps = (int) source.getValue();
            String device = source.getName();
            int devnumber = Integer.parseInt(device.substring(device
                    .lastIndexOf('e')+1));
           if (!join[devnumber])
                if (inv[devnumber])
                    part.getDevice(devnumber).load(
                            new MNSEvent(Integer.toString(100 - fps)));
                else
                    part.getDevice(devnumber).load(
                            new MNSEvent(Integer.toString(fps)));
            else
            {
                int devCounter;
                int devMAx = part.getDevices().length;
                if (part.getname().trim().equalsIgnoreCase("mouth")) {
                    devCounter = 1;
                } else
                    devCounter = 0;
                
                for (int a = devCounter; a < devMAx; a++){
                    if (join[a])
	                    if (inv[a])
	                        part.getDevice(a).load(
	                                new MNSEvent(Integer.toString(100 - fps)));
	                    else
	                        part.getDevice(a).load(
	                                new MNSEvent(Integer.toString(fps)));

                }
            }
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        String actioncommand = e.getActionCommand();
        int device = Integer.parseInt(actioncommand.substring(actioncommand
                .indexOf(' ') + 1));
        actioncommand = actioncommand.substring(0, actioncommand.indexOf(' '))
                .trim();
        System.out.println(":" + actioncommand + ":" + device + ":");
        int gain = 0;
        try {
            if (actioncommand.equalsIgnoreCase("accl")) {
                System.out.println(accl[device].getText());
                gain = Integer.parseInt(accl[device].getText());
                part.setPartAccel(gain, device);
            } else if (actioncommand.equalsIgnoreCase("speed")) {
                System.out.println(speed[device].getText());
                gain = Integer.parseInt(speed[device].getText());
                part.setPartSpeed(gain, device);
            } else if (actioncommand.equalsIgnoreCase("max")) {
                System.out.println(max[device].getText());
                gain = Integer.parseInt(max[device].getText());
                part.setPartMax(gain, device);
            } else if (actioncommand.equalsIgnoreCase("min")) {
                System.out.println(min[device].getText());
                gain = Integer.parseInt(min[device].getText());
                part.setPartMin(gain, device);
            }
        } catch (NumberFormatException et) {

        }

    }

    /**
     * @param part
     *            -create the control panel using the given robot part
     */
    public MNSPartPanel(MNSRobotPart part) {
        super();
        System.out.println(part.getname().trim() + "  creating ");

        this.part = part;
        GridBagConstraints gbc = new GridBagConstraints();
        GridBagLayout gbl = new GridBagLayout();
        setLayout(gbl);
        gbc.fill = GridBagConstraints.BOTH;
        setBorder(BorderFactory
                .createTitledBorder(part.getname().toUpperCase()));

        int devCounter;
        int devMAx = part.getDevices().length;
        if (part.getname().trim().equalsIgnoreCase("mouth")) {
            devCounter = 1;
            //set the controls for more complex commands
            inv = new boolean[devMAx - 0];
            join = new boolean[devMAx - 0];

        } else{
            devCounter = 0;
            //set the controls for more complex commands
            inv = new boolean[devMAx - devCounter];
            join = new boolean[devMAx - devCounter];

        }
        
        collects = new JTextField[devMAx - devCounter];

        accl = new JTextField[devMAx];
        speed = new JTextField[devMAx];
        min = new JTextField[devMAx];
        max = new JTextField[devMAx];

        acclb = new JButton[devMAx];
        speedb = new JButton[devMAx];
        minb = new JButton[devMAx];
        maxb = new JButton[devMAx];


        for (int n = devCounter; n < devMAx; n++) {
            
            JPanel boxPane = new JPanel();
            JCheckBox c = new JCheckBox("join " + n);
            c.setName("join " + n);
            c.setSelected(false);
            c.addItemListener(this);
            boxPane.add(c);
            c = new JCheckBox("inv " + n);
            c.setName("inv " + n);
            c.setSelected(false);
            c.addItemListener(this);
            boxPane.add(c);

            JSlider slider = createJSlider(0, 100, "Device" + n);
            slider.addChangeListener(this);//new ChangeListener() {

            JLabel plabel = new JLabel("BioDevice-" + n, JLabel.LEFT);

            accl[n] = new JTextField("Acceleration" + n);
            accl[n].setText(Integer.toString(part.getPartAccel(n)));
            System.out.println(part.getname().trim() + "  " + part.getPartMin(n));
            JLabel alabel = new JLabel("Accel " + n, JLabel.LEFT);
            alabel.setBorder(BorderFactory
                    .createEtchedBorder(EtchedBorder.RAISED));
            acclb[n] = new JButton("accl " + n);
            acclb[n].addActionListener(this);

            speed[n] = new JTextField("Speed" + n);
            speed[n].setText(Integer.toString(part.getPartSpeed(n)));
            System.out.println(part.getname().trim() + "  " + part.getPartMin(n));
            JLabel slabel = new JLabel("Speed " + n, JLabel.LEFT);
            slabel.setBorder(BorderFactory
                    .createEtchedBorder(EtchedBorder.RAISED));
            speedb[n] = new JButton("speed " + n);
            speedb[n].addActionListener(this);

            max[n] = new JTextField("Maximum" + n);
            max[n].setText(Integer.toString(part.getPartMax(n)));
            System.out.println(part.getname().trim() + "  " + part.getPartMin(n));
            JLabel mlabel = new JLabel("Max " + n, JLabel.LEFT);
            mlabel.setBorder(BorderFactory
                    .createEtchedBorder(EtchedBorder.RAISED));
            maxb[n] = new JButton("max " + n);
            maxb[n].addActionListener(this);

            min[n] = new JTextField("Minimum" + n);
            min[n].setText(Integer.toString(part.getPartMin(n)));
            System.out.println(part.getname().trim() + "  " + part.getPartMin(n));
            JLabel ilabel = new JLabel("Min " + n, JLabel.LEFT);
            ilabel.setBorder(BorderFactory
                    .createEtchedBorder(EtchedBorder.RAISED));
            minb[n] = new JButton("min " + n);
            minb[n].addActionListener(this);

            int g = n * 8;
            add(plabel, gbc, 0, g, 1, 1);
            add(boxPane, gbc, 1, g, 1, 1);
            add(slider, gbc, 0, g + 2, 2, 6);

            add(alabel, gbc, 2, g + 2, 2, 1);
            add(accl[n], gbc, 2, g + 3, 2, 1);
            add(acclb[n], gbc, 2, g + 4, 2, 1);

            add(slabel, gbc, 4, g + 2, 2, 1);
            add(speed[n], gbc, 4, g + 3, 2, 1);
            add(speedb[n], gbc, 4, g + 4, 2, 1);

            add(mlabel, gbc, 6, g + 2, 2, 1);
            add(max[n], gbc, 6, g + 3, 2, 1);
            add(maxb[n], gbc, 6, g + 4, 2, 1);

            add(ilabel, gbc, 8, g + 2, 2, 1);
            add(min[n], gbc, 8, g + 3, 2, 1);
            add(minb[n], gbc, 8, g + 4, 2, 1);

            repaint();
        }
    }

    /**
     * @param c
     * @param gbc
     * @param x
     * @param y
     * @param w
     * @param h
     */
    public void add(Component c, GridBagConstraints gbc, int x, int y, int w,
            int h) {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        add(c, gbc);
    }

    /**
     * @param cont
     * @param c
     * @param gbc
     * @param x
     * @param y
     * @param w
     * @param h
     */
    public void add(Container cont, Component c, GridBagConstraints gbc, int x,
            int y, int w, int h) {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = w;
        gbc.gridheight = h;
        cont.add(c, gbc);
    }

    /**
     * Creating JSlider
     * 
     * @param min -
     *            min value
     * @param max -
     *            max value
     * @param name -
     *            name of the JSlider
     * @return the JSlider
     */
    protected JSlider createJSlider(int min, int max, String name) {

        JSlider slider = new JSlider(min, max);
        slider.setName(name);
        // Determine if currently showing ticks
        boolean b = slider.getPaintTicks(); // false

        // Show tick marks
        slider.setPaintTicks(true);

        // Set major tick marks every 25 units
        int tickSpacing = 25;
        slider.setMajorTickSpacing(tickSpacing);

        // Set minor tick marks every 5 units
        tickSpacing = 5;
        slider.setMinorTickSpacing(tickSpacing);

        slider.setPaintLabels(true);

        return slider;
    }

    private MNSRobotPart part;

    private JTextField[] collects, accl, speed, min, max;

    private JButton[] acclb, speedb, minb, maxb;

    private boolean[] inv;

    private boolean[] join;
}