/*
 * Light Development Playlist Editor
 * Copyright (C) 2004 Ulrich Hilger
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package monitors;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;

import mainframe.MIFrame;
import nns.NNS;

import com.lightdev.app.playlisteditor.ui.JPlayerDisplay;

import devs.musicpackage.DanceGenerator;
import devs.musicpackage.PlayItemAction;
import devs.musicpackage.SpectrumAnalyzer;
import devs.musicpackage.StopPlaybackAction;
import devs.musicpackage.TogglePauseAction;
/**
 * main class of application Playlist Editor.
 * 
 * @author Ulrich Hilger
 * @author Light Development
 * @author <a href="http://www.lightdev.com">http://www.lightdev.com </a>
 * @author <a href="mailto:info@lightdev.com">info@lightdev.com </a>
 * @author published under the terms and conditions of the GNU General Public
 *         License, for details see file gpl.txt in the distribution package of
 *         this software as well as any licensing notes inside this
 *         documentation
 * 
 * @version 3, January 3, 2005
 */

public class RythmGenerator extends MIFrame implements ActionListener,
        MouseListener {

    /**
     * constructor
     */
    public RythmGenerator(SpectrumAnalyzer analyzer, DanceGenerator dGenerator) {
        super(500, 300, "Dance generator", true, true);
        this.dGenerator = dGenerator; 
        buildGui(analyzer);
    }
    
    private DanceGenerator dGenerator;

    /**
     * set the directory to look for audio files
     * 
     * @param musicDir
     *            String the music directory to set
     */
    public void setMusicDirectory(String musicDir) {

    }

    /**
     * build the user interface of this application
     */
    private void buildGui(SpectrumAnalyzer analyzer) {

        
		listArea = new JTextArea("Songs");
		listArea.setLineWrap(true);
		listArea.setEditable(false);
		listArea.addMouseListener(this);
		listArea.setFocusable(true);

        // panel to hold playlist and play statistics panel
        JScrollPane scrollPane = new JScrollPane(listArea);
        scrollPane.setBorder(BorderFactory.createTitledBorder("Available Songs"));
        scrollPane.getViewport().setBackground(Color.WHITE);
        GridBagConstraints gbc = new GridBagConstraints();
        GridBagLayout gbl = new GridBagLayout();
        JPanel listPanel = new JPanel(gbl);
        gbc.fill = GridBagConstraints.BOTH	;
        gbc.weightx = 100;
        gbc.weighty = 100;

        addp(scrollPane, gbc, 0, 0, 10, 10, listPanel);

        // create a panel to show information about the currently played song
        displayPanel = new JPlayerDisplay();
        //displayPanel.setOpaque(false);
        displayPanel.addMouseListener(this);

        JButton btn = new JButton("load");
        btn.setFont(new Font("Default", 1, 10));
            JPanel ButtonPanel = new JPanel();
        Dimension btnSize = new Dimension(100, 21);
        btn.setActionCommand("load");
        btn.setText("Load");
        btn.setPreferredSize(btnSize);
        btn.addActionListener(this);
        ButtonPanel.add(btn);
        addp(ButtonPanel, gbc, 10, 0, 3, 10, listPanel);

        // panel for player buttons
        JPanel playerButtonPanel = new JPanel();
        btn = new JButton(new TogglePauseAction(dGenerator));
        btn.setActionCommand("pause");
        btn.setFont(new Font("Default", 1, 10));
        btn.setText("Pause");
        btn.setPreferredSize(btnSize);
        //btn.addActionListener(dGenerator);
        playerButtonPanel.add(btn);

        playPauseBtn = new JButton(new PlayItemAction(dGenerator));
        btn.setActionCommand("play");
        btn.setFont(new Font("Default", 1, 10));
        playPauseBtn.setText("Play");
        playPauseBtn.setPreferredSize(btnSize);
        playerButtonPanel.add(playPauseBtn);

        btn = new JButton();
        btn.setActionCommand("prev");
        btn.setFont(new Font("Default", 1, 10));
        btn.setText("Rew");
        btn.setPreferredSize(btnSize);
        playerButtonPanel.add(btn);

        btn = new JButton(new StopPlaybackAction(dGenerator));
        btn.setActionCommand("stop");
        btn.setFont(new Font("Default", 1, 10));
        btn.setText("Stop");
        btn.setPreferredSize(btnSize);
        //btn.addActionListener(dGenerator);
        playerButtonPanel.add(btn);

        // add contorls to the playpanel
        JPanel playerPanel = new JPanel(new BorderLayout());
        playerPanel.add(playerButtonPanel, BorderLayout.CENTER);

        // create a panel to show information about the currently played song
        displayPanel = new JPlayerDisplay();
        //displayPanel.setOpaque(false);
        displayPanel.addMouseListener(this);

        // create a spectrum analyzer
//        analyzer.addMouseListener(this);
//        analyzer.setVisible(false);
 //       dGenerator.setAnalyzer(analyzer);
 //       dGenerator.getSynchronizer().add(analyzer);

        // create a helper panel for the southern part of the right split pane
        // component
        southPanel = new JPanel(new BorderLayout());
        //southPanel.add(analyzer, BorderLayout.CENTER);
        southPanel.add(displayPanel, BorderLayout.CENTER);
        southPanel.add(playerPanel, BorderLayout.SOUTH);

        // get main frame content pane and set layout
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());

        // add to main frame
        // contentPane.add(statusPanel, BorderLayout.NORTH);
        contentPane.add(listPanel, BorderLayout.CENTER);
        contentPane.add(southPanel, BorderLayout.SOUTH);

    }

	public void addp(Component c, GridBagConstraints gbc, int x, int y, int w,
			int h, JPanel destination) {
		gbc.gridx = x;
		gbc.gridy = y;
		gbc.gridwidth = w;
		gbc.gridheight = h;
		destination.add(c, gbc);
	}

    /**
     * increase or decrease the volume by a given value
     * 
     * @param change
     *            int the value to increase (+ sign) or decrease (- sign) the
     *            volume by
     */
    public void changeVolume(int change) {
        int value = volume.getValue() + change;
        int min = volume.getMinimum();
        int max = volume.getMaximum();
        if (value <= min) {
            volume.setValue(min);
        } else if (value >= max) {
            volume.setValue(max);
        } else {
            volume.setValue(value);
        }
    }



    /* --------------------- MouseListener start ---------------------- */

    /**
     * Invoked when the mouse button has been clicked (pressed and released) on
     * a component.
     * 
     * <p>
     * Part of the set of MouseListener interface methods but unused here
     * </p>
     * 
     * @param e
     *            MouseEvent
     */
    public void mouseClicked(MouseEvent e) {
//        Object source = e.getSource();
//        if (source.equals(dGenerator.getAnalyzer())) {
//            southPanel.add(displayPanel, BorderLayout.NORTH);
//            dGenerator.getAnalyzer().setVisible(false);
//            displayPanel.setVisible(true);
//        } else if (source.equals(displayPanel)) {
//            southPanel.add(dGenerator.getAnalyzer(), BorderLayout.NORTH);
//            dGenerator.getAnalyzer().setSize(displayPanel.getWidth()-20, displayPanel.getHeight()-20);
//            dGenerator.getAnalyzer().setVisible(true);
//            displayPanel.setVisible(false);
//        }
    }

    /**
     * Invoked when a mouse button has been pressed on a component.
     * 
     * <p>
     * Used to detect whether or not the user pressed the mouse on the song
     * slider during a playing song thus initiating a change of the current play
     * position
     * </p>
     * 
     * @param e
     *            MouseEvent
     */
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Invoked when a mouse button has been released on a component.
     * 
     * <p>
     * Used to detect whether or not the user released the mouse button during a
     * manual move of the song slider indicating the position the player shall
     * jump to during play. This triggers the actual reposition action.
     * </p>
     * 
     * @param e
     *            MouseEvent
     */
    public void mouseReleased(MouseEvent e) {
        //Application.log("mouseReleased", getClass(), Level.FINEST);
        Object source = e.getSource();
        if (source.equals(listArea) && listArea.getSelectedText() != null){
            if (listArea.getSelectedText().endsWith("mp3")) {
                String song = listArea.getSelectedText();
                String list = listArea.getText();
                StringTokenizer tokenizer = new StringTokenizer(list, "\n");
                while(tokenizer.hasMoreTokens()){
                    String s = tokenizer.nextToken();
                    if (s.length() > 5 && s.indexOf(song) >= 0){
                        listArea.append("\nCurrent Song: "+s);
                        dGenerator.setCurrentsong(currentFileOpener.getDescriptor()+s);
                        break;
                    }
                }
            }
        }
    }

    /**
     * Invoked when the mouse enters a component.
     * 
     * <p>
     * Part of the set of MouseListener interface methods but unused here
     * </p>
     * 
     * @param e
     *            MouseEvent
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Invoked when the mouse exits a component.
     * 
     * <p>
     * Part of the set of MouseListener interface methods but unused here
     * </p>
     * 
     * @param e
     *            MouseEvent
     */
    public void mouseExited(MouseEvent e) {
    }

    /* --------------------- MouseListener end ---------------------- */

    public void actionPerformed(ActionEvent e) {
        if(e.getActionCommand() == "load"){
            currentFileOpener = new OpenFile(listArea, "/home/nabhoth/workspace/CoL/rbtrsc/");
        }
    }

    /* --------------------- class fields start ---------------------- */

    private OpenFile currentFileOpener;
    
    private JTextArea listArea;

    /** play/pause button */
    private JButton playPauseBtn;

    /** slider to change the volume */
    private JSlider volume;

    /** player display */
    private JPlayerDisplay displayPanel;

    /** label indicating the current status */
    private JLabel statusMsg;

    /** helper panel for the lower right part of the main window */
    private JPanel southPanel;

    private String currentSong;

    private FileDialog fileDialog;

    /* --------------------- class fields end ---------------------- */

    class OpenFile {

        public OpenFile(JTextArea destin, String destination) {
            this.destin = destin;
            Frame frame = null;
            for (Container p = getParent(); p != null; p = p.getParent()) {
                if (p instanceof Frame) {
                    frame = (Frame) p;
                    break;
                }
            }

            if (fileDialog == null) {
                fileDialog = new FileDialog(frame);
                fileDialog.setSize(700, 500);
            }
            fileDialog.setMode(FileDialog.LOAD);
            fileDialog.setVisible(true);
            String directory = fileDialog.getDirectory();
            this.destination = directory;
            destin.setText("");
            visitAllFiles(new File(directory));
        }

        // Process only files under dir
        private  void visitAllFiles(File dir) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                File file = new File(dir, children[i]);
                if (!(file.isDirectory()) && file.getName().endsWith("mp3"))
                    destin.append(file.getName()+"\n");
            }
        }

        public String getDescriptor() {

            return destination;
        }

        private File currentFile;

        private JTextArea destin;

        private String destination;
    }

}
