/*
 * Created on Jun 28, 2004
 *
 */
package mns;

import java.util.Vector;

import mainframe.Tools;
import monitors.CommCenter;
import biocell.EnergyPrimitive;

/**
 * @author Martin Lukac
 * 
 * For specific purposes a Threaded command is an internal class allowing to run
 * commands on one self and generate some regular, cyclic, time-variant
 * behavior. This command thread represents standard servo and control response
 * sequence. THe relation between the different motor or control values can be
 * specified and can generate potentially nice results Thread of this type is
 * automatically: SINGLE_ACT and RANDOM_VALUE_MODE - it will replace any missing
 * default values by random values with current settings
 */
class CommandThread extends Command implements BioDeviceListener {

    /*
     * (non-Javadoc)
     * 
     * @see biocell.BioEnergySpender#energyConsumed(biocell.EnergyPrimitive)
     */
    public void energyConsumed(EnergyPrimitive ep) {
        // TODO Auto-generated method stub

    }

    /**
     * @param name
     */
    public CommandThread(String name) {
        super(name);
        initiate();
    }

    public CommandThread(String[] data) {
        super(data[0]);
        initiate();
        Vector comms = new Vector();
        // Generating simple command
        for (int a = 1; a < data.length; a++)
            comms.add(data[a]);

        this.setCommand(comms.toArray());
    }

    public CommandThread(String name, char key) {
        super(name);
        initiate();
        Vector comms = new Vector();
        comms.add("null");
        this.setCommand(comms.toArray());
        setKey(key);
        initiate();
    }

    public CommandThread(String[] data, char key) {
        super(data[0]);
        initiate();
        setKey(key);
        Vector comms = new Vector();
        // Generating simple command
        for (int a = 1; a < data.length; a++)
            comms.add(data[a]);

        this.setCommand(comms.toArray());
    }

    /**
     * some commands can be coordinated through events such as speech or some
     * other cyclic or random movements this serves to time coordinating them
     * 
     * @param bdE -
     *            caught BioDeviceEvent
     */
    synchronized public void bioEvent(BioDeviceEvent bdE) {
        switch (bdE.getState()) {
        case BioDevice.CYCLING: {
            CommCenter.showDebug(this.getPart().getname()
                    + " started executing");
            break;
        }
        case BioDevice.DONE: {
            setAssignDevices(this.getAssignDevices() - 1);
            if (this.getAssignDevices() <= 0)
                setDone(true);
            CommCenter.showDebug("State ended "+this.getAssignDevices());
            break;
        }
        case BioDevice.BUSY: {
            CommCenter.showDebug("State started");
            break;
        }
        case BioDevice.END: {
            CommCenter.showDebug("State started");
            break;
        }
        default: {
            CommCenter.showDebug("state not found");
            break;
        }
        }

        notifyAll();
    }

    /**
     * private method for adding this to the biodevices as a listener
     * 
     */
    protected void addListener() {
        for (int a = 0; a < getPart().getDevices().length; a++)
            if (getPart().getDevice(a) != null) {
                getPart().getDevice(a).addListener(this);
                CommCenter.showDebug("Listener added");
            }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run() implemented Thread run() - the command is
     *      processed on all but on the first device which is the speech device.
     *      The commandThread is a simple one way type instruction for motion
     *      control
     */
    public void run() {
        try {
            synchronized (this) {
                CommCenter.showDebug("new commandthread");
                addListener();
                setDone(false);
                // acknoledge the robot part of activity
                if (getParentThread() != null)
                    getParentThread().bioEvent(
                            new BioDeviceEvent(this, BioDevice.BUSY));
                else
                    getPart().stateChanged(BioDevice.BUSY);
                Object[] comand = getInstructions();

                // execute single command with optional settings
                // send instructions to all devices
                // Speach commands are treated separatelly!!!
                String comands = (String) comand[0];
                String[] data = Tools.tokenize(comands);
                int diff = data.length - this.getAssignDevices();
                int datal = data.length;
                //fetch the data to the amount of processing devices
                if (diff < 0){
                    datal = data.length;
                    this.setAssignDevices(datal);
                }

                for (int a = 0; a < datal; a++) {
                    try {
                        if (getKey() == '0') {
                            if (this.getPart().getname().toLowerCase()
                                    .startsWith("mouth")) {
                                getPart().getDevice(1).load(
                                        new MNSEvent(data[a]));
                            } else {
                                getPart().getDevice(a).load(
                                        new MNSEvent(data[a]));
                            }
                            CommCenter.showDebug("Data loaded");
                        } else {
                            String[] com = new String[2];
                            com[0] = getKey() + "";
                            com[1] = data[a];
//                            if (getKey() == 'v')
//                                getPart().getDevice(0).loadCNTR(
//                                        new MNSEvent(com));
//                            else
                                getPart().getDevice(a).loadCNTR(
                                        new MNSEvent(com));
                                CommCenter.showDebug("Executing key command: "+com+" on "+a);
                        }
                    } catch (NumberFormatException e) {
                        e.printStackTrace();
                    }
                }
                CommCenter.showDebug("Data completely loaded");

                while (!isDone()) {
                    try {
                        wait(100);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
                CommCenter.showDebug("" + isDone());
                CommCenter.showDebug("CNSRobotPart" + getPart().getname()
                        + " processing:  " + getName() + " is done");

                // free the robot part ressources
                if (getParentThread() != null)
                    getParentThread().bioEvent(
                            new BioDeviceEvent(this, BioDevice.DONE));
                else
                    getPart().stateChanged(BioDevice.DONE);
                CommCenter.showDebug("command thread terminated");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @return Returns the parentThread.
     */
    public synchronized Command getParentThread() {
        return parentThread;
    }

    /**
     * @param parentThread
     *            The parentThread to set.
     */
    public synchronized void setParentThread(Command parentThread) {
        this.parentThread = parentThread;
    }

    /**
     * 
     */
    private void initiate() {
        this.setExecMode(RANDOM_VALUE_MODE);
        this.setExecType(SINGLE_ACT);

    }

    private Command parentThread = null;
}