/*
 * Created on Jul 8, 2004
 *
 */
package mns;

import java.util.Iterator;
import java.util.Vector;

import monitors.CommCenter;
import biocell.EnergyPrimitive;

/**
 * @author Martin Lukac
 * 
 * This is a part of mns package This is a multi command instruction command. It
 * is nominal (accept no parmeters) and executes the stored commands serially
 * and exits.
 */
public class ComplexCommand extends Command implements Runnable {


	/**
	 * @param name
	 * @param commands
	 */
	public ComplexCommand(String name, Vector commands) {
		super(name);
		coms = new Vector(commands);
		Vector outp = new Vector();
		for (int a = 0; a < coms.size(); a++){
		    Command c = (CommandThread)coms.get(a);
		    String d = c.getName()+" ";
		    Object [] com = c.getCommand();
		    for (int t = 0; t< com.length; t++)
		        d = d + (String)com[t];
		    outp.add(d);
		}
		this.setCommand(outp.toArray());
	}

	/**
	 * some commands can be coordinated through events such as speech or some
	 * other cyclic or random movements this serves to time coordinating them
	 * 
	 * @param bdE -
	 *            caught BioDeviceEvent
	 */
	synchronized public void bioEvent(BioDeviceEvent bdE) {
		switch (bdE.getState()) {
		case BioDevice.CYCLING: {
			CommCenter.showDebug("State started");
			break;
		}
		case BioDevice.DONE: {
				setDone(true);
			CommCenter.showDebug("State done");
			break;
		}
		case BioDevice.BUSY: {
			CommCenter.showDebug("State busy");
			break;
		}
		case BioDevice.END: {
		    setDone(true);
			CommCenter.showDebug("State over");
			break;
		}
		default: {
			CommCenter.showDebug("state not found");
			break;
		}
		}

		notifyAll();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run() implemented Thread run() - the command is
	 *      processed on all but on the first device which is the speech device.
	 *      The commandThread is a simple one way type instruction for motion
	 *      control
	 */
	public void run() {
		try {
            this.setDone(false);
			//synchronized (this) {
			Iterator it = coms.iterator();
			while(it.hasNext()){
			    Command thread = ((CommandThread)it.next()).cloneCommand();
			    //System.out.println("complex command processing: "+thread.getBioname());
			    ((CommandThread)thread).setParentThread(this);
			    ((CommandThread)thread).setPart(this.getPart());
			    ((CommandThread)thread).start();
			    
			    while(!isDone()){
			        sleep(150);
			    }
			}
				CommCenter.showDebug("CNSRobotPart" + getPart().getname()
						+ " processing:  " + getName() + " is done");
			getPart().stateChanged(BioDevice.DONE);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 *  
	 */
	private void initiate() {
		this.setExecMode(RANDOM_VALUE_MODE);
		this.setExecType(MULTIPLE_ACT);
		this.setNominal(true);

	}

	private Vector coms;
}