package mns;

import java.util.Iterator;
import java.util.Vector;

import mainframe.Tools;
import monitors.CommCenter;
import monitors.MNSRobotPanel;
import nns.BioSensorEvent;
import nns.BioSensorListener;
import brain.reflexgenerator.PMotionCalculator;
import biocell.BioThread;
import biocell.EnergyPrimitive;
import crlparser.ParserEvent;
import crlparser.ParserListener;
import devs.motorpackage.asc16.Servo;
import devs.speechpackage.SpeechGenerator;
import devs.speechpackage.SpeechRecognizer;

/*
 * This program is provided WITHOUT ANY WARRANTY either expressed * or implied.
 * You may study, use, modify, and distribute it for * any non-commercial
 * purposes. For any commercial use, prior * permission from the author must be
 * granted. *
 * 
 * @author Martin Lukac, 2003-2004 Created on Jan 1, 2004 * Intelligent Robotics
 *         Laboratory * Portland State University 00* Office: 70-08 FAB, phone:
 *         503-725-2783 * Mail: P.O. Box 751 * Portland, OR 97207-0751 * Email:
 *         lukacm@ece.pdx.edu * Updates and documentation: www.pdx.edu\~lukacm * *
 *         This is the main class of a single robot It provides tools and *
 *         methods to add, remove and communicarte with members of the robot *
 *         Note: this CNS, CNSRobotParts and subsequent devices have all *
 *         command queues. This allows to generate certain type of coherence *
 *         and a certain level of decoherence heheheheh enjoy *
 ******************************************************************************/
/**
 * This part of the code is the master controller of actions and of their
 * execution in time and in the problem space it self. It is the unique point o
 * access to most motoric functions similalrly to th cerebelum but in this case
 * this processes all possible commands from any other robotic or mental module
 * 
 * 
 * @author martin lukac
 * 
 */
public class MNS extends BioThread implements ParserListener, BioSensorListener {

    /*
     * (non-Javadoc)
     * 
     * @see mns.BioDeviceListener#bioEvent(mns.BioDeviceEvent)
     */
    public void bioEvent(BioDeviceEvent bdE) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see nns.BioSensorListener#sensorCRLPercieved(nns.BioSensorEvent)
     */
    public void sensorCRLPercieved(BioSensorEvent event) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see nns.BioSensorListener#sensorPercieved(nns.BioSensorEvent)
     */
    public void sensorPercieved(BioSensorEvent event) {
        Object data = event.getEventString();
        if (data instanceof String) {
            this.cQueue.sensorCRLPercieved(event);
            if (cQueue.getSize() <= 1)
                this.cQueue.sensorCRLPercieved(new BioSensorEvent("mns",
                        "start"));
        }
    }

    /**
     * Default Constructor this assusres now a order of different devices. This
     * has to be imporved in order to allow a bigger freedom of commands
     */
    public MNS() {
        super("MNS");
        cQueue = new MNScommandQueue();
        devices = new Vector();
        start();
    }

    /**
     * Default Constructor this assusres now a order of different devices. This
     * has to be imporved in order to allow a bigger freedom of commands
     */
    public MNS(MNScommandQueue cQueue) {
        super("MNS");
        this.cQueue = cQueue;
        devices = new Vector();
        start();
    }

    /**
     * @return Returns the execSignal. This signal is used for the network
     *         communication
     */
    public boolean isExecSignal() {
        return execSignal;
    }

    /**
     * @param execSignal
     *            The execSignal to set.
     */
    public void setExecSignal(boolean execSignal) {
        this.execSignal = execSignal;
    }

    // TODO to be migrated To a separated object used for construction
    /***************************************************************************
     * Constructors of the robot from file not constructors in the proper term -
     * just create the robot from a descriptor file THESE ARE NOT THREAD PROOF
     * AND ARE USED ONLY DURRING THE ROBOT NON ACTIVITY add a object to the
     * robot: this can be anything but if your goal is to make it work add only
     * parts such as Servos, BioDevice or other related shit
     * 
     * @param name -
     *            name of the CNSRobotPart to be added
     **************************************************************************/
    public void addbioPart(String name) {
        // colsole.p
        showMessage("MNS RobotPart " + name + " is being created");
        cnsRobotParts.add(new MNSRobotPart(cQueue, name.trim().toLowerCase()));
        stepEnergy(name);
    }

    /**
     * @param robotpart
     * @param indexs
     */
    public void addbioDevice(String robotpart, int[] indexs) {
        Iterator i = cnsRobotParts.iterator();
        MNSRobotPart robotprt;

        while (true) {
            robotprt = (MNSRobotPart) i.next();
            if (robotprt.getname().equalsIgnoreCase(robotpart)) {
                Vector temp = new Vector();
                for (int a = 0; a < indexs.length; a++) {
                    if (devices.get(indexs[a]) == null)
                        if (indexs[a] != 0)
                            devices.add(indexs[a], new Servo(indexs[a]));

                    temp.add((BioDevice) devices.elementAt(indexs[a]));
                }
                robotprt.addDevs(temp);
                break;
            }
        }
        showMessage("BioDevices in " + robotpart + " are allocated");
    }

    /**
     * @param robotpart
     * @param indexs
     */
    public void addbioDevice(String robotpart, String indexs) {
        Iterator i = cnsRobotParts.iterator();
        MNSRobotPart robotprt;

        String[] toks = Tools.tokenize(indexs);
        while (true) {
            robotprt = (MNSRobotPart) i.next();
            if (robotprt.getname().equalsIgnoreCase(robotpart)) {
                BioDevice[] temp = new BioDevice[toks.length];
                for (int a = 0; a < toks.length; a++) {
                    int index = Integer.parseInt(toks[a]);
                    if (index != 0)
                        devices.add(index, new Servo(index));
                    else
                        devices.add(0, new SpeechGenerator());

                    temp[a] = ((BioDevice) devices.elementAt(index));
                    showMessage(" BioDevice" + index + " in " + robotpart
                            + " is created");
                    showMessage("Created device is a: " + temp[a].getname());
                }

                try {
                    robotprt.addDevs(temp);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                break;
            }
        }
        showMessage(toks.length + " BioDevices in " + robotpart
                + " are created");
    }

    /**
     * adds a single command to a bioPart according to Class command
     * specifications
     * 
     * @param robotpart -
     *            the destination
     * @param biocommand -
     *            the command to be created Very unhandy but works perfectly
     */
    public void addbioCommand(String robotpart, String biocommand) {
        if (robotpart.startsWith("robot")) {
            mnsData.add(biocommand);
            showMessage("Command " + biocommand + " for main controller " + robotpart
                    + " is created");
        } else {
            comData.add(robotpart + " " + biocommand);
            Iterator i = cnsRobotParts.iterator();
            MNSRobotPart robotprt;
            while (i.hasNext()) {
                robotprt = (MNSRobotPart) i.next();
                if (robotprt.getname().equalsIgnoreCase(robotpart)) {
                    robotprt.addCommand(biocommand);
                    break;
                }
            }
            showMessage("Command " + biocommand + " for " + robotpart
                    + " created");
        }
    }

    /**
     * @param part
     * @param max
     */
    public void setPartParam(String part, String param, String data) {
//        Iterator i = cnsRobotParts.iterator();
//        stepEnergy(part + max);
//
//        showMessage(part + " sets the Servo Max: " + max);
//        MNSRobotPart robotprt;
//        while (true) {
//            robotprt = (MNSRobotPart) i.next();
//            if (robotprt.getname().equalsIgnoreCase(part)) {
//                robotprt.setPartMax(max);
//                break;
//            }
//
//        }
    }

    /**
     * @param part
     * @param max
     */
    public void setPartMax(String part, String max) {
        Iterator i = cnsRobotParts.iterator();
        stepEnergy(part + max);

        
        MNSRobotPart robotprt;
        while (true) {
            robotprt = (MNSRobotPart) i.next();
            if (robotprt.getname().equalsIgnoreCase(part)) {
                robotprt.setPartMax(max);
                break;
            }

        }
    }

    /**
     * @param part
     * @param min
     */
    public void setPartMin(String part, String min) {
        stepEnergy(part + min);

        Iterator i = cnsRobotParts.iterator();
        MNSRobotPart robotprt;
        while (true) {
            robotprt = (MNSRobotPart) i.next();
            if (robotprt.getname().equalsIgnoreCase(part)) {
                 robotprt.setPartMin(min);
                break;
            }

        }
    }

    /***************************************************************************
     * End of Constructors of the robot from file
     **************************************************************************/

    /**
     * @param pE -
     *            the caught parser event process the give parserEvent The CNS
     *            process either only XML dfiles either command strings. This
     *            method is used by any type of command system able to generate
     *            parser event in the MNS format of command - Mainly used by the
     *            NNS control and reflex system
     */
    public void load(ParserEvent pE) {
        stepEnergy(pE);
        Object data = pE.getData();
        if (data instanceof String) {
            this.cQueue.sensorCRLPercieved(new BioSensorEvent(this.getName(),
                    (String) data));
            if (cQueue.getSize() <= 1)
                this.cQueue.sensorCRLPercieved(new BioSensorEvent(this
                        .getName(), "start"));
        }
    }

    /**
     * @return - the complete array of commands following one after another as
     *         loaded in the MNSRobotPart
     */
    public Vector getComData() {
        return this.comData;
    }

    /**
     * @param indexes -
     *            sets a commands for the whole robot in a single step as a set
     *            of reflexes -- overrides the whole command queue
     */
    public void setMotState(int[] indexes) {
        int begin = 1;
        for (int a = 1; a < cnsRobotParts.size(); a++) {
            MNSRobotPart part = (MNSRobotPart) cnsRobotParts.get(a);
            int index = part.getDevices().length;
            String args = "";
            for (int b = 0; b < index; b++) {
                args += Integer.toString(indexes[b + begin]) + " ";
            }
            begin += index;
            // cQueue.addReflex(part.getname() + "*" + "move*" + args);
            this.load(new ParserEvent(part.getname() + "*" + "move*" + args));
            this.load(new ParserEvent("null" + "*" + "start"));
        }
    }

    /**
     * @return - the complete vector of the body positions represented in an
     *         array of integer positions of every servo motor
     */
    public int[] getMotState() {
        int[] devs = new int[devices.size() - 1];
        // skip the mouth at all
        for (int a = 2; a < devices.size(); a++) {
            devs[a - 1] = ((BioDevice) devices.get(a)).getCurrentPosition();
        }
        return devs;
    }

    /**
     * @return - the complete vector (including the jaw) of the body positions
     *         represented in an array of integer positions of every servo motor
     */
    public int[] getComState() {
        int[] devs = new int[devices.size() - 1];
        // skip the speech generator
        for (int a = 1; a < devices.size(); a++) {
            devs[a - 1] = ((BioDevice) devices.get(a)).getCurrentPosition();
        }
        return devs;
    }

    /**
     * @param monitor -
     *            add the monitoring device to this part
     */
    public void addMonitor(MNSRobotPanel monitor) {
        this.monitor = monitor;
    }

    /**
     * @param listener -
     *            add a CNSListener
     */
    public void addCNSListener(MNSListener listener) {
        this.listener = listener;
    }

    /**
     * @param name -
     *            name of the device to look for
     * @return - null or BioDevice
     */
    public MNSRobotPart getRobotPart(String name) {
        stepEnergy(name);

        Iterator i = cnsRobotParts.iterator();
        while (i.hasNext()) {
            MNSRobotPart result = (MNSRobotPart) i.next();
            if (result.getname().equalsIgnoreCase(name))
                return result;
        }
        return null;
    }

    /**
     * @return Returns the devices
     */
    public Vector getDevices() {
        return devices;
    }

    /**
     * @return Returns the cnsRobotParts.
     */
    public Vector getCnsRobotParts() {
        return cnsRobotParts;
    }

    /**
     * @param iD
     */
    public void setiD(String iD) {
        this.iD = iD;
    }

    /**
     * used for the initialization of the robot from the moods vectors
     */
    private void setAllParams() {
        Vector di = getCnsRobotParts();
        Iterator i = di.iterator();
        while (i.hasNext()) {
            MNSRobotPart o = (MNSRobotPart) i.next();

            for (int a = 0; a < o.getDevices().length; a++) {
                BioDevice current = o.getDevice(a);
                String[] com = new String[2];

                com[0] = "m";
                com[1] = Integer.toString(((int) (100 * this
                        .getMoodParam("volume"))));
                o.getDevice(a).loadCNTR(new MNSEvent(com));
                CommCenter.showDebug(" Seting volume of the voice");

                com[0] = "p";
                com[1] = Integer.toString(((int) (100 * this
                        .getMoodParam("pitch"))));
                o.getDevice(a).loadCNTR(new MNSEvent(com));
                CommCenter.showDebug(" Seting pitch of the voice");

                com[0] = "r";
                com[1] = Integer.toString(((int) (100 * this
                        .getMoodParam("rate"))));
                o.getDevice(a).loadCNTR(new MNSEvent(com));
                CommCenter.showDebug(" Setting speed of the voice");

                com[0] = "s";
                com[1] = Integer.toString(((int) (100 * this
                        .getMoodParam("speed"))));
                o.getDevice(a).loadCNTR(new MNSEvent(com));

                com[0] = "a";
                com[1] = Integer.toString(((int) (100 * this
                        .getMoodParam("acceleration"))));
                o.getDevice(a).loadCNTR(new MNSEvent(com));

            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {

        while (true) {
            CommCenter.showDebug(" MNS sleeping");
            synchronized (this) {
                /**
                 * Thread proof removal of the first element form the command
                 * Queue
                 */
                String command;
                while ((command = cQueue.get()) == null)
                    // the length of the sleep depends on the amount of energy
                    try {
                        long temp = ((long) (1000 * getMoodParam("waittime")));
                        wait(temp);// TODO - be sure energy is modified with
                        // activity
                    } catch (InterruptedException e) {
                        CommCenter.showDebug("Excution localized from sleep!!");
                    }
                CommCenter.showDebug("MNS executin: " + command);
                try {
                    // check for the existing syntax of the string
                    String dest;
                    try {
                        dest = command.substring(0, command.indexOf('*'));
                    } catch (StringIndexOutOfBoundsException e) {
                        dest = "null";
                    }
                    // if the destination = null, then it is a DIRECT COMMAND
                    if (dest.equalsIgnoreCase("robot")
                            || dest.equalsIgnoreCase("null")) {

                        showMessage("Executing Control command:" + command);

                        try {
                            dest = command.substring(command.indexOf('*') + 1,
                                    command.indexOf('*',
                                            command.indexOf('*') + 1));
                        } catch (Exception e) {
                            dest = command.substring(command.indexOf('*') + 1,
                                    command.length());
                        }
                        // the only direct command is yet wait
                        if (dest.equalsIgnoreCase("wait")) {
                            int delay = Integer.parseInt(command
                                    .substring(command.lastIndexOf('*') + 1));
                            try {
                                sleep(delay * 1000);
                            } catch (InterruptedException e) {
                            }
                            cQueue.masterChanges(true);
                        } else if (dest.equalsIgnoreCase("mood")) {
                            this.setMood(command.substring(command
                                    .lastIndexOf('*') + 1));
                            setAllParams();
                            try {
                                sleep(2000);
                            } catch (InterruptedException e) {
                            }
                            cQueue.masterChanges(true);
                        } else if (dest.equalsIgnoreCase("dance")) {
                            /** FIXME - signal to NNS */
                            if (listener != null) {
                                listener.process(new MNSEvent("dancegenerator",
                                        this.getName(), command));
                            }

                            showMessage("Executing Robot command:" + command);
                        } else if (dest.equalsIgnoreCase("follow")
                                || dest.equalsIgnoreCase("auto")) {
                            if (listener != null) {
                                listener.process(new MNSEvent("motionsensor",
                                        this.getName(), command));
                            }
                        } else if (listener != null) {
                            listener.process(new MNSEvent("robot", this
                                    .getName(), command));
                        }
                    } else {
                        // execute the command for the robotic part
                        showMessage("Executing RobotPart command:" + command);
                        // get the destination robot part
                        MNSRobotPart part = getRobotPart(dest);
                        // get the command name
                        String instr = command
                                .substring(command.indexOf('*') + 1);
                        String params = null;
                        // parsing the arguments according to if the command is
                        // simple or complex
                        if (part != null) {
                            try {
                                params = instr
                                        .substring(instr.indexOf('*') + 1);
                                instr = instr.substring(0, instr.indexOf('*'));
                            } catch (Exception e) {
                                params = null;
                                instr.trim();
                            }
                            part.execute(instr, params);
                        } else {

                            showMessage("Robot part " + dest
                                    + " does not exists");
                            cQueue.masterChanges(true);
                        }

                    }

                    showMessage("Signalization: " + isExecSignal() + " size "
                            + cQueue.getSize());

                    // send the notification high level notification to systems
                    // of similar capacity
                    if (listener != null) {
                        while (!cQueue.isAvailable())
                            try {
                                sleep(100);
                            } catch (Exception e) {
                            }

                        listener.process(new MNSEvent("null", this.getName(),
                                "INFORM " + command));
                    }
                } catch (Exception e) {
                    showMessage("Unrecognized command to MNS: " + command
                            + " :: " + e);
                    e.printStackTrace();
                    cQueue.masterChanges(true);
                }
                // to indicate the queue is empty the robot
                // send a signal
                if (cQueue.getSize() <= 0) {
                    while (!cQueue.isAvailable())
                        try {
                            sleep(100);
                        } catch (Exception e) {
                        }
                    showMessage("Robot is done" + listener);
                    if (listener != null)
                        listener.process(new MNSEvent("netclient", "MNS",
                                "DONE"));
                }
            }
        }
    }

    // private void

    /**
     * @param message
     */
    private void showMessage(String message) {
        if (monitor != null)
            monitor.showRobot(message);
        else
            System.out.println(message);
    }

    // the name of this robot is temporarily stored here
    private String iD;

    // all parts of the robot
    private Vector cnsRobotParts = new Vector(1, 1);

    // the pseudo-neural system of the robot
    private Vector devices;

    // command queue
    private MNScommandQueue cQueue = null;

    // current class is a thread;
    private Thread cnsThread;

    // private dump message containe
    private MNSRobotPanel monitor;

    // the client connecting the robot to the NNS implemented as a listener
    private MNSListener listener;

    // teh Vector of all ever registered commands
    private Vector comData = new Vector(1, 1);

    // teh Vector of all ever registered commands
    private Vector mnsData = new Vector(1, 1);

    private boolean execSignal = true;

}