/*
 * Created on Mar 22, 2004
 */
package mns;

import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;

import monitors.CommCenter;
import mainframe.Tools;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import crlparser.ParserEvent;
import crlparser.ParserListener;

/**
 * @author martin lukac
 *
 *Simple SAX parser Handler of the MNS command scripting files written in CRL
 *
 */
public class MNSCommandhandler extends DefaultHandler {
	
	/**
	 * Comment for <code>listener</code>
	 */
	private ParserListener listener;
	
	/**
	 * @param listener
	 */
	public MNSCommandhandler(ParserListener listener){
		this.listener = listener;
		comSet = new Vector(1,1);
		setcomSet("mns/generatedRobotCommands.crl");
		CommCenter.showComm("Handler created");
	}
	/**
	 * @param doc
	 */
	public void setDoc(Document doc) {
		this.doc = doc;
	}
	
	/**
	 * @param filename
	 */
	public void setcomSet(String filename){
		try{
			RandomAccessFile bir = new RandomAccessFile(filename, "r");
			String line;
			while((line = bir.readLine()) != null){
			    if (line.length() > 2 ){
				String [] p = Tools.tokenize(line);
				int level = Integer.parseInt(p[0]);
				for (int a = 1; a< p.length; a++){
					//checking for G - Group commands - requiring a tag and antitag
					if (p[a].charAt(p[a].length()-1) == 'G'){
						Instruction n =new Instruction(p[a].substring(0, p[a].lastIndexOf('G')), level);
						n.setGroup(true);
						comSet.add(n);
					}					
					//chekcing for N - Nominal commands - needs only tag and no parameters or data
					else if (p[a].charAt(p[a].length()-1) == 'N'){
						Instruction n = new Instruction(p[a].substring(0, p[a].lastIndexOf('N')), level);
						n.setNominal(true);
						comSet.add(n);
					} else comSet.add(new Instruction(p[a], level));
				}
			    }
			}
		} catch (Exception e){
			CommCenter.showError("Instruction generation problem: ");e.printStackTrace();
		}
		CommCenter.showComm("XML Parser: Instruction set generated successfully");
	}
	
	/**
	 * @param name
	 * @return
	 */
	private boolean comByName(String name){
		Iterator i = comSet.iterator();
		while (i.hasNext()){
			if (((Instruction)i.next()).getName().equalsIgnoreCase(name))
				return true;
		}
		return false;
	}

	/**
	 * @param name
	 * @return
	 */
	private Instruction getcomByName(String name){
		Iterator i = comSet.iterator();
		while (i.hasNext()){
			Instruction istr = (Instruction)i.next();
			if (istr.getName().equalsIgnoreCase(name))
				return istr;
		}
		return null;
	}

	/**
	 * @return
	 */
	public Document getDoc() {
		return doc;
	}
	
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(
			String namespaceURI,
			String lname,
			String qname,
			Attributes attrs) {
		if (comByName(qname.trim())){
			Instruction i;
			if ((i = getcomByName(qname.trim())) != null){
				System.out.println("Got tag: "+i.getName()+" level: "+i.getLevel());
				if (i.getLevel() == ROBOT_LEVEL ){
					stageLevel = ROBOT_LEVEL;
					destination = "robot";
				} else if (i.getLevel() == DIRECT_COMMAND_LEVEL){
					commandName = i.getName();
					stageLevel = DIRECT_COMMAND_LEVEL;
				} else if (i.getLevel() == ROBOT_PART_LEVEL){
					destination = i.getName();
					stageLevel = ROBOT_PART_LEVEL;
				} else {
					commandName = i.getName();
					stageLevel = ROBOT_PART_COMMAND_LEVEL;
				}  
			} 			
			if (i.isGroup()){	
				String command = destination+"*"+qname;
				generateMotorCommand(command);
			}else if (i.isNominal()){
				String command = destination+"*"+qname;
				generateMotorCommand(command);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String namespaceURI, String lname, String qname) {
		
		if (comByName(qname.trim())){
			Instruction i;
			if ((i = getcomByName(qname.trim())) != null){
				if (i.getLevel() == ROBOT_LEVEL){
					stageLevel = ROBOT_LEVEL;
					destination = null;
				} else if (i.getLevel() == DIRECT_COMMAND_LEVEL){
					commandName = null;
					stageLevel = DIRECT_COMMAND_LEVEL;
				} else if (i.getLevel() == ROBOT_PART_LEVEL){
					destination = "robot";
					stageLevel = ROBOT_PART_LEVEL;
				} else {
					commandName = null;
					stageLevel = ROBOT_PART_COMMAND_LEVEL;
				} 


				
				if (i.isGroup()){
					String command = destination+"*"+PREFIX+qname;
					generateMotorCommand(command);
				}

			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char ch[], int start, int length) {
		
		// get the value of within tag
		String temp = "";
		for (int i = start; i < start+length; i++) {
			temp = temp + ch[i];
		}
		temp = temp.trim();
		CommCenter.showDebug("Detected data: "+destination+"*"+commandName+"*"+temp);
		if (stageLevel == ROBOT_LEVEL){
			stageLevel = ROBOT_LEVEL;
			destination = null;
		} else if (stageLevel == DIRECT_COMMAND_LEVEL){
			if (temp != null && temp.length() > 0){
				String command = destination+"*"+commandName+"*"+temp;
				generateMotorCommand(command);
			} 
		} else if (stageLevel == ROBOT_PART_LEVEL){
			stageLevel = ROBOT_PART_LEVEL;
		} else {
			String command = destination+"*"+commandName+"*"+temp;
			generateMotorCommand(command);
			stageLevel = ROBOT_PART_COMMAND_LEVEL;
		}
	}
	
	
	/**
	 * @param command
	 */
	protected void generateMotorCommand(String command){
		CommCenter.showDebug("Command generated: "+command);
		if (listener != null)
			listener.load(new ParserEvent(command));
		
	}
	/**
	 * @param node
	 * @param pW
	 */
	public void writeCRL(Node node, PrintWriter pW) {
		this.pW = pW;
		write(node, "\t");
	}
	
	/**
	 * @param node
	 * @param indent
	 */
	private void write(Node node, String indent) {
		switch (node.getNodeType()) {
			case Node.DOCUMENT_NODE :
			{
				pW.println(indent + "<?xml version='1.0'?>");
				NodeList list = node.getChildNodes();
				int a = 0;
				while (a < list.getLength()) {
					write((Node) list.item(a), indent);
				}
			}
		}
	}

	/**
	 * @author martin lukac
	 *
	 */
	class Instruction{
		public Instruction(String name, int level){
			this.name = name;
			this.level = level;
		}
		
		private String name;
		private int level;
		private boolean nominal = false;;
		private boolean group = false;;
		/**
		 * @return Returns the level.
		 */
		public int getLevel() {
			return level;
		}
		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}
		/**
		 * @return Returns the group.
		 */
		public boolean isGroup() {
			return group;
		}
		/**
		 * @param group The group to set.
		 */
		public void setGroup(boolean group) {
			this.group = group;
		}
		/**
		 * @return Returns the nominal.
		 */
		public boolean isNominal() {
			return nominal;
		}
		/**
		 * @param nominal The nominal to set.
		 */
		public void setNominal(boolean nominal) {
			this.nominal = nominal;
		}
	}
	//writer
	PrintWriter pW;
	
	int n_tab = 0;
	//init the keywords map
	//the discintinuous parsing result is stored here
	private Document doc;
	//the builder
	private DocumentBuilder dB;
	//the counter for building parts of robot
	private int stageLevel = 0;
	//the grammar it sefl of the CRL command processing is a simple leveled hierarchy
	private Vector comSet;
	//sections defined by tags
	private boolean robotBegin = false;
	
	
	//to construct a robot we need some templates
	//and some milestones -- hardcoded
	private String destination;
	private String commandName;
	
	
	public final static int ROBOT_LEVEL =0;
	public final static int DIRECT_COMMAND_LEVEL =1;
	public final static int ROBOT_PART_LEVEL =2;
	public final static int ROBOT_PART_COMMAND_LEVEL =3;
	
	public final static String PREFIX = "un";
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	public void endDocument() throws SAXException {
		// TODO Auto-generated method stub
		super.endDocument();
		if (stageLevel == 0 )
			CommCenter.showComm("The commands were parsed succesfully");
	}
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	public void startDocument() throws SAXException {
		// TODO Auto-generated method stub
		super.startDocument();
		CommCenter.showComm("Start to parsing the command file");
		stageLevel = 0;
	}
}