/*
 * Created on Jan 8, 2004 A serial-parallel timing commandQueue
 */
package mns;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import monitors.CommCenter;
import nns.BioSensorEvent;
import nns.BioSensorListener;
import biocell.BioCell;
import biocell.EnergyPrimitive;

/**
 * @author Martin Lukac
 * 
 * The core of the synchronization between Command and RobotParts, is this
 * object. It handles single command running or continuous comand running
 * 
 */
public class MNScommandQueue extends BioCell implements MNSRobotPartListener,
        BioSensorListener {

    /*
     * (non-Javadoc)
     * 
     * @see mns.BioDeviceListener#bioEvent(mns.BioDeviceEvent)
     */
    public void bioEvent(BioDeviceEvent bdE) {
        // TODO Auto-generated method stub

    }

    // Indicates if the robot operates on synchronous or sequential mode
    private boolean sync = false;

    // boolean state indicator - just to allow waiting of hanging thread
    // requests
    private boolean busy = false;

    // locking and unlocking thei object
    private boolean available = false;

    // command queue
    private List commandQueue;

    // the current list of active tasks. The guarantee of having one per robot
    // part
    private List processQueue;

    /**
     * Constructor simple implementing a commandQueue and a current processQueue
     */
    public MNScommandQueue() {
        super("MNS Command Queue");
        commandQueue = (List) (new Vector(1, 1));
        processQueue = (List) (new Vector(1, 1));
    }

    /*
     * (non-Javadoc)
     * 
     * @see mns.MNSRobotPartListener#stateChanged(mns.MNSRobotPartEvent)
     */
    synchronized public void stateChanged(MNSRobotPartEvent rpEvent) {
        while (busy)
            try {
                wait(10);
            } catch (InterruptedException e) {
                ;
            }
        busy = true;
        if (rpEvent.getState() <= BioDevice.CYCLING) {
            String name = rpEvent.getComSource().getname();
            Iterator i = processQueue.iterator();
            // check if the robot part that terminated is in the processlist and
            // removes it
            // else it is a "zombie or a bug"
            while (i.hasNext()) {
                String s = (String) i.next();
                if (s.equalsIgnoreCase(name)) {
                    i.remove();
                     CommCenter.showDebug(rpEvent.getCommand()
                     + " <-- command terminated ");
                    break;
                }
            }
        }
        // free the queue
        busy = false;
        available = true;
        notifyAll();
    }

    /*
     * (non-Javadoc)
     * 
     * @see devs.sensorpackage.BioSensorListener#sensorPercieved(devs.sensorpackage.BioSensorEvent)
     */
    synchronized public void sensorPercieved(BioSensorEvent event) {
        addReflex(event.getEventString());
    }

    /*
     * (non-Javadoc)
     * 
     * @see nns.BioSensorListener#sensorCRLPercieved(nns.BioSensorEvent)
     */
    public synchronized void sensorCRLPercieved(BioSensorEvent event) {
        String command = event.getEventString();
        stepEnergy(command);
        if (command.equalsIgnoreCase("start")) {
            available = true;
            notify();
        } else if (command.equalsIgnoreCase("stop")) {
            available = false;
            notify();
        } else if (command.equalsIgnoreCase("flush")) {
            this.flush();
            notify();
        } else {
            commandQueue.add(command);
            available = true;
            notify();
        }
        CommCenter.showDebug("CommCenter: " + "Size of queue is: "
                + commandQueue.size());
        CommCenter
                .showDebug("CommCenter: " + "Recieved Command is: " + command);
        notifyAll();
    }

    /**
     * @param enable
     */
    synchronized public void masterChanges(boolean enable) {
        if (enable) {
            if (sync)
                setAvailable(enable);
            notifyAll();
        } else
            setAvailable(!enable);
        notifyAll();
    }

    /**
     * @return
     */
    public synchronized String get() {

        // sleep while not available
        if (available == false || this.commandQueue.size() <= 0) {
            stepEnergy("null");
            return null;
        }
        // else return the next command
        String result = getCommand();
        String dest;
        try {
            dest = result.substring(0, result.indexOf('*'));
        } catch (StringIndexOutOfBoundsException e) {
            dest = result;
        }

        CommCenter.showDebug("MNSCommandQueue Distributing: " + result);

        if (dest.equalsIgnoreCase("robot") || dest.equalsIgnoreCase("null")) {
            // command is directly issued to the CommandQueue
            if (result.indexOf("mood") > -1) {
                notifyAll();
                stepEnergy(result);
                return result;
            } else {
                if (processQueueCommand(result)) {
                    notifyAll();
                    stepEnergy("null");
                    return null;

                } else {
                    notifyAll();
                    stepEnergy(result);
                    return result;
                }
            }
        } else {
            // checks wether the given part is already running a command
            if (checkRunning(result)) {
                setAvailable(true);
                notifyAll();
                stepEnergy(result);
                return result;
            } else {
                // return null and wait for commands to terminate
                notifyAll();
                stepEnergy("null");
                addReflex(result);
                return null;
            }
        }
    }

    /**
     * @param command
     * @return
     */
    private boolean checkRunning(String command) {
        stepEnergy(command);
        String com = null;
        try {
            com = command.substring(0, command.indexOf('*'));
        } catch (StringIndexOutOfBoundsException e) {
            try {
                com = command.substring(0, command.indexOf(' '));
            } catch (StringIndexOutOfBoundsException ev) {
                com = command;
            }
        }
        Iterator i = processQueue.iterator();
        String s;
        while (i.hasNext()) {
            s = (String) i.next();
            // System.out.print(s+" ");
            if (com.toLowerCase().startsWith((s.trim()))) {
                return false;
            }
        }
        // System.out.print(" <> "+com+" ");
        // System.out.println();
        processQueue.add(com.trim());
        return true;
    }

    /**
     * @param command
     * @return
     */
    private boolean getRunning(String command) {
        stepEnergy(command);

        String com = command.substring(0, command.indexOf('*'));
        Iterator i = processQueue.iterator();
        String s;
        while (i.hasNext()) {
            s = (String) i.next();
            if (com.toLowerCase().startsWith((s.trim()))) {
                i.remove();
                return true;
            }
        }
        return true;
    }

    /**
     * @return
     */
    private boolean checkRunning() {
        if (processQueue.size() > 0)
            return true;
        return false;
    }

    /**
     * @param command
     */
    private boolean processQueueCommand(String command) {
        stepEnergy(command);
        String dest = null;
        try {
            dest = command.substring(command.indexOf('*') + 1, command.indexOf(
                    '*', command.indexOf('*') + 1));
        } catch (Exception e) {
            dest = command
                    .substring(command.indexOf('*') + 1, command.length());
        }
        // internal commands for the control of the CommandQueue
        if (dest.toLowerCase().startsWith("sync")) {
            setSync(true);
            return true;
        } else if (dest.toLowerCase().startsWith("flush")
                || dest.toLowerCase().startsWith("stop")) {
            flush();
            return true;
        } else if (dest.toLowerCase().startsWith("unsync")) {
            setSync(false);
            return true;
        } else if (dest.toLowerCase().startsWith("continue")) {
            return true;
        } else
            return false;
    }

    /**
     * @param command
     */
    private synchronized void addReflex(String command) {
        stepEnergy(command);
        if (command.equalsIgnoreCase("start"))
            available = true;
        else if (command.equalsIgnoreCase("stop"))
            available = false;
        else if (command.equalsIgnoreCase("flush"))
            this.flush();
        else
            commandQueue.add(0, command);
        notifyAll();
        CommCenter.showDebug("CommCenter: " + "Size of queue is: "
                + commandQueue.size());
        CommCenter
                .showDebug("CommCenter: " + "Recieved Command is: " + command);
    }

    /**
     * @param sync
     */
    public synchronized void setSync(boolean sync) {
        stepEnergy("sync");
        // the priority is obvivous ...
        if (sync) {
            this.sync = true;
            available = true;
        } else {
            while (checkRunning()) {
                try {
                    wait(100);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            CommCenter.showDebug("sync is false now");
            this.sync = false;
            available = true;

        }
        CommCenter.showDebug("sync is " + sync + " now");
        notifyAll();
    }

    /**
     * 
     * 
     */
    public synchronized void flush() {
        masterChanges(false);
        available = false;
        sync = false;
        commandQueue.clear();
        notifyAll();
    }

    /**
     * @return
     */
    public int getSize() {
        return commandQueue.size();
    }

    /**
     * @return Returns the available.
     */
    public synchronized boolean isAvailable() {
        return available;
    }

    /**
     * @param available
     *            The available to set.
     */
    public synchronized void setAvailable(boolean available) {
        this.available = available;
        notifyAll();
    }

    /**
     * @return Returns the busy.
     */
    public boolean isBusy() {
        return busy;
    }

    /**
     * @param busy
     *            The busy to set.
     */
    public void setBusy(boolean busy) {
        this.busy = busy;
    }

    /**
     * @return Returns the sync.
     */
    public boolean isSync() {
        return sync;
    }

    /**
     * @return
     */
    private String getCommand() {
        String comm = ((String) commandQueue.remove(0));
        try {
            String t = comm.substring(comm.indexOf('*') + 1, comm.indexOf('*',
                    comm.indexOf('*') + 1));
            if (t.trim().equalsIgnoreCase("null")) {
                if (commandQueue.size() > 0) {
                    stepEnergy(((String) commandQueue.get(0)));
                    return ((String) commandQueue.remove(0)).toLowerCase();
                } else {
                    stepEnergy("stop");
                    return "stop";
                }
            } else
                return comm.trim().toLowerCase();
        } catch (StringIndexOutOfBoundsException e) {

        }
        return comm.trim().toLowerCase();
    }
}