package mns;

import java.util.Vector;

import biocell.EnergyPrimitive;

import monitors.CommCenter;
import devs.speechpackage.SpeechGenerator;

/**
 * Created on Jun 28, 2004
 * 
 * For specific purposes a Threadded command is an internal class allowing to
 * run commands on one self and generate some regular, cyclic, time-variant
 * behavior. Thus by default this type of command will be called when the period
 * parameter is found durring creation of this command the default state is
 * described by PERIODIC_ACT and RRANGE_VALUE_MODE
 *
 * @author Martin Lukac
 */
public class SpeachThread extends CommandThread implements BioDeviceListener {
	/* (non-Javadoc)
	 * @see biocell.BioEnergySpender#energyConsumed(biocell.EnergyPrimitive)
	 */
	public void energyConsumed(EnergyPrimitive ep) {
		// TODO Auto-generated method stub

	}
	/**
	 * @param name
	 */
	public SpeachThread(String name) {
		super(name);
		initiate();
		initiate();
		Vector comms = new Vector();
			comms.add("null");
			comms.add("75");
		
		this.setCommand(comms.toArray());
	}

	/**
	 * some commands can be coordinated through events such as speech or some
	 * other cyclic or random movements this serves to time coordinating them
	 * 
	 * @param bdE -
	 *            caught BioDeviceEvent
	 */
	synchronized public void bioEvent(BioDeviceEvent bdE) {

		if (bdE.getState() >= BioDevice.CYCLING) {
			CommCenter.showDebug("Speech movement ");
		} else if (bdE.getState() == BioDevice.DONE) {
			if (bdE.getSource() instanceof SpeechGenerator) {
				//FIXME - set the device in some decent position
				getPart().getDevice(1).load(new MNSEvent(String.valueOf(50)));
				setDone(true);
				CommCenter.showDebug("Speach command is done");
			}
		}
		notify();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			CommCenter.showDebug("New speechthread");
			CommCenter.showDebug(getInstructions());
			getPart().getDevice(0).addListener(this);
			getPart().getDevice(1).addListener(null);
			setDone(false);
			synchronized (this) {
				Object[] comand = getInstructions();
				String line = (String)comand[0];
				CommCenter.showDebug("Speech Arguments: "+line);
				getPart().getDevice(0).load(new MNSEvent(line));
				while (!isDone()) {
					try {
						int position = (getPart().getDevice(1))
								.getCurrentPosition();
//						System.out.println(" : "+position);
						position = (int)(((Math.random()) * (position)) + position)%100;
//						if (position > 100) position = 100;
//						else if (position < 0) position = 0;
//						System.out.println(position);
						getPart().getDevice(1).load(
								new MNSEvent(String.valueOf(position)));

						wait(200);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				setDone(true);
				getPart().commandOccured(new CommandEvent(this, "DONE"));
				CommCenter.showDebug("speech thread terminated");
				return;
			}
		} catch (Exception e) {
			CommCenter.showError("Command " + e);
			StackTraceElement [] st = e.getStackTrace();
			for (int a = 0; a < st.length; a++){
				CommCenter.showDebug(st[a].getClassName()+" ==> "+st[a].getLineNumber());
			}
		}
	}
	

	/**
	 *  
	 */
	private void initiate() {
		setExecMode(Command.RRANGE_VALUE_MODE);
		setExecType(Command.PERIODIC_ACT);
	}

}