package mns;

import biocell.EnergyPrimitive;
import monitors.CommCenter;
import devs.speechpackage.SpeechGenerator;

/**
 * Created on Jun 28, 2004
 * 
 * For specific purposes a Threadded command is an internal class allowing to
 * run commands on one self and generate some regular, cyclic, time-variant
 * behavior. Thus by default this type of command will be called when the period
 * parameter is found durring creation of this command the default state is
 * described by PERIODIC_ACT and RRANGE_VALUE_MODE
 *
 * @author Martin Lukac
 */
public class SynchroSpeachThread extends Command implements BioDeviceListener {
	/**
	 * @param name
	 */
	public SynchroSpeachThread(String name) {
		super(name);
		initiate();
	}
	/* (non-Javadoc)
	 * @see biocell.BioEnergySpender#energyConsumed(biocell.EnergyPrimitive)
	 */
	public void energyConsumed(EnergyPrimitive ep) {
		// TODO Auto-generated method stub

	}

	/**
	 * some commands can be coordinated through events such as speech or some
	 * other cyclic or random movements this serves to time coordinating them
	 * 
	 * @param bdE -
	 *            caught BioDeviceEvent
	 */
	synchronized public void bioEvent(BioDeviceEvent bdE) {

		if (bdE.getState() >= BioDevice.CYCLING) {
			CommCenter.showDebug("Speach movement ");
		} else if (bdE.getState() == BioDevice.DONE) {
			if (bdE.getSource() instanceof SpeechGenerator) {
				getPart().getDevice(1).load(new MNSEvent(String.valueOf(100)));
				setDone(true);
				CommCenter.showDebug("Speach command is done");
			}
		}
		notify();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			CommCenter.showDebug("New speechthread");
			getPart().getDevice(0).addListener(this);
			setDone(false);
			synchronized (this) {
				Object[] comand = getInstructions();
				String line = (String)comand[0];
//				for (int a = 0; a< comand.length; a++)
//					line += (String)comand[a];
				//generate the speech by sending the text to the synthesizer
				// device
				getPart().getDevice(0).load(new MNSEvent(line));
				while (!isDone()) {
					try {
						int position = (getPart().getDevice(1))
								.getCurrentPosition();
						position = (int) (Math
								.round(((Math.random() * 2) - 1) * position));
						if (position > 100) position = 100;
						else if (position < 0) position = 0;
						getPart().getDevice(1).load(
								new MNSEvent(String.valueOf(position)));

						wait(50);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
				setDone(true);
				getPart().commandOccured(new CommandEvent(this, "DONE"));

				CommCenter.showDebug("End speechthread");

				return;
			}
		} catch (Exception e) {
			CommCenter.showError("Command " + e);
		}
	}

	/**
	 *  
	 */
	private void initiate() {
		setExecMode(Command.RRANGE_VALUE_MODE);
		setExecType(Command.PERIODIC_ACT);
	}

}