package multitool;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author martin lukac, Randall Borck 
 * 
 * Server designed to allow the
 *         communication between robots so as to collect the data through the
 * @see MultiTool
 */
public class MServer extends Thread {
    /**
     *  
     */
    public MServer() {
        this.start();
    }

    /**
     * @param listener
     */
    public MServer(ServerListener listener) {
        this.listener = listener;
        this.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        int i = 0;
        try {
            ServerSocket msg = new ServerSocket(7000);
            showComm("Server Started...");
            while (true && status == 0) {
                Socket socketIn = msg.accept();
                showComm("Incoming Connection from: "
                        + socketIn.getInetAddress().toString() + ":"
                        + socketIn.getPort());
                new MsgHandler(socketIn).start();
                i++;
            }
        } catch (Exception e) {
            showComm(e);
        }
    }

    private ServerListener listener;

    /**
     * @param listener
     */
    public void addListener(ServerListener listener) {
        this.listener = listener;
    }

    /**
     * @param o
     */
    protected void showComm(Object o) {
        if (listener != null)
            listener.load(new ServerEvent(this, o));
        else
            System.out.println(o);
    }

    /**
     *  
     */
    public void stopServer() {
        this.status = 1;
    }

    /**
     * @author Randall Borck, Martin Lukac
     */
    class MsgHandler extends Thread {
        /**
         * @param i
         */
        public MsgHandler(Socket i) {
            socketIn = i;
            try {
                in = new BufferedReader(new InputStreamReader(socketIn
                        .getInputStream()));
                out = new PrintWriter(socketIn.getOutputStream(), true);
                validRobot = false;
            } catch (IOException e) {
                showComm(e);
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Runnable#run()
         */
        /***********************************************************************
         * Registration section & setting up the connection \ Start the main
         * thread for every incoming socket. This will setup and maintain the
         * connection while a connection exists. If a connection is dropped,
         * then the server will quietly drop that client from the client list.
         **********************************************************************/
        public void run() {
            try {
                //Validate the incoming user request
                //Collect the user name
                rbtName = new String(in.readLine());
                if (rbtName.startsWith("REGISTER")) {
                    rbtName = new String(rbtName.substring(8)); //8 =
                    // strlen("REGISTER");
                    rbtName = new String(rbtName.concat(" ")); //used so we
                    // don't go past
                    // the end
                    rbtName = new String(rbtName
                            .substring(rbtName.indexOf(" ")));
                    rbtName = new String(rbtName.trim());
                    if (!rbtName.equals("") && rbtName != null /*
                     * &&
                     * rbtName.length() >
                     * 5
                     */) {
                        showComm("rbtName '" + rbtName + "'");
                        validRobot = true;
                        synchronized (clients) {
                            Enumeration myClientList = clients.elements();
                            while (myClientList.hasMoreElements()) {
                                MsgHandler myHandle = (MsgHandler) myClientList
                                        .nextElement();
                                if (myHandle.rbtName.equals(this.rbtName)) {
                                    validRobot = false;
                                    String errorMsg = new String(
                                            "ERROR: Robot Name already in use.");
                                    try {
                                        this.out.println(errorMsg);
                                        showComm(errorMsg);
                                    } catch (Exception e) {
                                        showComm(e);
                                    }
                                }
                            }
                        }
                    } else if (rbtName.equals("")) {
                        showComm("Assigning random name...");
                        int robotNum;
                        validRobot = false;
                        for (robotNum = 0; !validRobot; robotNum++) {
                            validRobot = true; //First, assume the name will
                            // work
                            rbtName = new String("robot" + robotNum);
                            synchronized (clients) {
                                Enumeration myClientList = clients.elements();
                                while (myClientList.hasMoreElements()) {
                                    MsgHandler myHandle = (MsgHandler) myClientList
                                            .nextElement();
                                    if (myHandle.rbtName.equals(this.rbtName)) {
                                        validRobot = false; //If the chosen
                                        // name doesn't
                                        // work, pick
                                        // another
                                    }
                                }
                            }
                        }
                        showComm("Random robot name assigned: " + rbtName);
                    }
                } else
                    validRobot = false;
                if (!validRobot) {
                    showComm("Unable to connect Robot.");
                    try {
                        socketIn.close();
                    } catch (IOException e) {
                        showComm(e);
                    }
                } else {
                    //Add session to the vector table
                    clients.addElement(this);
                    showComm("Loaded user. Current load: " + clients.size()
                            + " Robots.");
                    //message to incoming recipient
                    out.println("Welcome " + rbtName + "!");
                    while (true) {
                        String message;
                        try {
                            message = new String(in.readLine());
                        } catch (NullPointerException e) {
                            break;
                        }
                        showComm("Request From: " + rbtName);
                        showComm("\tIncoming Message: " + message);
                        processMsg(rbtName, message);
                    }
                    showComm("Leaving and killing current client thread!");
                }
            } catch (IOException e) {
                showComm(e);
            } finally {
                clients.removeElement(this);
                try {
                    socketIn.close();
                } catch (IOException e) {
                    showComm(e);
                }
            }
        }

        /***********************************************************************
         * @param fromUser
         * @param message
         *            Command parsing - Get the command and perform the
         *            appropriate actions \
         **********************************************************************/
        //Parse the incoming message command and deal with the message packet
        //using the appropriate method
        public void processMsg(String fromUser, String message) {
            //Syntax: BROADCAST <message>
            //showComm(message.substring(0,3));
            if (message.toUpperCase().startsWith("BROADCAST "))
                broadcast(fromUser, message.substring(9));
            //Syntax: WHISPER <toUser> <message>
            //Sends a message to an individual user
            else if (message.toUpperCase().startsWith("WHISPER ")) {
                showComm(message);
                String msg = message.substring(8); //Gets rid of the WHISPER
                // part
                msg = new String(msg.trim());
                msg = new String(msg.concat(" "));
                String toUser = new String(msg.substring(0, msg.indexOf(" ")));
                toUser = new String(toUser.trim());
                msg = new String(msg.substring(msg.indexOf(" ")));
                whisper(fromUser, toUser, msg);
            }
            //Syntax: WHO
            //This will return a string with the details of what clients are
            //logged in and what the status of the robots logged in is (i.e.
            // ready/busy)
            else if (message.toUpperCase().startsWith("WHO")) {
                who();
            }
            //Syntax: BYE
            //This will terminate the client connection to the server
            else if (message.toUpperCase().startsWith("BYE")) {
                try {
                    out.println("Good-Bye!");
                    showComm("Robot left: " + fromUser);
                    socketIn.close();
                    clients.removeElement(this);
                } catch (Exception e) {
                    showComm("Robot: " + fromUser + "\n\t" + e);
                }
            }
            //Syntax: DONE
            //This will set the ready flag to done for the robot
            else if (message.toUpperCase().startsWith("DONE")) {
                ready = true;
            }
            //Syntax: FILE <toUser> <filename>
            //This sets up the transfer, then the client should wait
            //until the message "GO" is received. At that point, while
            //there is still file text to send, open a file reader and
            //send packets with the keyword "SEND". When the file reader
            //gets to the end of the file stream, another message is sent
            //with the keyword "STOP".
            /*
             * Example communication from sender's perspective: client ->
             * server: FILE robot1 robotFile.txt server -> client: GO client ->
             * server: SEND ... client -> server: SEND ... client -> server:
             * STOP Example communication from receiver's perspective: server ->
             * client: FILE puppetmaster robotFile.txt client -> server: GO
             * server -> client: SEND ... server -> client: SEND ... server ->
             * client: STOP
             */
            else if (message.toUpperCase().startsWith("FILE")) {
                showComm("Message begins with: " + message);
                String msg = message.substring(5); //Gets rid of the FILE part
                msg = new String(msg.trim());
                msg = new String(msg.concat(" "));
                String toUser = new String(msg.substring(0, msg.indexOf(" ")));
                toUser = new String(toUser.trim());
                msg = new String(msg.substring(msg.indexOf(" ")));
                msg = new String(msg.trim());
                showComm("msg after toUser Parsing = " + msg);
                msg = new String(msg.concat(" "));
                String filename = new String(msg.substring(0, msg.indexOf(" ")));
                //			filename = new String(msg.substring(0,msg.indexOf(" ")));
                msg = new String(msg.substring(msg.indexOf(" ")));
                showComm("Parsing: fromUser=" + fromUser + "\ttoUser=" + toUser
                        + "\tfilename=" + filename);
                showComm(fromUser + " " + toUser + " " + filename);
                sendFile(fromUser, toUser, filename);
            } else if (message.toUpperCase().startsWith("GO")) {
                showComm(message);
                String msg = message.substring(2); //Gets rid of the GO part
                msg = new String(msg.trim());
                msg = new String(msg.concat(" "));
                String toUser = new String(msg.substring(0, msg.indexOf(" ")));
                toUser = new String(toUser.trim());
                msg = new String(msg.substring(msg.indexOf(" ")));
                sendPacket(toUser, "GO");
            } else if (message.toUpperCase().startsWith("SEND")) {
                showComm(message);
            } else if (message.toUpperCase().startsWith("INFORM ")) {
                inform(fromUser, message.substring(7));
                showComm(message);
            } else {
                showComm("Unknown Command: " + message);
            }
        }

        /***********************************************************************
         * Implement the functions to perform tasks \
         **********************************************************************/

        /**
         * @param fromUser
         * @param msgOut
         */
        //Implement the INFORM function to inform all users about a single
        // user.
        public void inform(String fromUser, String msgOut) {
            synchronized (clients) {
                //                MsgHandler handle = (MsgHandler) clients.elementAt(0);
                //                if (!fromUser.equalsIgnoreCase(handle.rbtName)) {
                //                    synchronized (handle.out) {
                //                        handle.out
                //                                .println("INFORM: " + fromUser + " " + msgOut);
                //                        showComm("INFORMING " + handle.rbtName + "\n\tmsg: "
                //                                + msgOut);
                //                    }
                //                    handle.out.flush();
                //                } else {
                Enumeration list = clients.elements();
                while (list.hasMoreElements()) {
                    MsgHandler handle = (MsgHandler) list.nextElement();
                    if (!fromUser.equalsIgnoreCase(handle.rbtName)) {
                        try {
                            synchronized (handle.out) {
                                handle.out.println("INFORM: " + fromUser + ">"
                                        + msgOut);
                                showComm("INFORMING " + handle.rbtName
                                        + "\n\tmsg: " + msgOut);
                            }
                            handle.out.flush();
                        } catch (Exception e) {
                            showComm(e);
                        }
                    }
                }
            }
        }

        /**
         * @param fromUser
         * @param msgOut
         */
        //Implement the BROADCAST function to broadcast a message to all users.
        public void broadcast(String fromUser, String msgOut) {
            synchronized (clients) {
                //MsgHandler handle = (MsgHandler) clients.elementAt(0);
                //				if (!fromUser.equalsIgnoreCase(handle.rbtName)) {
                //					synchronized (handle.out) {
                //						handle.out.println("BROADCAST Msg to " + handle.rbtName
                //								+ "\n\tmsg: " + msgOut);
                //						showComm("BROADCAST Msg to " + handle.rbtName
                //								+ "\n\tmsg: " + msgOut);
                //					}
                //					handle.out.flush();
                //				} else {
                Enumeration list = clients.elements();
                while (list.hasMoreElements()) {
                    MsgHandler handle = (MsgHandler) list.nextElement();
                    if (!fromUser.equalsIgnoreCase(handle.rbtName)) {
                        try {
                            synchronized (handle.out) {
                                handle.out
                                        .println("BROADCAST Msg to "
                                                + handle.rbtName + "\n\tmsg: "
                                                + msgOut);
                                showComm("BROADCAST Msg to " + handle.rbtName
                                        + "\n\tmsg: " + msgOut);
                            }
                            handle.out.flush();
                        } catch (Exception e) {
                            showComm(e);
                        }
                    }
                }
            }
        }

        //Implement the WHO function
        public void who() {
            String whoIsOnline = new String("Robots Logged In: ");
            synchronized (clients) {
                {
                    Enumeration list = clients.elements();
                    while (list.hasMoreElements()) {
                        MsgHandler handle = (MsgHandler) list.nextElement();
                        showComm("Robotname: " + handle.rbtName + "From: "
                                + handle.socketIn.getPort());
                        synchronized (handle) {
                            whoIsOnline = new String(whoIsOnline.concat(" "
                                    + handle.rbtName));
                            if (handle.isReady())
                                whoIsOnline = new String(whoIsOnline
                                        .concat("\tStatus: ready\n"));
                            else
                                whoIsOnline = new String(whoIsOnline
                                        .concat("\tStatus: busy.\n"));
                        }
                    }
                }
                try {
                    out.println(whoIsOnline);
                    showComm(rbtName + " Requesting WHO");
                    showComm("\t" + whoIsOnline);
                } catch (Exception e) {
                    showComm(e);
                }
            }
        }

        /**
         * @param fromUser
         * @param toUser
         * @param msgOut
         */
        //Implement the WHISPER function to share secrets between users.
        public void whisper(String fromUser, String toUser, String msgOut) {
            synchronized (clients) {
                Enumeration list = clients.elements();
                while (list.hasMoreElements()) {
                    MsgHandler handle = (MsgHandler) list.nextElement();
                    try {
                        synchronized (handle.out) {
                            if (handle.rbtName.equals(toUser)) {
                                handle.out
                                        .println(fromUser + " says>" + msgOut);
                                showComm(fromUser + " WHISPERS TO " + toUser
                                        + ": " + msgOut);
                            }
                        }
                        handle.out.flush();
                    } catch (Exception e) {
                        showComm(e);
                    }
                }
            }
        }

        /**
         * @param fromUser
         * @param toUser
         * @param filename
         */
        //Implement the FILE function
        public void sendFile(String fromUser, String toUser, String filename) {
            showComm("Message begins with: 1");
            if (initiateSend(fromUser, toUser, filename)) {
                showComm("Message begins with: 2");
                String msg = new String("GO");
                sendPacket(fromUser, msg);
                while (true) {
                    try {
                        String message = new String(in.readLine());
                        showComm("FileTransfer by " + rbtName);
                        if (message.startsWith("STOP")) {
                            showComm("STOP received: " + message);
                            sendPacket(toUser, message);
                            break;
                        } else
                            sendPacket(toUser, message);
                    } catch (Exception e) {
                        showComm(e);
                    }
                }
            }
        }

        /**
         * @param fromUser
         * @param toUser
         * @param filename
         * @return
         */
        //Private function that sets up a file transfer to a user
        //Returns true if the user is found, false if the request is made to
        //an invalid user.
        private boolean initiateSend(String fromUser, String toUser,
                String filename) {
            synchronized (clients) {
                Enumeration list = clients.elements();
                while (list.hasMoreElements()) {
                    MsgHandler handle = (MsgHandler) list.nextElement();
                    try {
                        synchronized (handle.out) {
                            if (handle.rbtName.equals(toUser)) {
                                handle.out.println("FILE " + fromUser + " "
                                        + filename);
                                showComm("Requesting file transfer setup");
                                handle.ready = false;
                                return true;
                            }
                        }
                        handle.out.flush();
                    } catch (Exception e) {
                        showComm(e);
                    }
                }
            }
            return false;
        }

        /**
         * @param toUser
         * @param message
         */
        //Sends a packet to a user
        public void sendPacket(String toUser, String message) {
            synchronized (clients) {
                Enumeration list = clients.elements();
                while (list.hasMoreElements()) {
                    MsgHandler handle = (MsgHandler) list.nextElement();
                    try {
                        synchronized (handle.out) {
                            if (handle.rbtName.equals(toUser)) {
                                handle.out.println(message);
                                showComm("Packet sent to " + toUser
                                        + " : sendPacket: " + message);
                                return;
                            }
                        }
                        handle.out.flush();
                    } catch (Exception e) {
                        showComm(e);
                    }
                }
            }
            return;
        }

        /**
         * @return - the state variable
         */
        public boolean isReady() {
            return ready;
        }

        protected BufferedReader in;

        protected PrintWriter out;

        protected Socket socketIn;

        private String rbtName;

        private boolean validRobot;

        private boolean ready;
    }

    protected static Vector clients = new Vector(5, 5);
    
    private int status = 0;
}