package multitool;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JPanel;
import monitors.CommCenter;
import mns.MNSEvent;
/**
 * Author: Randall Borck -- Description: Program implements a UDP chat client
 * on port 7000.
 */
public class MultiClient implements ActionListener {
	/**
	 * @param listener
	 */
	public MultiClient(MultiClientListener listener) {
		this.listener = listener;
		startClient();
	}
	/**
	 *  
	 */
	public MultiClient() {
	}
	/**
	 *  
	 */
	public void startClient() {
		new ChatClientThread().start();
	}
	/**
	 * @param listener - @see MultiClientListener
	 */
	public void addListener(MultiClientListener listener) {
		this.listener = listener;
		startClient();
	}
	/**
	 * @param mnEvent - @see MNSEvent
	 */
	public void process(MNSEvent mnEvent) {
		if (mnEvent.getData() != null) {
			sendClientEvent((String) mnEvent.getData());
		}
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent event) {
		Object source = event.getSource();
		/**
		 * Execute server's LOGOFF request.
		 */
		if (source == quitButton) {
			String tmpString;
			tmpString = "BYE";
			try {
				out.println(tmpString);
			} catch (Exception exception) {
				CommCenter.showError("Multiclient: " + "quitButton Error: "
						+ exception);
			}
		}
		/**
		 * Error catching
		 */
		else {
			startClient();
			showComm("Multiclient: " + "Client Started");
		}
	}
 	/**
	 * @param message - String
	 *
	 * Sends input with the "BROADCAST" server command.
	 */
	public void sendClientEvent(String message) {
		String tmpString = "BROADCAST ";
		try {
			tmpString += message;
			out.println(tmpString);
			CommCenter.showDebug("Multiclient: " + "Sending event is over: "
					+ tmpString);
		} catch (Exception exception) {
			CommCenter.showError("Multiclient: " + "SendMessage Error: "
					+ exception);
		}
	}
	/**
	 * @param recipient - String
	 * @param command - String
	 * 
	 * Whispers the input to the designated user.
	 */
	public void whisperMethod(String recipient, String command) {
		String tmpString = "WHISPER ";
		try {
			tmpString += recipient + " " + command;
			out.println(tmpString);
			CommCenter.showDebug("Multiclient: " + tmpString);
		} catch (Exception exception) {
			CommCenter.showError("Multiclient: " + "WhisperMethod Error: "
					+ exception);
		}
	}
	/**
	 * @param filename - String
	 * @param recipient - String
	 * 
	 * Sends a file to the designated recipient
	 */
	public void fileSend(String filename, String recipient) {
		String tmpString;
		try {
			tmpString = new String("FILE " + " " + recipient + " " + filename);
			//open the file stream
			FileReader reader = new FileReader(filename);
			BufferedReader fin = new BufferedReader(reader);
			//Set the wait for GO flag
			sendFileOK = false;
			//Send the FILE send information to the server
			out.println(tmpString);
			CommCenter.showDebug("Multiclient: " + "File command sent: "
					+ tmpString);
			//Wait for "GO"
			CommCenter.showDebug("Multiclient: " + "Waiting for GO and sent: "
					+ tmpString);
			java.util.Date now = new java.util.Date();
			long startTime = now.getTime();
			//Waits for 1 second, then aborts if GO is not received
			while (!sendFileOK && ((now.getTime() - 1000) < startTime)) {
				//check time
				now = new java.util.Date();
			}
			CommCenter.showDebug("Multiclient: "
					+ "Done waiting for GO. tmpString = " + tmpString);
			if (sendFileOK) {
				CommCenter.showDebug("Multiclient: " + "Received GO signal0");
				while (true) {
					CommCenter
							.showDebug("Multiclient: " + "Received GO signal1");
					try {
						tmpString = new String(fin.readLine());
						CommCenter.showDebug("Multiclient: "
								+ "Received GO signal2: " + tmpString);
						tmpString = new String("SEND " + tmpString);
						out.println(tmpString);
						CommCenter.showDebug("Multiclient: "
								+ "Received GO signal3");
					} catch (Exception e) {
						tmpString = new String("STOP");
						out.println(tmpString);
						out.flush();
						CommCenter.showDebug("Multiclient: "
								+ "Received GO signal4: " + tmpString);
						break;
					}
				}
			} else
				CommCenter.showDebug("Multiclient: "
						+ "Didn't receive GO after 1 second. Aborting.");
		} catch (Exception e) {
			CommCenter.showError("Multiclient: " + "File sending error: " + e);
		}
	}
	/**
	 * Implements the client side of the "WHO" command.
	 */
	public void whoInquire() {
		String tmpString;
		try {
			tmpString = "WHO";
			out.println(tmpString);
		} catch (Exception exception) {
			CommCenter.showError("Multiclient: " + "whoInquire Error: "
					+ exception);
		}
	}
	/**
	 * @param o - print the object on the MultiTool screen
	 */
	private void showComm(Object o) {
				if (listener != null)
					listener.multiCommand(new MultiClientEvent("display: "+o.toString()));
				else CommCenter.showComm(o);
	}
	/**
	 * @author martin
	 * 
	 * TODO To change the template for this generated type comment go to Window -
	 * Preferences - Java - Code Generation - Code and Comments
	 */
	/**
	 * Creates a new thread to send/receive data from the server.
	 */
	private class ChatClientThread extends Thread {
		/**
		 * Constructs datagram socket.
		 */
		public ChatClientThread() {
			super();
			try {
				localPort = 0;
				host = InetAddress.getByName("localhost");
				conn = new Socket(host, toPort);
				in = new BufferedReader(new InputStreamReader(conn
						.getInputStream()));
				out = new PrintWriter(conn.getOutputStream(), true);
				CommCenter.showDebug("Multiclient: " + "server: " + host);
			} catch (Exception e) {
				CommCenter.showError("Multiclient: " + "Constructor Error: "
						+ e);
			}
		}
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {
			try {
				String tmpString = "REGISTER  MultiTool";
				CommCenter.showDebug("Sending:  " + tmpString);
				out.println(tmpString);
				/***************************************************************
				 * Incoming \
				 **************************************************************/
				while (true) {
					tmpString = new String(in.readLine());
					tmpString = tmpString.trim();
					CommCenter.showDebug("Multiclient: " + "Recieved packet: "
							+ tmpString);
					/**
					 * Server logoff signal.
					 */
					if (tmpString.toUpperCase().startsWith("GOOD-BYE!")) {
						conn.close();
						System.exit(0);
					} else if (tmpString.toUpperCase().startsWith("START")) {
						/**
						 * Start the processing of the whole group of robots
						 */
					    showComm(new MultiClientEvent("Multiclient: " + tmpString.trim()));
						listener.multiCommand(new MultiClientEvent("Start"));
					} else if (tmpString.toUpperCase().startsWith("GO")) {
						/**
						 * the indicator that the recipient acknowledges the file reception
						 */
						sendFileOK = true;
					} else if (tmpString.toUpperCase().endsWith("DONE")) {
						/**
						 * a robot from the lot finnished its task, the multitool must be acknowledged
						 */
						showComm("Multiclient: " + "Recieved DONE");
						listener.multiCommand(new MultiClientEvent("Done"));
					}
					else {
						/**
						 * Output the data to the debug window in the CommCenter.
						 */
					    showComm("Multiclient: " + tmpString.trim());
					}
				}
			} catch (Exception e) {
				CommCenter.showError("Multiclient: " + e);
			}
		}
	}
	private InetAddress host;
	private Socket conn;
	private PrintWriter out;
	private BufferedReader in;
	private boolean sendFileOK;
	private int localPort = 0;
	private int toPort = 7000;
	private JPanel buttonPanel;
	private JButton sendButton;
	private JButton whoButton;
	private JButton quitButton;
	private JButton fileButton;
	private JButton whisperButton;
	private MultiClientListener listener = null;
	private Vector files;
}