/*
 * Created on Jan 1, 2004
 */
package multitool;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Iterator;
import java.util.Vector;

import monitors.MultiPanel;

/**
 * @author Martin Lukac 
 * 
 * This is the main class of a single robot It provides
 *         tools and methods to add, remove and communicarte with members of the
 *         robot Note: this CNS, CNSRobotParts and subsequent devices have all
 *         command queues. This allows to generate certain type of coherence and
 *         a certain level of decoherence heheheheh enjoy
 */
public class MultiTool extends Thread implements MultiClientListener { // implements
    /**
     * Default Constructor this assusres now a order of different devices. This
     * has to be imporved in order to allow a bigger freedom of commands
     */
    public MultiTool(ActionListener listener) {//CommCenter commcenter) {
        super();
        this.listener = listener;
        handler = new CRLMultihandler(listener);
        parser = new CRLSAXParser(handler);
        showComm("Parser is up and Ready");
        // start the instruction command queue and thread*/
        this.start();
    }

    /*
     * (non-Javadoc)
     * 
     * @see robot.ChatClientListener#robotCommand(robot.ChatClientEvent)
     */
    synchronized public void multiCommand(MultiClientEvent ccEvent) {
        String s = ccEvent.getmessage();
        if (fQueue != null) {
            if (s != null && s.toLowerCase().startsWith("done")){
            mode = ACTIVE_MODE;
            if (counter > 0) counter--;
            else counter = 0;
            showComm("MultiTool recieved : " + s);           
            notifyAll();
            }
        } else if (s != null && s.toLowerCase().startsWith("display")){
            showComm(s.substring(s.indexOf(' '), s.length()));
        } else {
            mode = IDLE_MODE;
            showComm("Multitool recieved: inactive: "+s.length());
        }
    }

    /**
     * @param m
     */
    public void showComm(String m) {
        listener.actionPerformed(new ActionEvent(this, this.hashCode(), m));
    }

    /**
     * 
     */
    public void addClient() {
        this.mClient = new MultiClient();
        mClient.addListener(this);
    }

    /**
     * Each file loaded like this is fed COMPLETE to the brain careful with this
     * 
     * @param commandFile -
     *            loads a command file to the CNS
     */
    synchronized public void sendFiles() {
        //empty instruction queue
        this.fQueue = new Vector(handler.getScripts());
        showComm("Sending the script files");
        this.mode = SEND_MODE;
        notifyAll();
    }

    /**
     * The only public method to execute commands from a external parent such as
     * the CNS executes a command if it exists with the set of parameters
     * 
     * @param comm -
     *            name of the command
     * @param argument -
     *            argument to the command
     */
    synchronized public void executeGroupScript() {
        this.mode = ACTIVE_MODE;
        notifyAll();
    }

    /**
     * @param f
     */
    synchronized public void process(File f) {
        parser.process(f);
    }

    /***************************************************************************
     * END - Methods to control and communicate
     **************************************************************************/
    /**
     * overriden from Thread
     */
    public void run() {
        showComm("MultiTool is Ready");
        while (true) {
            try {
                synchronized (this) {
                    /** remove the first element form the command Queue */
                    String command;
                    while (mode == IDLE_MODE)
                        try {
                            wait(1000);
                        } catch (InterruptedException e) {
                        }
                         Iterator i = fQueue.iterator();
                        while (mode.equalsIgnoreCase(SEND_MODE) && i.hasNext()) {
                            MultiFile mFile = (MultiFile) i.next();
                            mClient.fileSend(mFile.getName(), mFile.getDest());
                        }
                        while (mode != null && mode.equalsIgnoreCase(ACTIVE_MODE) && i.hasNext()) {
                            MultiFile mFile = (MultiFile) i.next();
                            i.remove();
                            if (mFile.getsynchro()) {
                                mClient.whisperMethod(mFile.getDest(), "start");
                                counter++;
                                showComm("Parallel arbiter: " + counter);
                            } else {
                                while (counter != 0) {
                                    try {
                                        wait(100);
                                    } catch (InterruptedException e) {
                                    }
                                }
                                showComm("Serial arbiter");
                                mClient.whisperMethod(mFile.getDest(), "start");
                                counter++;
                                mode = IDLE_MODE;

                            }
                        }
                        showComm("Scripts executed and done");
                        mode = IDLE_MODE;
                }
            } catch (Exception e) {
                showComm("Multitool error: " + e);
                e.printStackTrace();

            }
        }
    }

    //counter of currently active robots
    private int counter = 0;

    private Vector mnsRobots;

    //current class is a thread;
    private Thread cnsThread;

    //listener
    private Vector cnsListeners;

    //commqnd queue
    private Vector fQueue = null;

    //private monitor
    private MultiPanel monitor;

    private String mode = IDLE_MODE;

    private ActionListener listener = null;

    private MultiClient mClient = null;

    private CRLSAXParser parser = null;

    private CRLMultihandler handler = null;
    
    public static String SEND_MODE = "sending";
    
    public static String ACTIVE_MODE = "start";
    
    public static String IDLE_MODE = "sleep";
    
    //public static String 
}