/*
 * Created on Jan 1, 2004
 */
package nns;

import java.util.Iterator;
import java.util.Vector;

import mns.BioDevice;
import mns.BioDeviceEvent;
import mns.MNS;
import mns.MNSEvent;
import mns.MNSListener;
import mns.MNScommandQueue;
import monitors.CommCenter;
import biocell.BioThread;
import biocell.EnergyPrimitive;
import brain.NetClient.RobotClient;
import brain.reflexgenerator.PMotionCalculator;
import brain.reflexgenerator.Translator;
import devs.musicpackage.DanceGenerator;
import devs.speechpackage.SpeechRecognizer;

/**
 * @author Martin Lukac
 * 
 * 
 * This is the main class of a single robot Sensory system This part acts in a
 * similar wway to the Spine - Cerebellum portion of the CNS. This is also the
 * bifurcation of the information where the sensor stimulus will be either
 * directly executed and/or sent in parallel to the brain system. Theoretically
 * tese two operations should be always executed in parallel but in that case
 * the inhibition of of prospective reflexes by the brain might be difficult to
 * achieve. Thus using switches and variablles one can indicates to the NNS
 * where the given signal should go.
 */
public class NNS extends BioThread implements BioSensorListener, MNSListener {

    /*
     * (non-Javadoc)
     * 
     * @see mns.MNSListener#process(mns.MNSEvent)
     */
    public void process(MNSEvent mnEvent) {

        // control the NNS
        if (mnEvent.getData() instanceof String) {
            String comm = (String) mnEvent.getData();
            comm = comm.substring(comm.indexOf('*')+1);
            CommCenter.showDebug(comm);
            if (comm.startsWith("stop")) {
                this.setEnabled(false);
                for (int a = 0; a < nnsModules.size(); a++)
                    ((BioSensor) nnsModules.get(a)).seState(BioDevice.DONE);
            } else if (comm.startsWith("start")) {
                for (int a = 0; a < nnsModules.size(); a++)
                    ((BioSensor) nnsModules.get(a)).seState(BioDevice.START);

                this.setEnabled(true);
            }
            // return;
        }
        // TODO here to add logic for output behavior of the robot
        CommCenter.showDebug("Sensors are: " + isEnabled());
        if (isEnabled()) {
            Iterator i = nnsModules.iterator();
            String destination = (String) mnEvent.getTarget();
            while (i.hasNext()) {
                Object ob = i.next();
                String o = ((BioSensor) ob).getname();
                if (o.startsWith(destination)) {
                    if (o.startsWith("netclient")) {
                        String data = (String) mnEvent.getData();
                        CommCenter.showDebug(o+" "+destination);
                        if (data.startsWith("INFORM")) {
                            ((RobotClient) ob).sendClientInform(data);
                        } else
                            ((RobotClient) ob).sendClientEvent((String) mnEvent
                                    .getData());
                    } else {
                        CommCenter.showDebug(o+" "+destination);
                        ((BioSensor)ob).process(mnEvent);
                        break;
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see nns.BioSensorListener#sensorCRLPercieved(nns.BioSensorEvent)
     */
    public void sensorCRLPercieved(BioSensorEvent event) {
        if (isEnabled()) {
            String message = event.getEventString();
            BioSensorListener list = (BioSensorListener) nnsListeners.get(0);
            list.sensorPercieved(new BioSensorEvent(message.substring(0,
                    message.indexOf('*')), message.substring(message
                    .indexOf('*') + 1)));
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see nns.BioSensorListener#sensorPercieved(nns.BioSensorEvent)
     */
    synchronized public void sensorPercieved(BioSensorEvent evt) {
        if (isEnabled()) {
            String message = evt.getEventString();
            String sender = (String) evt.getSource();
            // Manually enumrate all parts of the robot that send bio events.
            if (sender.startsWith("vision")) {
                // BioSensorListener list =
                // (BioSensorListener)nnsListeners.get(0);
                System.out.println("By majority the motion detected: "
                        + message);
                // list.sensorPercieved(new BioSensorEvent("NNS","stop"));
            } else if (sender.startsWith("motion")) {
                // BioSensorListener list =
                // (BioSensorListener)nnsListeners.get(0);
                System.out.println("By majority the motion detected: "
                        + message);
                // list.sensorPercieved(new
                // BioSensorEvent("NNS","neck*random"));
                processMotion(message);
            } else if (sender.startsWith("speech")) {
                processSpeech(message);
            } else if (sender.startsWith("net")) {
                processNet(message);
            }
//            else if (sender.startsWith("dance")) {
//            	setMotState(message);
//            }
        }

    }

    /**
     * Default Constructor this assusres now a order of different devices. This
     * has to be imporved in order to allow a bigger freedom of commands
     */
    public NNS(BioSensorListener listener) {

        super("NNS");
        nnsModules = new Vector(1, 1);
        nnsListeners = new Vector(1, 1);
        // implementing for the facility as a biosensorlistener
        // the NNS class is connected to the MNScommandqueue so as it can
        // directly
        // insert commands generated by simple logical rules or reflex loops
        nnsListeners.add(listener);
        //addnnsModule(new SpeechRecognizer(this));
        addnnsModule(new PMotionCalculator(new Translator(this)));
        addnnsModule(new DanceGenerator(this));
        // addnnsModule(new Milford5120("/dev/ttyS1", this));

        // start the instruction command queue and thread*/
        start();
    }

    /**
     * @return the array of states of the motor system of the robot
     */
    public int[] getMotState() {
        return ((MNS) getListener().get(1)).getMotState();
    }

    public void setMotState(int[] motstates) {
        if (((MNScommandQueue) getListener().get(0)).getSize() < 10)
            ((MNS) getListener().get(1)).setMotState(motstates);
    }

    private void processMotion(String message) {
        ((BioSensorListener) nnsListeners.get(0))
                .sensorPercieved(new BioSensorEvent("mns", message));
    }

    private void processNet(String message) {
        ((BioSensorListener) nnsListeners.get(0))
                .sensorPercieved(new BioSensorEvent("mns", message));

    }

    private void processSpeech(String message) {
        int a = getBioByName("alice");
        ((BioSensorListener) nnsModules.get(a))
                .sensorPercieved(new BioSensorEvent("mouth", message));
    }

    private int getBioByName(String name) {
        int n = 0;
        while (n < nnsModules.size()) {
            BioSensor b = (BioSensor) nnsModules.get(n);
            if (b.getname().startsWith(name))
                return n;
            n++;
        }
        return -1;
    }

    private String generateCommand(String keyword) {
        Vector commands = ((MNS) nnsListeners.get(0)).getComData();
        int counter = (int) (commands.size() * Math.random());

        return null;
    }

    /**
     * @param sensor
     */
    public void addnnsModule(BioSensor sensor) {
       // sensor.addListener(this);
        nnsModules.add(sensor);
    }

    /**
     * @param actuator
     */
    public void addnnsModule(MNSListener actuator) {
        // sensor.(this);
        nnsModules.add(actuator);
    }

    /**
     * @return Returns the listener.
     */
    public Vector getListener() {
        return nnsListeners;
    }

    /**
     * @param listener
     *            The listener to set.
     */
    public void setListener(BioSensorListener listener) {
        nnsListeners.add(0, listener);
    }

    /**
     * @param listener
     *            The listener to set.
     */
    public void addListener(BioSensorListener listener) {
        nnsListeners.add(listener);
    }

    /**
     * @return Returns the sensors.
     */
    public Vector getSensors() {
        return nnsModules;
    }

    /**
     * @return Returns the sensors.
     */
    public BioSensor getSensor(String key) {
        Iterator i = nnsModules.iterator();
        while (i.hasNext()) {
            BioSensor sensor = (BioSensor) i.next();
            String name = sensor.getname();
            //System.out.println(name);
            if (name.trim().startsWith(key))
                return sensor;
        }
        return null;
    }

    /**
     * @param sensors
     *            The sensors to set.
     */
    public void setSensors(Vector nnsModules) {
        this.nnsModules = nnsModules;
    }

    /***************************************************************************
     * End of Constructors of the robot from file
     **************************************************************************/
    /***************************************************************************
     * Methods to control and communicate with the robot - only upward stream
     **************************************************************************/

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        while (true) {
            synchronized (this) {
                try {
                    long tim = 100 * ((long) getMoodParam("waittime"));
                    wait(tim);
                } catch (Exception e) {

                }
                // ((MNS)nnsListeners.get(0)).get;
                // BioSensorListener listener =
                // (BioSensorListener)nnsListeners.get(0);
                // DynamicMood dm = getCurrentmood();
                // System.out.println("got the first listener");
            }
        }
    }

    /**
     * 
     */
    public void test() {
    }

    // connected Vectors continuously (as far as possible) generating and
    // collecting data from the environment
    // not that these sensors are linked by a different protocol than
    // networkclient, netdigger, or so
    // : these are the native sensors,....
    private Vector nnsModules;

    // private monitor
    private Vector nnsListeners;

    // output enabled
    private boolean enabled = false;

    // shotrcut to MNS commands - the tree of commands
    private Vector commands;

    public static void main(String[] args) {
        new NNS(null);
    }

    /**
     * @return Returns the enabled.
     */
    public boolean isEnabled() {
        return enabled;
    }

    /**
     * @param enabled
     *            The enabled to set.
     */
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /*
     * (non-Javadoc)
     * 
     * @see mns.BioDeviceListener#bioEvent(mns.BioDeviceEvent)
     */
    public void bioEvent(BioDeviceEvent bdE) {
        // TODO Auto-generated method stub

    }

    /*
     * (non-Javadoc)
     * 
     * @see biocell.BioEnergySpender#energyConsumed(biocell.EnergyPrimitive)
     */
    public void energyConsumed(EnergyPrimitive ep) {
        // TODO Auto-generated method stub

    }
}