/*
 * Created on Apr 4, 2005
 *
 * The register of all possible moods with regards to a neccessary-evil classification
 */

package biocell;

import java.util.Vector;


/**
 * @author martin lukac
 *
 * Dynamic mood - the main tool behind the bio extension of this software.
 * It provides the interface to Energy and Strategy and is also prepared
 * to be easily connected to the quantum simulator,
 * DynamicMood is the main parameter in the adaptive process of the robot
 * 
 */
public class DynamicMood extends Thread implements EnergyPrimitivesListener{
	

	//parameters for various parts of the robot. later to be exported
	//all is in double in range [0,1]
	//all numbers are only ratios and percentages. They are to be the
	// result of more
	//elaborate strategy to generate behaviors in space and time
	//TODO change to interval [-1,1]
	
	

	/**
	 * Comment for <code>bioname</code>
	 */
	public String bioname;

	/**
	 * Comment for <code>dependency</code>
	 */
	public String dependency;
	
	
	/**
	 * @param name
	 */
	public DynamicMood(String name) {
		super(name);
		bioname = name;
		parameters = new Vector(1,1);
		energyPredictor = new Vector(1,1);
		strategyPredictor = new Vector(1,1);
		setTemplateMood(bioname);
		GlobalState.addCurrentMood(this);
	}

	/**
	 * @param name
	 */
	public DynamicMood() {
		super();
		parameters = new Vector(1,1);
		energyPredictor = new Vector(1,1);
		strategyPredictor = new Vector(1,1);
	}

	/* (non-Javadoc)
	 * @see biocell.EnergyPrimitivesListener#energyModified(biocell.EnergyPrimitive)
	 */
	public void energyModified(EnergyPrimitive Ep) {
		// TODO Auto-generated method stub

	}
	/**
	 * @return
	 */
	public String getname(){
		return bioname;
	}

	/**
	 * @param name
	 */
	public void setname(String name){
		this.bioname = name;
	}
	/**
	 * sets the dependency of this energy module to a given device 
	 * @param dependency
	 */
	public void setdependency(String dependency){
		this.dependency = dependency;
	}
	
	private Vector parameters;
	private Vector energyPredictor;
	private Vector strategyPredictor;
	


	/**
	 * @return Returns the energyPredictor.
	 */
	public double[] getArrayPredictor(int register) {
		
		Object test[];
		
		switch(register){
			case 0:
				test = this.parameters.toArray();
				break;
			case 1:
				test = this.energyPredictor.toArray();
				break;
			case 2:
				test = this.strategyPredictor.toArray();
				break;
			default:
				test = this.parameters.toArray();
			break;
		}

		int t = 0;
		double result[] = new double[test.length];
		while(t<test.length){
			result[t] = ((MoodParam)test[t]).getv();
			t++;
		}
		//notifyAll();
		return result;
	}
	
	/**
	 * 
	 * sets the energyPredictor.
	 * 
	 * @param register
	 * @param reist
	 */
	public void setArrayPredictor(int register, double []reist) {
		Vector test;
		switch(register){
			case 0:
				test = parameters;
				break;
			case 1:
				test = energyPredictor;
				break;
			case 2:
				test = strategyPredictor;
				break;
			default:
				test = parameters;
			break;
		}

		int t = 0;
		while(t<reist.length && t < test.size()){
			MoodParam m = (MoodParam)test.get(t);
			m.setv(reist[t]);
			t++;
		}
	}

	

	/**
	 * 
	 * tries to remove an element from all three data vectors and returns boolean confirmation of wether the operation was succeful
	 * 
	 * @param name
	 * @return
	 */
	public boolean remParameter(String name){
		int n = 0;
		while(n < parameters.size()){
		//parameters.g
			MoodParam dn = ((MoodParam)parameters.get(n));
			if (name.equalsIgnoreCase(dn.getn())){
				parameters.remove(n);
				energyPredictor.remove(n);
				strategyPredictor.remove(n);
				return true;
			}
		}
		//notifyAll();
		return false;
	}

	/**
	 * query for returning a parameter from the mood control field,
	 * in case of non success returns -100
	 * 
	 * @param name
	 * @return
	 */
	public double getParameter(String name){
		int n = 0;
		while(n < parameters.size()){
		//parameters.g
			MoodParam dn = ((MoodParam)parameters.get(n));
			if (name.equalsIgnoreCase(dn.getn())){
				//notifyAll();
				return dn.getv();
			}
			n++;
		}
		//notifyAll();
		return -100;
	}

	/**
	 * 
	 * returns one of the stored datafields in the following order:
	 * 0 - mood parameters
	 * 1 - energy parameters
	 * 2 - strategy parameters
	 * 
	 * @param index
	 * @return
	 */
	public Vector getParameters(int index) {
		switch(index){
		case 0:
			return parameters;
		case 1:
			return energyPredictor;
		case 2:
			return strategyPredictor;
		default:
			return parameters;
		}
	}
	
	/**
	 * sets a given parameter in the mood control field
	 * the given data type must previously exists
	 * 
	 * @param name
	 * @param value
	 */
	synchronized public void setParameter(String name, double value){
		for (int a = 0; a < parameters.size(); a++){
			MoodParam mp = (MoodParam)parameters.get(a);
			if (mp.getn().equalsIgnoreCase(name)){
				mp.setv(value);
				break;
			}
		}
	}
	
	/**
	 *  adds a parameter structure to all three fields
	 * 
	 * @param name
	 * @param value
	 */
	public void addParameter(String name, double value){
		parameters.add(new MoodParam(name, value));
		energyPredictor.add(new MoodParam(name, value));
		strategyPredictor.add(new MoodParam(name, value));
		//notifyAll();
	}

	/**
	 * resets all parameters field to 0 size
	 * 
	 * @param name
	 * @param value
	 */
	public void cleanParameter(){
		parameters = new Vector(parameters.size());
		energyPredictor = new Vector(energyPredictor.size());
		strategyPredictor = new Vector(strategyPredictor.size());
		//notifyAll();
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
	}

	
	/**
	 * sets the mood to a predefined template
	 * 
	 * @param name
	 */
	synchronized public void setTemplateMood(String name){
		Vector moods = GlobalState.getMoods();
		for (int a = 0; a < moods.size(); a++) {
			DynamicMood d = (DynamicMood) moods.get(a);
			if (d.bioname.equalsIgnoreCase(name)) {
				this.parameters.clear();
				Vector init = d.getParameters(0);
				for (int n = 0; n < init.size(); n++){
					MoodParam p = (MoodParam)init.get(n);
					this.addParameter(p.getn(), p.getv());
				}
				this.bioname = name;
				notifyAll();
				return;
			}
		}

		DynamicMood d = (DynamicMood) moods.get(0);
		//get the field represeinting the current mood 
		Vector init = d.getParameters(0);
		this.parameters.clear();
		for (int n = 0; n < init.size(); n++){
			MoodParam p = (MoodParam)init.get(n);
			this.addParameter(p.getn(), p.getv());
			this.bioname = d.bioname;
		}
	}

	/**
	 *  dumps the mood to the stdin
	 */
	public void printMood(){
		for (int a = 0; a < this.parameters.size(); a++){
			MoodParam p = (MoodParam)this.parameters.get(a);
			System.out.println(p.getn()+"  "+p.getv());
		}			
	}


}

