package biocell;

import monitors.CommCenter;


/**
 
 
 Class Energy
 the bottom class of the agent. 
 the low level information processing unit. USed for processing 
 energy and information parsing concerning the internal state. 
 Perceptron is connected to the environment and sends information
 to the agent deeper level of prediction. At the end of one cycle
 the agent executes commands via another neural network
 
 -multi-qualitative objects perception
 -creating of complex representation
 -predicting
 
 2002 by Martin Lukac
 
 **/



//class taking care of all quasi biological processes
//all signal are also passing through it

public class Energy extends Thread implements EnergyPrimitivesListener{
	private Strategy strategy = null;
	/**
	 * Current energy: <code>nrg</code>
	 */
	public double nrg;
	
	/**
	 * optimal energy  <code>optnrg</code>
	 */
	public double optnrg = 1;
	
	/**
	 * the step energy consumption coefficient <code>nrgConsCoeff</code>
	 */
	private double nrgConsCoeff = 0.0001;
	
	/**
	 * the unit step <code>stepenergy</code>
	 */
	private double stepenergy = 0.0001;
	
	/**
	 * the length of the radius memory <code>rlength</code>
	 */
	private int rlength = 1000;
	
	/**
	 * radius of evaluation function <code>radius</code>
	 */
	private double[] radius;
	
	/**
	 * Comment for <code>nrgs</code>
	 */
	private double [] nrgs;
	/**
	 * The energy thread check the parasited thread for activity 
	 * in various time interval based on how well the module is doing
	 *  <code>timeStep</code>
	 */
	private long timeStep = 10000;
	
	/**
	 * the BioThread in supervision <code>thread</code>
	 */
	private Object BioCog = null;
	
	/**
	 * @param thread
	 */
	public Energy(BioThread thread) {
		
		super();

		this.BioCog = thread;

		//generate the sigam of random size in [-1,1]
		this.radius = new double[rlength];
		this.nrgs = new double[rlength];
		for (int a = 0; a < rlength; a++){
			this.radius[a] = (Math.random() * 2);
			this.nrgs[a] = (Math.random() * 2);
		}
		this.start();
		
	}
	
	/**
	 * @param thread
	 */
	public Energy(BioCell biocell) {
		
		super();
		//setting initial energy

		this.BioCog = biocell;
		
		//generate the sigam of random size in [-1,1]
		radius = new double[rlength];
		this.nrgs = new double[rlength];
		for (int a = 0; a < rlength; a++){
			this.radius[a] = (Math.random() * 2);
			this.nrgs[a] = (Math.random() * 2);
		}
		this.start();
		
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 * 
	 * this is the unitary computation of the energy modification
	 * it is based on the fact of simple substraction ofunitary step
	 * as defined by the definition.
	 */
	public void run(){
		
		while(true){
			synchronized (this){
				try{
					wait(timeStep);
				}catch (Exception e){

				}
				double a = 1;
				
				if (BioCog instanceof BioThread){
					BioThread thread = (BioThread)BioCog;
					DynamicMood dm = thread.getCurrentmood();
//TODO debug energy in order to accumulate more slowly and to degrade with time
					nrg = dm.getParameter("energy");
					
					if (thread.getState() == Thread.State.NEW){
						//pay the price for creation
						a = (int)(nrg*3)/Thread.State.NEW.name().hashCode();
					} else if (thread.getState() == Thread.State.RUNNABLE){
						//normal execution is expensive
						a = (int)(nrg*4)/Thread.State.RUNNABLE.name().hashCode();
					} else if (thread.getState() == Thread.State.TIMED_WAITING){
						//timed sleeping is more expensive
						a = (int)(nrg*2)/Thread.State.TIMED_WAITING.name().hashCode();
					} else if (thread.getState() == Thread.State.WAITING){
						//sleeping state
						a = (int)(nrg*1)/Thread.State.WAITING.name().hashCode();				
					} else if (thread.getState() == Thread.State.BLOCKED){
						//the other extreme of inactivity is overactivity
						a = (int)(nrg*5)/Thread.State.BLOCKED.name().hashCode();
					} else if (thread.getState() == Thread.State.TERMINATED){
						
					}
					//get the current configuration/information/perception
					//this is the place where the perceptron should be inserted
					timeStep = (long)(10000*dm.getParameter("waittime"));
	
					//consume unitary energy value for data retrieval/perception
					econs();
					
					//MoodParam.parameters field
					double data[] = dm.getArrayPredictor(0);
					
					if (data.length != 0){
						//MoodParam.energyprocesor field
						double [] predict = Eval(data);
						
						//activate strategy calculation
						for (int b = 0;b < predict.length; b++)
							predict[b] = a*predict[b];
						
						update(a);
						
						//write the updated energy fields in the action shape
						dm.setArrayPredictor(1, update(predict));
						CommCenter.showBioData("Energy: '"+dm.getName()+" dependency: "+dm.dependency+"',  real evaluated: "+nrgfunct(nrg)+", real: "+nrg);
						
						if(strategy == null)
							strategy = new Strategy(dm);
						//TODO check the impact of the strategy on the energy consumption and the feedback
						else
							strategy.step(singleReturnEval(data));
					}
					dm.setParameter("energy", nrg);
				} else if (BioCog instanceof BioCell){
					BioCell noThread = (BioCell)BioCog;
					//get the current configuration
					DynamicMood dm = noThread.getCurrentmood();
					nrg = dm.getParameter("energy");
					
					timeStep = (long)(10000*dm.getParameter("waittime"));
					//MoodParam.parameters field
					double data[] = dm.getArrayPredictor(0);

					//consume unitary energy value for data retrieval/perception
					econs();
					if (data.length != 0){

						// MoodParam.energypredictor field
						double [] predict = Eval(data);
		
						//write the updated energy fields in the action shape
						dm.setArrayPredictor(1, update(predict));
							
						//activate strategy calculation
						CommCenter.showBioData("Energy:"+dm.getName()+" dependency: "+dm.dependency+" evaluated: "+nrgfunct(nrg)+" amount: "+nrg);
						
						if(strategy == null)
							strategy = new Strategy(dm);
						else
							strategy.step(singleReturnEval(data));
					}
					dm.setParameter("energy", nrg);
				}
			}		
		}
	}

	
	/* (non-Javadoc)
	 * @see biocell.EnergyPrimitivesListener#energyModified(biocell.EnergyPrimitive)
	 */
	public void energyModified(EnergyPrimitive Ep) {
		double temp = Ep.getValue();
		nrg += optnrg/Ep.getName().hashCode();
		nrg -= (optnrg/Ep.getName().hashCode())*Math.random();
		//System.out.println("Energy: "+nrg+" consumed: "+(optnrg/Ep.getName().hashCode())*Math.random()+" <> "+Ep.getName());
	}

	
	/**
	 * unit energy consumption
	 */
	public void econs(){		
		nrg -= optnrg*nrgConsCoeff*Math.random();
	}
	
	/**
	 * @return
	 */
	public double state(){
		return (optnrg/(10*nrg));
	}
	/**
	 * consumption of the energy for each action or time period
	 * and generation of the next step strategy. In the DynamicMood Register
	 * the original field is replaced by the new one, or some elements are updated
	 * the total succesfulness is in the strategypredictor field
	 */
	private void update(double state) {
		// subtracts a random amount of energy
		this.nrg += state;
		nrg -= optnrg * nrgConsCoeff * Math.random();
		//TODO -- fix energy consumption
		//double predicted = unitEval();
	}
	
	/**
	 * consumption of the energy for each action or time period
	 * and generation of the next step strategy. In the DynamicMood Register
	 * the original field is replaced by the new one, or some elements are updated
	 * the total succesfulness is in the strategypredictor field
	 */
	private double[] update(double state[]) {
		double result[] = new double [state.length];
		double temnrg = nrg;
		// subtracts a random amount of energy related to the action
		for (int a = 0; a < state.length; a++){
			nrg += state[a] * nrg * nrgConsCoeff;
			nrg -=  nrg * nrgConsCoeff * Math.random();
		}
		//making energy prediction basewd on the comopuation of the arguments
		for (int a = 0; a < state.length; a++) {
			result[a] = nrg * state[a] * nrgConsCoeff
			* Math.sqrt(Math.pow((a*state[a]*nrg* Math.random() - (a + 1)*state[a]*nrg* Math.random()),2));
		}
		
		//setting the real value for the amount of food [0..1] by Gaussian curve
		for (int a = 0; a < result.length; a++) {
			if (nrgfunct(nrg) < nrgfunct(result[a]))
				result[a] = nrgfunct(result[a]);
			else result[a] = 1 - nrgfunct(result[a]);
		}
		return result;
	}
	/**
	 * evaluating current energetic state and making some basical predictions based on AR
	 * @return
	 */
	private double[] Eval(double [] signal) {
		//energy evaluator for the whole function
		double result = 1;
		//energy evaluator for individual actions
		double res [] = new double[signal.length];
		//the mood representation - working copy
		double[] predict = new double[signal.length];
		//making energy prediction basewd on the comopuation of the arguments
		//predict[0] = 5 * signal[0] + (nrg - Math.random());
		for (int a = 0; a < signal.length; a++) {
			predict[a] = 5 * signal[a] + (nrg - Math.random());
		}
		//setting the real value for the amount of food [0..1] by Gaussian curve
		for (int a = 0; a < predict.length; a++) {
			res[a] = nrgfunct(predict[a]);
		}
		return res;
	}

	/**
	 * evaluating current energetic state and making some basical predictions based on AR
	 * @return
	 */
	private double singleReturnEval(double [] signal) {
		//energy evaluator for the whole function
		double result = 1;
		//the mood representation - working copy
		double[] predict = new double[signal.length];
		//making energy prediction basewd on the comopuation of the arguments
		//predict[0] = 5 * signal[0] + (nrg - Math.random());
		for (int a = 0; a < signal.length; a++) {
			predict[a] = 5 * signal[a] + (nrg - Math.random());
		}
		//setting the real value for the amount of food [0..1] by Gaussian curve
		for (int a = 0; a < predict.length; a++) {
			result *= nrgfunct(predict[a]);
		}
		return result/predict.length;
	}

	
	/**
	 * evaluating current energetic state
	 * and making some basical predictions based on AR
	 * We floor the function values to 10e20
	 * @return
	 */
	public double unitEval() {
		double result = 0;
		double[] predict = new double[rlength];
		//making prediction about energy
		predict[0] = Math.sqrt(Math.pow((nrg - optnrg),2));
		for (int a = 1; a < rlength; a++) {
			predict[a] = nrg * Math.random()
			* Math.floor(Math.sqrt(Math.pow(a*predict[a - 1] * Math.random() - ((a + 1) * Math.random()*predict[a - 1]), 2))+1);
		}
		for (int a = 0; a < predict.length; a++) {
			//setting the real value for the amount of food [0..1]
			//by Gaussian curve
			predict[a] = nrgfunct(predict[a]);
			result += predict[a];
		}
		//	System.out.println(result +" energy ");
		return result/predict.length;
	}
	
	
	
	/**
	 * evaluating current energetic state and making some basical predictions based on AR
	 * @return
	 */
	private long EvalTime(long period) {
		long result = 0;
		double predict = (double)period;
		//making energy prediction
		predict = 5 * period + (nrg - Math.random());
		//setting the real value for the amount of food [0..1] by Gaussian curve
		result = period * (long)nrgfunct(predict);
		return result;
	}
	
	/**
	 * function evaluating the current energy neccessity compared to the optimal value
	 * @param val
	 * @return
	 */
	private double nrgfunct(double val) {
		double radius = Math.pow((optnrg - nrg), 2);
		//testing this method
		radius = shiftVal(radius, this.radius);
		return (Math.exp(-Math.pow(val - optnrg, 2))/(2 * Math.pow(radius, 2)));
	}
	
	/**
	 * @param a
	 * @param arr
	 * @return
	 */
	private double shiftVal(double a, double[] arr) {
		double dump = 0;
		//left shifting
		for (int b = 1; b < arr.length; b++) {
			arr[b] = arr[b-1];
			dump += arr[b - 1];
		}
		arr[0] = a;
		dump += arr[0];
		dump = dump / arr.length + 1;
		return dump;
	}
	
}

