package biocell;


import java.util.Vector;

import monitors.CommCenter;

/**
 * @author martin lukac
 * 
 * This class represents the control module of the energy based model. This
 * class should not be called by user classes but the
 * @BioCell and the
 * @BioEnergySpender interface should beused. This is the decision making
 *                   module. It is not a brain but rather a unitary cell of self
 *                   controlability and adaptation. It uses
 * Energy wich should be connected in loop .
 */
public class Strategy {
	
	
	
	/**
	 * Comment for <code>SCenter</code>
	 */
	private double SCenter = 1;
	
	/**
	 * <code>lambda</code> the error exponentially influencing the depth of the window 
	 */
	private double lambda = 0.3;
	
	/**
	 * <code>Radius</code> - evaluation function radius
	 */
	private double Radius = 100;
	/**
	 * <code>lastAct</code> - last Activity score
	 */
	private double lastAct = 100;
	/**
	 * strategy between Inf and 1 <code>Strategy</code>
	 */
	private double Strategy  = 100;
	
	/**
	 * the last strategy <code>lastStra</code>
	 */
	private double lastStra = 5;
	
	/**
	 * the unit step <code>stepstrategy</code>
	 */
	private double stepstrategy = 0.001;
	/**
	 * <code>Dynamic Moods as xecuted in space and time - vector of arrays</code>
	 */
	private Vector actions = new Vector();
	
	/**
	 * <code>memSize</code> - the size of the memory
	 */
	private int memSize = 10000;
	/**
	 * <code>actionMemSize - the size of the working memory of the moods</code>
	 */
	private int actionMemSize = 500;
	/**
	 * <code>moodParameters</code> - the mood this strategy is working on
	 */
	private DynamicMood moodParameters;
	
	/**
	 * <code>lastAction</code> - vector
	 */
	private double[] lastAction;
	/**
	 * <code>radius</code> - the radius of function evaluated and the step
	 */
	private double[] radius = new double[memSize];
	
	/**
	 * <code>strategy</code> - array of strategy memory
	 */
	private double[] strategy = new double[memSize];
	
	
	/**
	 * sample constructor using default values for the function radius
	 * @param e
	 */
	Strategy(DynamicMood moodParameters) {
		this.moodParameters = moodParameters;
		//the memory size is set, so now fill it
		double raw[] = moodParameters.getArrayPredictor(0);
		double energy[] = moodParameters.getArrayPredictor(1);
		actions = new Vector();
		//init the strategy with the current mood - fill the memory
		for (int a = 0; a< actionMemSize; a++){
			double perturbation[] = new double[energy.length];
			for(int y = 0; y < raw.length; y++)
			    //the strategy memory initialization is almost random functional evaluation
				perturbation[y] = stFunction(raw[y]*energy[y]*Math.random());
			actions.add(new StrategyParam(perturbation));
		}
		//generate the one step action memory
		lastAction = new double[moodParameters.getArrayPredictor(3).length];
		//generate the memory and current variables for Strategy
		// Radius of the Strategy evaluation, and initiate lastAction long term memory
		Strategy = InitAc(strategy, Strategy);
		Radius = InitAcSimple(radius, Radius);
		lastAct = InitAc(lastAction, lastAct);
		CommCenter.showBioData("Strategy initiated: "+Strategy+" "+Radius+" "+lastAct);
	}
	/**
	 * 
	 */
	public void step(double amount) {
		CommCenter.showBioData("Strategy step");
		//compares energy evaluation with the current center
		double inStrgy = stFunction(amount);
		//predicting actions based on current input and current internal
		//Strategy predictions
		makeActions(Strategy+inStrgy);
		updateStrategy(stFunction(stFunction(inStrgy*Strategy*amount)));
		CommCenter.showBioData("Strategy done and updated by "+inStrgy+"  "+Strategy+" "+lastAct);
	}
	
	
	/**
	 * execute Actions
	 * input the curent incoming strategy coupled with all previous under-levels
	 * @param ext
	 */
	private void makeActions(double ext) {
		//dummy variable - the last action in the time line
		double[] order = ((StrategyParam)actions.get(1)).getv();
		//the evaluated actions according to current strategy
		double[] inter = new double [order.length];
		//evaluate the actions 
		
		//the best action to take
		int counter = 0;
		for (int f = 0; f < order.length; f++){
//			System.out.print(order[f]+"  ");
			inter[f] = Math.sqrt(Math.pow((order[f] * ext - Strategy), 2));
			if (stFunction(inter[f]) < Strategy)
				counter = f;
		}
		
//		System.out.println();
		double result[];
		
		//execute the action - modify all concerned fields
			result = updateAction(inter[counter], inter);
	}
	
	/**
	 * @param valuex
	 *            specific method for updating the strategy
	 */
	private void updateStrategy(double value) {
		int range = Window();
		
		lastStra = Strategy;
		
		
		//update strategy for this state even if direction will not be taken
		CommCenter.showBioData("Strategy: updating1X: "+value+"  "+Strategy+"  "+Radius+" "+range+" "+lambda);
		Strategy = stFunction(updatePrimitives(value, strategy, range)); 
		CommCenter.showBioData("Strategy: updating1X: "+value+"  "+Strategy+"  "+Radius+ " "+lambda);
		Radius = Math.sqrt(Math.pow((value - Strategy), 2));
		CommCenter.showBioData("Strategy: updating1X: "+value+"  "+Strategy+"  "+Radius);
		Radius = updatePrimitives(Radius, radius, range);
		CommCenter.showBioData("Strategy: updating1X: "+value+"  "+Strategy+"  "+Radius);
	}
	
	/**
	 * @param action
	 * @param Act
	 * @return
	 */
	private double updatePrimitives(double action, double[] Act, int depth) {
		double result = 0;
		for (int a = Act.length - 1; a > 0; a--) {
			Act[a] = Act[a - 1];
		}
		Act[0] = action;
		
		for (int a = 1; a < depth-1; a++)
			result += Math.sqrt(Math.pow(Act[a]-Act[a-1],2)) * Math.pow(lambda, (a));
			//result += Act[a] * Math.pow(lambda, (a + 1));
		result /= depth;
		return result;
	}
	
	
	/**
	 * @param val
	 * @return
	 */
	private double stFunction(double val) {
		return Math.sqrt((Radius * Radius) + Math.pow((val - SCenter), 2)) / Radius;
	}
	
	/**
	 * @param al
	 * @param fixe
	 * @return
	 */
	private double ZeroToInf(double al, double fixe) {
		return Math.sqrt(Math.pow((al - fixe), 2));
	}
	
	/**
	 * @return - the size of the history window based on the length of 
	 * the strategy[] array. 
	 * Also updates lambda, factor interferring directly wo the update
	 * of all parameters
	 */
	private int Window() {
		//lambda
		lambda = 1 / Strategy * lambda;
		//history - worst is the performence more there is search 
		//in the time line
		int range;
		if (Strategy < 1)
		     range = strategy.length;
		else
		     range = (int) (strategy.length - (strategy.length / (Strategy+1)));
		if (range <= 1)
			range = 1;
		return range;
	}
	/**
	 * This method updates the memory of the automaton 
	 * and writes the decided strategy so as the decided actions
	 * in the Dynamic Mood
	 * 
	 * @param best - the value of the best strategy
	 * @param action - the action field
	 * @return the parameters of strategy vector
	 */
	private double[] updateAction(double best, double action[]) {
		//get the current commands/settings
		double temp[] = ((StrategyParam)actions.get(0)).getv();

		//get the window
		int depth = Window()*actionMemSize/strategy.length;
		if (depth <= 1)
			depth = 1;
		
		//do the best - maximize the reward
		int n = 0;
		for (int f = 1; f < action.length; f++){
			if (stFunction(action[f]) < stFunction(action[f-1]))
				n = f;
		}
		
		//apply the coefficients
		if(best < Strategy)
			temp[n] += stepstrategy;
		else 
			temp[n] -= stepstrategy;
		
		//generate the result
		double resulta[] = new double[temp.length];
		for (int b = 0; b < action.length; b++) {
			resulta[b] = temp[b];
		}
		
		//add the decided action vector - local memory
		actions.add(0, new StrategyParam(resulta));

		//cut to correct size
		while (actions.size() > actionMemSize){
			actions.remove(actions.size()-1);
		}
		
//		System.out.println();
		double result = 0;
		int a = 0;
		CommCenter.showBioData("Strategy: Action updating1X: "+actions.size()+"  "+depth);
		
//		//generate the new set of oefficients
//		while(a < actions.size() && a < depth){
//			
//			for (int b = 0; b < temp.length; b++) {
//				//modify the final action  - the strategy is reinforced through
//				//the error (lambda) and of the window size
//				resulta[b]  += temp[b]* Math.pow(lambda, (a));
//			}
//			a++;
//			temp = ((StrategyParam)actions.get(a)).getv();
//		}
		//normalize it to 1/n 
//		for (int b = 0; b < temp.length; b++) {
//			System.out.print(resulta[b]+" ");
//		}
//		System.out.println();

		//set the decided strategy
		moodParameters.setArrayPredictor(2, resulta);
		moodParameters.setParameter("strategy", Strategy);
		return resulta;
		
	}
	/**
	 * method setting initial values for this action
	 * returns the average of the action values.
	 * @param Act
	 * @param base
	 * @return
	 */
	private double InitAcSimple(double[] Act, double base) {
		double result = 0;
		for (int a = 0; a < Act.length; a++) {
			Act[a] = base * Math.random()+1;
			result += Act[a];
		}
		result /= Act.length;
		return result;
	}
	
	
	/**
	 * method setting initial values for this action
	 * returns the average of the action values.
	 * @param Act
	 * @param base
	 * @return
	 */
	private double InitAc(double[] Act, double base) {
		double result = 0;
		for (int a = 0; a < Act.length; a++) {
			Act[a] = stFunction(base * Math.random());
			result += Act[a];
		}
		result /= Act.length;
		return result;
	}
	
	class StrategyParam{
		public StrategyParam(double value[]){
			this.value = value;
		}
		
		/**
		 * @return Returns the strategy.
		 */
		public double getStrategy() {
			return strategy;
		}
		/**
		 * @param strategy The strategy to set.
		 */
		public void setStrategy(double strategy) {
			this.strategy = strategy;
		}
		public double[] getv(){return this.value;}
		
		/**
		 * @return Returns the name.
		 */
		public synchronized String getName() {
			return name;
		}
		/**
		 * @param name The name to set.
		 */
		public synchronized void setName(String name) {
			this.name = name;
		}
		private String name;
		private double value[];
		private double strategy;
	}
	
}