/*
 * Created on Mar 22, 2004
 * 
 */

/**
 * @author martin lukac
 * 
 * this SAX Parser Handler, handles the creation of the robot -- warning: it is hard wired.
 */
package robot;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;

import mainframe.Tools;
import mns.MNS;
import monitors.CommCenter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;


public class CRLRobothandler extends DefaultHandler {

    private MNS mnsCenter;

    /**
     * @param o
     */
    public CRLRobothandler(Object o) {
        if (o instanceof MNS)
            mnsCenter = (MNS) o;
        currentcommands = new Vector[4];

        try {
            RandomAccessFile bir = new RandomAccessFile(
                    "mns/robotCommands.crl", "r");
            String line;
            int count = 0;
            while ((line = bir.readLine()) != null) {
                if (line.length() > 2) {
                    String[] p = Tools.tokenize(line);
                    Vector v = new Vector(p.length - 1);
                    for (int a = 1; a < p.length; a++) {
                        v.add(p[a]);
                    }
                    currentcommands[count] = v;
                    count++;
                }else {
                	Vector v = new Vector(1);
                	currentcommands[count] = v;
                    count++;
                }
            }
            bir.close();
        } catch (Exception e) {
            CommCenter.showError("Instruction generation problem: ");
            e.printStackTrace();
        }
        CommCenter
                .showComm("CRL Robot Handler: Instruction set generated successfully");
    }

    /**
     * @param doc
     */
    public void setDoc(Document doc) {
        this.doc = doc;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
     *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    public void startElement(String namespaceURI, String lname, String qname,
            Attributes attrs) {
        // e = doc.createElement(qname);
        // doc.appendChild(e);
        if (qname.trim().equalsIgnoreCase("crl")) {
            CommCenter.showComm("<:> Robot descriptor opened correctly <:>");
            stageCounter = -1;
        } else if (qname.trim().equalsIgnoreCase("robot")) {
            robotBegin = true;
            stageCounter = robotLevel;
        } else if (qname.trim().equalsIgnoreCase("command") && stageCounter == robotLevel) {
            stageCounter = robotCommandLevel;
            commandBegin = true;
            commandName = qname.trim();
            targetCommandInstrs = "";
        } else if (qname.trim().equalsIgnoreCase("part")) {
            stageCounter = robotPartLevel;
            partBegin = true;
        } else if (qname.trim().equalsIgnoreCase("devices")) {
            devicesBegin = true;
            stageCounter = robotPartDevicesLevel;
        } else if (qname.trim().equalsIgnoreCase("max")) {
            maxBegin = true;
            stageCounter = robotPartDevicesParamsLevel;
        } else if (qname.trim().equalsIgnoreCase("min")) {
            minBegin = true;
            stageCounter = robotPartDevicesParamsLevel;
        } else if (qname.trim().equalsIgnoreCase("accel")) {
            accelBegin = true;
            stageCounter = robotPartDevicesParamsLevel;
        } else if (qname.trim().equalsIgnoreCase("speed")) {
            speedBegin = true;
            stageCounter = robotPartDevicesParamsLevel;
        } else if (qname.trim().equalsIgnoreCase("command")) {
            stageCounter = robotPartCommandLevel;
            commandBegin = true;
            commandName = qname.trim();
            targetCommandInstrs = "";
        } else if (qname.trim().equalsIgnoreCase("targets")
        		|| qname.trim().equalsIgnoreCase("parameters")) {
            stageCounter = robotPartCommandParamLevel;
            trgtInstrBegin = true;
            targetCommandInstrs += ":";
        } else {
            targetCommandInstrs += ":" + qname.trim() + " ";
            trgtInstrBegin = true;
            //stageCounter++;
        }
        if (!lname.equals("")) {
            String tabs = "";
            for (int i = 0; i < n_tab; i++) {
                tabs = tabs + "\t";
            }
            n_tab++;
            // String temp = "\n" + "Robot Part : " + qname+ " ";
            // System.out.print(temp);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public void endElement(String namespaceURI, String lname, String qname) {
        if (qname.equalsIgnoreCase("crl")) {
            // close and rewirte the MNS command file
            try {
                FileWriter bsr = new FileWriter(new File("mns/generatedRobotCommands.crl"));
                for (int a = 0; a < currentcommands.length; a++) {
                    String line = new String();
                    line = a + " ";
                    Vector v = currentcommands[a];
                    for (int b = 0; b < v.size(); b++) {
                        String s = (String)v.get(b);
                        if (s.length() > 1)
                        	line += s + " ";
                    }
                    System.out.println("Constructor writing: "+line);
                    bsr.write(line+"\n");
                }
                bsr.close();
            } catch (Exception e) {
                CommCenter.showError("Instruction generation problem: ");
                e.printStackTrace();
            }

            CommCenter.showComm("<:> Robot descriptor finnished correctly <:>");
            stageCounter = -1;
        } else if (qname.trim().equalsIgnoreCase("robot")) {
            robotBegin = false;
            stageCounter = -1;
        } else if (qname.trim().equalsIgnoreCase("command") && stageCounter == robotCommandLevel) {
            commandBegin = false;
            stageCounter = robotLevel;
            targetCommandInstrs = commandName + ' ' + targetCommandInstrs;
            mnsCenter.addbioCommand(partName, targetCommandInstrs);
        } else if (qname.trim().equalsIgnoreCase("part")) {
            partBegin = false;
            stageCounter = robotLevel;
            // partName = "robot";
        } else if (qname.trim().equalsIgnoreCase("devices")) {
            devicesBegin = false;
            stageCounter = robotPartLevel;
            // mnsCenter.addbioDevice(partName, commandName);
        } else if (qname.trim().equalsIgnoreCase("max")) {
            maxBegin = false;
            stageCounter = robotPartDevicesLevel;
        } else if (qname.trim().equalsIgnoreCase("min")) {
            minBegin = false;
            stageCounter = robotPartDevicesLevel;
        } else if (qname.trim().equalsIgnoreCase("accel")) {
            accelBegin = false;
            stageCounter = robotPartDevicesLevel;
        } else if (qname.trim().equalsIgnoreCase("speed")) {
            speedBegin = false;
            stageCounter = robotPartDevicesLevel;
        } else if (qname.trim().equalsIgnoreCase("command")) {
            commandBegin = false;
            stageCounter = robotPartLevel;
            targetCommandInstrs = commandName + ' ' + targetCommandInstrs;
            mnsCenter.addbioCommand(partName, targetCommandInstrs);
        } else if (qname.trim().equalsIgnoreCase("targets")
        		|| qname.trim().equalsIgnoreCase("parameters")) {
            trgtInstrBegin = false;
            stageCounter = robotPartCommandLevel;
        } else {
            //stageCounter--;
            trgtInstrBegin = false;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ContentHandler#characters(char[], int, int)
     */
    public void characters(char ch[], int start, int length) {
        // get the value of within tag
        String temp = "";
        for (int i = start; i < start + length; i++) {
            temp = temp + ch[i];
        }
        temp = temp.trim();
        if (!temp.equals(""))
            switch (stageCounter) {
            case robotLevel: { //robotLevel
                mnsCenter.setiD(temp);
                // indicates the global level for command generation
                partName = "robot";
                break;
            }
            case robotCommandLevel: {//robotCommandLevel
                checkCommand(temp, 1);
                break;
            }
            case robotPartLevel: {//robotPartLevel
                partName = temp;
                checkCommand(temp, 2);
                mnsCenter.addbioPart(temp);
                break;
            }
            case robotPartDevicesLevel: //robotPart
            // initial device protection
            {
                commandName = temp;
                mnsCenter.addbioDevice(partName, commandName);
                break;
            }
            case robotPartDevicesParamsLevel: {
                if (maxBegin) {
                    targetCommandInstrs = temp.trim();
                    mnsCenter.setPartMax(partName, targetCommandInstrs);
                } else if (minBegin) {
                    targetCommandInstrs = temp.trim();
                    mnsCenter.setPartMin(partName, targetCommandInstrs);
                } else if (accelBegin) {
                    targetCommandInstrs = temp.trim();
                    // mnsCenter.setPartMin(partName,
                    // targetCommandInstrs);
                } else if (speedBegin) {
                    targetCommandInstrs = temp.trim();
                    // mnsCenter.setPartMin(partName,
                    // targetCommandInstrs);
                }
                break;
            }
            case robotPartCommandLevel: {
                commandName = temp.trim();
                checkCommand(temp, 3);
                break;
            }
            case robotPartCommandParamLevel: {
                    targetCommandInstrs += temp.trim();
                break;
            }
            default: {
                CommCenter.showError("element: " + temp.trim() + " not found: "
                        + stageCounter);
                break;
            }
            }
    }

    /**
     * @param node
     * @param pW
     */
    public void writeCRL(Node node, PrintWriter pW) {
        this.pW = pW;
        write(node, "\t");
    }

    public void checkCommand(String command, int level) {
        Vector v = currentcommands[level];
        Iterator i = v.iterator();
        while (i.hasNext()) {
            String s = ((String) i.next()).trim();
            if (s.equalsIgnoreCase(command))
                return;
        }
        v.add(command);
    }

    /**
     * @param node
     * @param indent
     */
    private void write(Node node, String indent) {
        switch (node.getNodeType()) {
        case Node.DOCUMENT_NODE: {
            pW.println(indent + "<?xml version='1.0'?>");
            NodeList list = node.getChildNodes();
            int a = 0;
            while (a < list.getLength()) {
                write((Node) list.item(a), indent);
            }
        }
        // case Node.ELEMENT_NODE
        }
    }

    // writer
    PrintWriter pW;

    int n_tab = 0;

    // init the keywords map
    // Gramm gram = new Gramm();
    private Element e;

    // the discintinuous parsing result is stored here
    private Document doc;

    // the builder
    private DocumentBuilder dB;

    // the counter for building parts of robot
    private int stageCounter = 0;

    // sections defined by tags
    private boolean robotBegin = false;

    private boolean partBegin = false;
    private boolean devicesBegin = false;
    private boolean commandBegin = false;
    private boolean trgtInstrBegin = false;
    private boolean maxBegin = false;
    private boolean minBegin = false;
    private boolean speedBegin = false;
    private boolean accelBegin = false;

    // to construct a robot we need some templates
    // and some milestones -- hardcoded
    private String partName;

    private String deviceName;

    private String commandName;

    private String targetCommandInstrs;

    // /array of vectors containing commands per level of meaning
    private Vector[] currentcommands;
    
    public static final int robotLevel = 0;
    public static final int robotCommandLevel = 1;
    public static final int robotPartLevel = 2;
    public static final int robotPartDevicesLevel = 3;
    public static final int robotPartDevicesParamsLevel = 4;
    public static final int robotPartCommandLevel = 5;
    public static final int robotPartCommandParamLevel = 6;
}