/*
 * Created on Jun 15, 2004
 *
 */
package robot;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import mns.BioDevice;
import mns.Command;
import mns.MNSRobotPart;
import monitors.CommCenter;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import devs.motorpackage.asc16.Servo;
import devs.speechpackage.SpeechGenerator;

/**
 * @author martin lukac
 *
 * constructor of the Robot Tree
 */
public class Constructor{
    
    public Constructor (){
        
    }
    
    public static DefaultMutableTreeNode constructModel(Robot robot){
       Vector mnsrobotparts = Robot.getController().getCnsRobotParts(); 
       rHierarch = new Vector (1,1);
       rHierarch.add(0, new String("Robot"));
       for (int a = 0;a<mnsrobotparts.size(); a++){
           Vector robotpart = new Vector();
           robotpart.add(((MNSRobotPart)mnsrobotparts.elementAt(a)).getName());
           Vector commands = ((MNSRobotPart)mnsrobotparts.elementAt(a)).getCommands();
           for (int b = 0; b< commands.size(); b++){
               robotpart.add(commands.elementAt(b));
               System.out.println(commands.elementAt(b));
           }
           rHierarch.add(robotpart);
       }
       
       return processHierarchy(rHierarch);
    }
    
    private static DefaultMutableTreeNode processHierarchy(Vector hierarchy) {
        DefaultMutableTreeNode node =
          new DefaultMutableTreeNode(hierarchy.elementAt(0));
        DefaultMutableTreeNode child;
        for(int i=1; i<hierarchy.size(); i++) {
          Object nodeSpecifier = hierarchy.elementAt(i);
          if (nodeSpecifier instanceof Vector)  // Ie node with children
            child = processHierarchy((Vector)nodeSpecifier);
          else
            child = new DefaultMutableTreeNode(nodeSpecifier); // Ie Leaf
          node.add(child);
        }
        return(node);
      }
    
    private Robot robot;
    private static Vector rHierarch;
    private static PrintWriter out = null;
    public static void saveXMLModel(Robot robot, File f){
        
        try {
            out = new PrintWriter(new FileWriter(f));
        
		     DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		     dbf.setValidating(false);
		     try{
		         DocumentBuilder parser = dbf.newDocumentBuilder();
		         Document doc = parser.newDocument();
		         Vector mnsrobotparts = Robot.getController().getCnsRobotParts(); 
		         Vector arch = new Vector (1,1);
//                 Element o = doc.createElement("crl");
//		         doc.appendChild(o);

		         //create the robot 
                 Element nd = doc.createElement("robot");
                 doc.appendChild(nd);
                
                 nd.appendChild(doc.createTextNode("robot"));
		         for (int a = 0;a<mnsrobotparts.size(); a++){
		             MNSRobotPart part = ((MNSRobotPart)mnsrobotparts.elementAt(a));
		             
		             //insert the robot part 
	                 Element np = doc.createElement("part");
	                 np.appendChild(doc.createTextNode(part.getname()));
	                 nd.appendChild(np);

	                 //insert device indexes
	                 BioDevice [] devs = ((BioDevice[])part.getDevices());
	                 int lengt = devs.length;
	                 String devices ="";
	                 String min = "";
	                 String max = "";
	                 for (int m = 0; m < lengt; m++){
	                     try{
	                     if (devs[m] instanceof Servo){
	                         int p = ((Servo)devs[m]).getMotorNumber();
	                         devices = devices+" "+p;
	                     	max = max+" "+((Servo)devs[m]).getNormHighPosition();
		                    min = min+" "+((Servo)devs[m]).getNormLowPosition();
	                     } else if (devs[m] instanceof SpeechGenerator){
	                         devices = devices+" "+0;
		                     	max = max+" 100";
		                     	min = min+" 100";
	                     }
	                     }catch(Exception e){
	                         
	                     }
	                 }
	                 Element nm = doc.createElement("devices");
	                 nm.appendChild(doc.createTextNode(devices));
	                 np.appendChild(nm);
	                 
//	                 //insert the device Min/Max
	                 Element nq = doc.createElement("min");
	                 nq.appendChild(doc.createTextNode(min));
	                 nm.appendChild(nq);
	                 nq = doc.createElement("max");
	                 nq.appendChild(doc.createTextNode(max));
	                 nm.appendChild(nq);
	                 
	                 //insert individual commands
		             Vector commands = ((MNSRobotPart)mnsrobotparts.elementAt(a)).getCommands();
		             for (int b = 0; b< commands.size(); b++){

		                 Element nc = doc.createElement("command");
		                 Command c = (Command)commands.get(b);
		                 nc.appendChild(doc.createTextNode(c.getName()));
		                 np.appendChild(nc);
		                 Object [] inst = c.getCommand();
		                 if (inst != null)
		                 for (int k = 0; k < inst.length; k++){
		                     Element nparam = doc.createElement("targets");
			                 nparam.appendChild(doc.createTextNode((String)inst[k]));
			                 nc.appendChild(nparam);
		                 }
		                 //Element nt
		             }

		         }
		         out.println("<?xml version='1.0'?>");
		         out.print("<crl>");
		         printXMLTree(doc.getDocumentElement(),"   ");
		         out.println("\n </crl>");
		         out.close();
		         
		     } catch (ParserConfigurationException e){
		         
		     }
        } catch (IOException e){
            
        }
        CommCenter.showComm("Robot saved to: "+f.getAbsolutePath());

    }
    
    private  static void printXMLTree(Node nl, String indent){
        switch(nl.getNodeType()){
        
	        case Node.DOCUMENT_NODE:
	        Node d = nl.getFirstChild();
	        out.println(indent+"<?xml version='1.0'?>");
	        while(d != null){
	            printXMLTree(d, indent);
	            d = d.getNextSibling();
	        }
	        break;
	        case Node.TEXT_NODE:
		        Node a = nl.getFirstChild();
	        	Text textNode = (Text)nl;
	        	String text = textNode.getData().trim();
	        	if(text != null && text.length() > 0){
			        out.print(text);
	        	}
	        	break;
//		        while(a != null){
//		            printXMLTree(a);
//		            a = a.getNextSibling();
//		        }
//		        break;
	        case Node.ELEMENT_NODE:
		        Element e = (Element)nl;
	        	out.print("\n"+indent+"<"+e.getTagName()+">");
	        	String newindent = "     "+indent;
	        	Node child = e.getFirstChild();
	        	int counter = 0;
		        while(child != null){
		            printXMLTree(child, newindent);
		            child = child.getNextSibling();
		            counter++;
		        }
		        if (counter > 1)
		            out.print("\n"+indent+"</"+e.getTagName()+">");
		        else out.print("</"+e.getTagName()+">");
		        break;
        }
    }
}
