/*
 * Created on May 9, 2004
 */
package robot;

import java.io.File;

import mns.MNS;
import mns.MNSCommandhandler;
import mns.MNSListener;
import mns.MNScommandQueue;
import monitors.MNSRobotPanel;
import nns.BioSensorEvent;
import nns.NNS;
import biocell.BioThread;
import biocell.GlobalState;
import brain.NetClient.RobotClientEvent;
import brain.NetClient.RobotClientListener;
import crlparser.CRLSAXParser;
import crlparser.ParserEvent;

/**
 * @author martin lukac
 * 
 * The object on the Top of the Robot it self, provides ways to constuct the robot
 * and so far also some functions that are to be migrated to the NNS and perhaps to brain
 */
public class Robot extends BioThread implements RobotClientListener {
	/**
	 * 
	 */
	static{
		new GlobalState();
		GlobalState.iniMainMood();
	}
	public Robot() {
		super("robot");

		mainQueue = new MNScommandQueue();
		motorsystem = new MNS(mainQueue);
		sensorsystem = new NNS(mainQueue);
		motorsystem.addCNSListener(sensorsystem);
		sensorsystem.addListener(motorsystem);
	}

	/**
	 * reconstructs the robot according to the describing file
	 * 
	 * @param descriptor -
	 *            robot descriptor
	 */
	public void constructRobot(File descriptor) {
        createdFrom = descriptor.getAbsolutePath();
		//now parse the robot from a CRL descriptor
		new CRLSAXParser().process(descriptor, motorsystem);
	}

	public void saveRobot() {
		//now parse the robot from a CRL descriptor
        String outputfile;
        //check for extension
        try{
            outputfile = createdFrom.substring(0, createdFrom.lastIndexOf('.'));
        } catch (StringIndexOutOfBoundsException e){
            outputfile = createdFrom;
        }
        //check if this file is a previous saved file
        try{
            outputfile = outputfile.substring(outputfile.indexOf("Save"));
        } catch (StringIndexOutOfBoundsException e){
            //outputfile = createdFrom;
        }
        
        outputfile += "Saved.crl";
        Constructor.saveXMLModel(this, new File(outputfile));
	}

	/**
	 * wakes the all connected and tected or defined hardware. This might take a
	 * while depending on owhat kind of HW and SW modules are used
	 */
	public void wake() {
		//genrate all BioDevices needed to build the robot and robotparts
		//motorsystem.initiate();
	}

	/**
	 * set the id for this robot - of course let's usi it when it is not alone
	 * 
	 * @param iD
	 */
	public void setiD(String iD) {
		this.iD = iD;
	}

	/**
	 * @param monitor
	 */
	public void addMNSMonitor(MNSRobotPanel monitor) {
		motorsystem.addMonitor(monitor);
	}

	/**
	 * executes a command file and wiats for start
	 * 
	 * @param script
	 */
	public void execScript(File script) {
		//start the command parser
		commandParser = new CRLSAXParser();
		commandParser.chngHandler(new MNSCommandhandler(motorsystem));
		commandParser.process(script);
	}

	/**
	 * executes a command file
	 * 
	 * @param script -
	 *            CRL script
	 */
	public void execScriptNow(File script) {
		//start the command parser
		commandParser = new CRLSAXParser();
		commandParser.chngHandler(new MNSCommandhandler(motorsystem));
		commandParser.process(script);
		motorsystem.load(new ParserEvent("start"));
	}

	/**
	 * executes a command file
	 * 
	 * @param script -
	 *            CRL script
	 */
	public void execCommand(String command) {
		motorsystem.load(new ParserEvent(command));
	}

	/**
	 *
	 */
	public void stopScript() {
		mainQueue.flush();
	}

	/**
	 * @param listener -
	 *            add a CNSListener
	 */
	public void addCNSListener(MNSListener listener) {
		motorsystem.addCNSListener(listener);
	}

	/**
	 * @param ccEvent - route it to the NNS
	 */
	public void clientCommand(RobotClientEvent ccEvent) {
		if (ccEvent.getmessage().equalsIgnoreCase("file"))
			execScript(ccEvent.getcommand());
		else if (ccEvent.getmessage().toLowerCase().startsWith("broadcast")) {
			//recieved a message from another robot or of a sensor
			//this is a generic non directly related message - a global percept
			String message = ccEvent.getmessage();
			//cuting the header
			message = message.substring(message.indexOf(' '));
			//getting the from user
			String fromuser = message.substring(0, message.indexOf(' '));
			//get the actual message
			message = message.substring(message.trim().indexOf(' ')+1);
			sensorsystem.sensorPercieved(new BioSensorEvent("net client",fromuser+" "+message));
		} else if (ccEvent.getmessage().toLowerCase().startsWith("inform")) {
			//recieved a message from another robot or of a sensor
			//here the idea is to show a volontary response to the robot.
			String message = ccEvent.getmessage();
			//cuting the header
			message = message.substring(message.indexOf(' '));
			//getting the from user
			String fromuser = message.substring(0, message.indexOf(' '));
			//get the actual message
			message = message.substring(message.trim().indexOf(' ')+1);
			sensorsystem.sensorPercieved(new BioSensorEvent("net client",fromuser+" "+message));
		} else {
			//command from the server -- exploit for the future
			String command = ccEvent.getmessage();
			motorsystem.setExecSignal(true);
			System.out.println("Executing command: " + command);
			execCommand(command);
		}
	}

	/**
	 * @return
	 */
	public static MNS getController() {
		return motorsystem;
	}

	/**
	 * @return
	 */
	public static NNS getSensor() {
		return sensorsystem;
	}

	/**
	 * 
	 *
	 */
	public void startClient() {
		//		this.client = new RobotClient(mns);
	}

	/* (non-Javadoc)
	 * @see devs.sensorpackage.BioSensorListener#sensorPercieved(devs.sensorpackage.BioSensorEvent)
	 */
	public void sensorPercieved(BioSensorEvent event) {
		// TODO Auto-generated method stub
	}

	public static void main(String[] args) {
		new Robot();
	}

	
	/**
	 * @return Returns the mainQueue.
	 */
	public static synchronized MNScommandQueue getMainQueue() {
		return mainQueue;
	}
	/**
	 * @param mainQueue The mainQueue to set.
	 */
	public static synchronized void setMainQueue(MNScommandQueue mainQueue) {
		Robot.mainQueue = mainQueue;
	}
	/**
	 * @return Returns the motorsystem.
	 */
	public static synchronized MNS getMotorsystem() {
		return motorsystem;
	}
	/**
	 * @param motorsystem The motorsystem to set.
	 */
	public static synchronized void setMotorsystem(MNS motorsystem) {
		Robot.motorsystem = motorsystem;
	}
	/**
	 * @return Returns the sensorsystem.
	 */
	public static synchronized NNS getSensorsystem() {
		return sensorsystem;
	}
	/**
	 * @param sensorsystem The sensorsystem to set.
	 */
	public static synchronized void setSensorsystem(NNS sensorsystem) {
		Robot.sensorsystem = sensorsystem;
	}
	/**
	 * @return Returns the commandParser.
	 */
	public synchronized CRLSAXParser getCommandParser() {
		return commandParser;
	}
	/**
	 * @param commandParser The commandParser to set.
	 */
	public synchronized void setCommandParser(CRLSAXParser commandParser) {
		this.commandParser = commandParser;
	}
	public static final String construct_rules = "rbtsrc/rules.dts";

	//the individual parser of every robot to parse commands
	private CRLSAXParser commandParser;

	public static MNS motorsystem = null;

	public static NNS sensorsystem = null;

	public static MNScommandQueue mainQueue = null;

    private String createdFrom ="";
    
	protected String iD;

}